require 'rspec'
require 'watir-webdriver'

require_relative '../../lib/browser_container/site'
require_relative '../../lib/module/DateUtility'
require_relative '../../lib/module/MongoUtility'
require_relative '../rspec_helper'


describe '[Story #CREHAB-418: Calendar Date Picker]', :regression => true do
  include DateUtility
  include MongoUtility

  app_title = "VA FitHeart"

  before(:all) do
    @driver = Watir::Browser.new
    @site = Site.new(BASE_URL, @driver)

    @login = @site.loginPage
    @eula = @site.eula
    @roa = @site.roa
    @landing = @site.landing
    @leftNav = @site.leftMenu
    @modal = @site.modal
    @goal = @site.goal
    @error = @site.error
    @fitness_listview = @site.fitness_listview
    @fitness_activity = @site.fitness_activity
    @calendar = @site.calendar_picker
    @date_filter = @site.date_filter

    @today = getDateNthDaysAgo(0, "%m/%d/%Y")
    @threeMonthAgo = getDateNthMonthsFromNow(-3, "%m/%d/%Y")
    @yesterday = getDateNthDaysFromNow(-1,"%m/%d/%Y")

    acceptEulaAndQuickLogin("zztest.patient01", "pass", "D123401")

    @landing.appTitle_element.when_visible

    expect(@landing.appTitle).to eq(app_title)
    @landing.welcomeTitleForAuthenticatedUser_element.when_visible
    expect(@landing.welcomeTitleForAuthenticatedUser).to eq("Welcome to VA FitHeart")

    @leftNav.leftMenu_element.when_visible
    @leftNav.leftMenu

    @leftNav.fitness_activity_element.when_visible
    @leftNav.fitness_activity

    @fitness_listview.primaryHeader_element.when_visible
    expect(@fitness_listview.primaryHeader_element.text).to eq("Fitness Activity")

  end

  after(:all) do
    @site.quitDriver
  end

  context "AC: Implement Calendar Date Picker for Date Fields" do
    it ". A calendar picker control is available for date entry fields " do
      @date_filter.startDatePicker_element.when_visible
      @date_filter.endDatePicker_element.when_visible
    end

    it ". When a user selects the calendar picker, a calendar view appears. " do
      #End Date calendar
      @date_filter.endDatePicker_element.click

      expect(@date_filter.isEndDateCalendarDisplayed).to eq(true)
      expect(@date_filter.isStartDateCalendarDisplayed).to eq(false)

    end

    it ". When the calendar picker is selected, the calendar opens inside the active screen or modal, keeping the originating screen in view." do
      #manual testing
    end

    it ". Users can advance the calendar picker one month at a time within the available date range constraints." do
      #selected date is today
      expect(getMonthOfYear(@today) +  " " + getYear(@today)).to eq(@calendar.getTitleToggleButtonText)

      nextMonthAndYear = getNthMonthFromGivenDate(@today, 1)

      @calendar.clickNextButton
      Watir::Wait.until {nextMonthAndYear == @calendar.getTitleToggleButtonText}
      expect(@calendar.getTitleToggleButtonText).to eq(nextMonthAndYear)

      nextMonthAndYear = getNthMonthFromGivenDate(@today, 2)
      @calendar.clickNextButton
      Watir::Wait.until {nextMonthAndYear == @calendar.getTitleToggleButtonText}
      expect(@calendar.getTitleToggleButtonText).to eq(nextMonthAndYear)

      nextMonthAndYear = getNthMonthFromGivenDate(@today, 3)
      @calendar.clickNextButton
      Watir::Wait.until {nextMonthAndYear == @calendar.getTitleToggleButtonText}
      expect(@calendar.getTitleToggleButtonText).to eq(nextMonthAndYear)

      nextMonthAndYear = getNthMonthFromGivenDate(@today, 4)
      @calendar.clickNextButton
      Watir::Wait.until {nextMonthAndYear == @calendar.getTitleToggleButtonText}
      expect(@calendar.getTitleToggleButtonText).to eq(nextMonthAndYear)

      previousMonthAndYear = getNthMonthFromGivenDate(@today, 3)
      @calendar.clickPreviousButton
      Watir::Wait.until {previousMonthAndYear == @calendar.getTitleToggleButtonText}
      expect(@calendar.getTitleToggleButtonText).to eq(previousMonthAndYear)

      previousMonthAndYear = getNthMonthFromGivenDate(@today, 2)
      @calendar.clickPreviousButton
      Watir::Wait.until {previousMonthAndYear == @calendar.getTitleToggleButtonText}
      expect(@calendar.getTitleToggleButtonText).to eq(previousMonthAndYear)

      previousMonthAndYear = getNthMonthFromGivenDate(@today, 1)
      @calendar.clickPreviousButton
      Watir::Wait.until {previousMonthAndYear == @calendar.getTitleToggleButtonText}
      expect(@calendar.getTitleToggleButtonText).to eq(previousMonthAndYear)

      previousMonthAndYear = getNthMonthFromGivenDate(@today, 0)
      @calendar.clickPreviousButton
      Watir::Wait.until {previousMonthAndYear == @calendar.getTitleToggleButtonText}
      expect(@calendar.getTitleToggleButtonText).to eq(previousMonthAndYear)

    end

    it ". Users can advance the calendar picker one year at a time within the available date range constraints." do
      #click Header button

      @calendar.clickTitleToggleButton
      year = @calendar.getTitleToggleButtonText
      Watir::Wait.until {getYear(@today) == year}
      expect(@calendar.getTitleToggleButtonText).to eq(getYear(@today))

      @calendar.clickNextButton
      Watir::Wait.until {@calendar.getTitleToggleButtonText == getNThYearFromGivenDate(@today, 1)}
      expect(@calendar.getTitleToggleButtonText).to eq(getNThYearFromGivenDate(@today, 1))

      @calendar.clickNextButton
      Watir::Wait.until {@calendar.getTitleToggleButtonText == getNThYearFromGivenDate(@today, 2)}
      expect(@calendar.getTitleToggleButtonText).to eq(getNThYearFromGivenDate(@today, 2))

      @calendar.clickNextButton
      Watir::Wait.until {@calendar.getTitleToggleButtonText == getNThYearFromGivenDate(@today, 3)}
      expect(@calendar.getTitleToggleButtonText).to eq(getNThYearFromGivenDate(@today, 3))

      @calendar.clickPreviousButton
      Watir::Wait.until {@calendar.getTitleToggleButtonText == getNThYearFromGivenDate(@today, 2)}
      expect(@calendar.getTitleToggleButtonText).to eq(getNThYearFromGivenDate(@today, 2))

      @calendar.clickPreviousButton
      Watir::Wait.until {@calendar.getTitleToggleButtonText == getNThYearFromGivenDate(@today, 1)}
      expect(@calendar.getTitleToggleButtonText).to eq(getNThYearFromGivenDate(@today, 1))

      @calendar.clickPreviousButton
      Watir::Wait.until {@calendar.getTitleToggleButtonText == getNThYearFromGivenDate(@today, 0)}
      expect(@calendar.getTitleToggleButtonText).to eq(getNThYearFromGivenDate(@today, 0))

      expect(@calendar.areAllMonthsDisplayed).to eq(true)

    end

    it ".  Users can select a day on the calendar." do
      #move to calendar view
      month = getMonthOfYear(@today)
      @calendar.pickMonth(month)
      Watir::Wait.until {getMonthOfYear(@today) +  " " + getYear(@today) == @calendar.getTitleToggleButtonText}
      expect(getMonthOfYear(@today) +  " " + getYear(@today)).to eq(@calendar.getTitleToggleButtonText)

      #fitness activity date filter range is between -6 years and today

      twentyDaysAgo = getDateNthDaysFromNow(-20, "%m/%d/%Y")

      puts "Twenty days ago is "  + twentyDaysAgo.to_s

      if (getMonthOfYear(@today) != getMonthOfYear(twentyDaysAgo) )
        previousMonthAndYear = getNthMonthFromGivenDate(@today, -1)
        @calendar.clickPreviousButton
        Watir::Wait.until {previousMonthAndYear == @calendar.getTitleToggleButtonText}
        expect(@calendar.getTitleToggleButtonText).to eq(previousMonthAndYear)
      end

      dayStr = getDay(twentyDaysAgo)
      puts "Day String: " + dayStr.to_s

      @calendar.pickDay(dayStr)

      Watir::Wait.until {@date_filter.endDate == twentyDaysAgo}
      expect(@date_filter.endDate).to eq(twentyDaysAgo)

      @date_filter.startDatePicker_element.click
      expect(@date_filter.isStartDateCalendarDisplayed).to eq(true)
      @calendar.pick("05/23/2014")
      Watir::Wait.until {@date_filter.startDate == "05/23/2014"}
      expect(@date_filter.startDate).to eq("05/23/2014")

      @date_filter.endDatePicker_element.click
      expect(@date_filter.isEndDateCalendarDisplayed).to eq(true)
      @calendar.pick("01/22/2011")
      Watir::Wait.until {@date_filter.endDate == "01/22/2011"}
      expect(@date_filter.endDate).to eq("01/22/2011")

    end

    it ". Date range constraints can be applied to the calendar picker so that excluded date values are not selectable." do
      #fitness activity filter - no future date allowed
      @date_filter.reset_element.click

      Watir::Wait.until {@date_filter.startDate == @threeMonthAgo}
      expect(@date_filter.startDate).to eq(@threeMonthAgo)
      expect(@date_filter.endDate).to eq(@today)

      tomorrow = getDateNthDaysFromNow(1, "%m/%d/%Y")
      @date_filter.startDatePicker_element.click
      expect(@date_filter.isStartDateCalendarDisplayed).to eq(true)
      @calendar.pick(tomorrow)

      @calendar.pickDay(tomorrow)
      expect(@calendar.isDayEnabled(getDay(tomorrow))).to eq(false)

      threeYearsLater = getDateNthYearsFromNow(3, "%m/%d/%Y")
      @calendar.pick(threeYearsLater)

      expect(@calendar.isMonthOrYearEnabled(getMonthOfYear(threeYearsLater))).to eq(false)

      @site.refreshBrowser

      Watir::Wait.until {@date_filter.startDate == @threeMonthAgo}
      expect(@date_filter.startDate).to eq(@threeMonthAgo)
      expect(@date_filter.endDate).to eq(@today)

      fourYearsAgo = getDateNthYearsFromNow(-4, "%m/%d/%Y")
      #puts "Four years ago " + fourYearsAgo.to_s

      @date_filter.startDatePicker_element.click
      expect(@date_filter.isStartDateCalendarDisplayed).to eq(true)
      @calendar.pick(fourYearsAgo)

      Watir::Wait.until {@date_filter.startDate == fourYearsAgo}
      expect(@date_filter.startDate).to eq(fourYearsAgo)

      sixYearsAgo = getDateNthYearsFromNow(-6, "%m/%d/%Y")
      @date_filter.startDatePicker_element.click
      expect(@date_filter.isStartDateCalendarDisplayed).to eq(true)
      @calendar.pick(sixYearsAgo)

      Watir::Wait.until {@date_filter.startDate == sixYearsAgo}
      expect(@date_filter.startDate).to eq(sixYearsAgo)

      @date_filter.startDatePicker_element.click
      @calendar.flipToYear(getYear(sixYearsAgo))
      @calendar.pickMonth(getMonthOfYear(sixYearsAgo))
      oneDayBefore = getDay(sixYearsAgo).to_i  - 1
      #puts "Select Day "  +  oneDayBefore.to_s
      @calendar.pickDay(oneDayBefore)

      expect(@calendar.isDayEnabled(oneDayBefore)).to eq(false)
    end

    it ". The calendar picker can be constrained to allow only year selections." do
      @site.refreshBrowser

      Watir::Wait.until {@date_filter.startDate == @threeMonthAgo}
      expect(@date_filter.startDate).to eq(@threeMonthAgo)
      expect(@date_filter.endDate).to eq(@today)

      @date_filter.endDatePicker_element.click
      expect(@date_filter.isEndDateCalendarDisplayed).to eq(true)

      year = getYear(@today)
      @calendar.clickTitleToggleButton
      expect(year.to_s).to eq(@calendar.getTitleToggleButtonText)

      @calendar.clickTitleToggleButton
      expect(@calendar.getTitleToggleButtonText).to eq("2001 - 2020")

      sixYearsAgo = getDateNthYearsFromNow(-6, "%m/%d/%Y")

      expect(@calendar.isMonthOrYearEnabled(getYear(sixYearsAgo).to_i - 2)).to eq(false)
      expect(@calendar.isMonthOrYearEnabled(getYear(sixYearsAgo).to_i - 1)).to eq(false)
      expect(@calendar.isMonthOrYearEnabled(getYear(sixYearsAgo))).to eq(true)
      expect(@calendar.isMonthOrYearEnabled(getYear(sixYearsAgo).to_i + 1)).to eq(true)
      expect(@calendar.isMonthOrYearEnabled(getYear(sixYearsAgo).to_i + 2)).to eq(true)
      expect(@calendar.isMonthOrYearEnabled(getYear(sixYearsAgo).to_i + 3)).to eq(true)
      expect(@calendar.isMonthOrYearEnabled(getYear(sixYearsAgo).to_i + 4)).to eq(true)
      expect(@calendar.isMonthOrYearEnabled(getYear(sixYearsAgo).to_i + 5)).to eq(true)
      expect(@calendar.isMonthOrYearEnabled(getYear(sixYearsAgo).to_i + 6)).to eq(true)

      expect(@calendar.isMonthOrYearEnabled(getYear(sixYearsAgo).to_i + 7)).to eq(false)
      expect(@calendar.isMonthOrYearEnabled(getYear(sixYearsAgo).to_i + 8)).to eq(false)

    end

    it ". The calendar picker can be constrained to allow only month and year selections." do
      @site.refreshBrowser

      Watir::Wait.until {@date_filter.startDate == @threeMonthAgo}
      expect(@date_filter.startDate).to eq(@threeMonthAgo)
      expect(@date_filter.endDate).to eq(@today)

      @date_filter.endDatePicker_element.click
      expect(@date_filter.isEndDateCalendarDisplayed).to eq(true)

      year = getYear(@today)
      @calendar.clickTitleToggleButton
      expect(year.to_s).to eq(@calendar.getTitleToggleButtonText)

      expect(@calendar.isMonthOrYearEnabled(getMonthOfYear(@today))).to eq(true)

      index = CalendarPicker::MONTHS.index(getMonthOfYear(@today))

      for i in (index + 1)..(11) do
        expect(@calendar.isMonthOrYearEnabled(CalendarPicker::MONTHS[i])).to eq(false)
      end

    end

    it ". Text box entries and calendar picker entries are kept in synch" do
      @site.refreshBrowser

      Watir::Wait.until {@date_filter.startDate == @threeMonthAgo}
      expect(@date_filter.startDate).to eq(@threeMonthAgo)
      expect(@date_filter.endDate).to eq(@today)

      @date_filter.endDatePicker_element.click
      expect(@date_filter.isEndDateCalendarDisplayed).to eq(true)

      expect(@calendar.getcurrentYear()).to eq(getYear(@today))
      expect(@calendar.getCurrentMonth).to eq(getMonthOfYear(@today))
      expect(@calendar.getCurrentDay()).to eq(getDay(@today))

      threeDaysAgo = getDateNthDaysFromNow(-3, "%m/%d/%Y")
      @calendar.pick(threeDaysAgo)

      Watir::Wait.until {@date_filter.endDate == threeDaysAgo}
      expect(@date_filter.endDate).to eq(threeDaysAgo)

      @date_filter.endDatePicker_element.click
      expect(@date_filter.isEndDateCalendarDisplayed).to eq(true)

      expect(@calendar.getcurrentYear()).to eq(getYear(threeDaysAgo))
      expect(@calendar.getCurrentMonth).to eq(getMonthOfYear(threeDaysAgo))
      expect(@calendar.getCurrentDay()).to eq(getDay(threeDaysAgo))

      fiveDaysAgo = getDateNthDaysFromNow(-5, "%m/%d/%Y")
      @calendar.pick(fiveDaysAgo)

      Watir::Wait.until {@date_filter.endDate == fiveDaysAgo}
      expect(@date_filter.endDate).to eq(fiveDaysAgo)

    end

    it ". This calendar control is not optimized for screen readers. Please directly enter dates in the text box provided. Open calendar control." do
      expect(@date_filter.startDatePicker_element.text).to eq("This calendar control is not optimized for screen readers. Please directly enter dates in the text box provided. Open Calendar Control.")
      expect(@date_filter.endDatePicker_element.text).to eq("This calendar control is not optimized for screen readers. Please directly enter dates in the text box provided. Open Calendar Control.")
    end

    it ". To ensure an equivalent experience, the day associated with the selected date shall be made available to voice over users as a text element shown below the date text entry field." do

      expect(@date_filter.dayOfWeekForStartDate_element.text).to eq(getDayofWeek(@threeMonthAgo))
      fiveDaysAgo = getDateNthDaysFromNow(-5, "%m/%d/%Y")
      expect(@date_filter.dayOfWeekForEndDate_element.text).to eq(getDayofWeek(fiveDaysAgo))
      puts "Day of the week " + @date_filter.dayOfWeekForEndDate_element.text
    end

  end

end
