#!/bin/env ruby
# encoding: utf-8
require 'rspec'
require 'watir-webdriver'

require_relative '../../lib/browser_container/site'
require_relative '../../lib/module/DateUtility'
require_relative '../../lib/module/MongoUtility'
require_relative '../rspec_helper'


describe '[Story #CREHAB-515: Accessible Option for Activity Graphs]
          [Story #CREHAB-513: Phone User - Fitness Activity Filter Page: weekly Activity Goals progress graphs]', :acceptance => true do
  include DateUtility
  include MongoUtility

  app_title = "VA FitHeart"

  before(:all) do
    removeCollection("fitness", "healthfactordb")
    removeCollection("fitnessgoals", "healthfactordb")

    @driver = Watir::Browser.new
    @site = Site.new(BASE_URL, @driver)

    @login = @site.loginPage
    @eula = @site.eula
    @roa = @site.roa
    @landing = @site.landing
    @leftNav = @site.leftMenu
    @modal = @site.modal
    @goal = @site.goal
    @error = @site.error
    @carousel = @site.carousel
    @fitness_listview = @site.fitness_listview
    @fitness_activity = @site.fitness_activity
    @date_filter = @site.date_filter

    @today = getDateNthDaysAgo(0, "%m/%d/%Y")
  end

  after(:all) do
    @site.quitDriver
  end

  context "[Before] Log in and navigate to" do
    it ". Login and navigate" do
      acceptEulaAndQuickLogin("zztest.patient01", "pass", "D123401")

      @landing.appTitle_element.when_visible

      expect(@landing.appTitle).to eq(app_title)
      @landing.welcomeTitleForAuthenticatedUser_element.when_visible
      expect(@landing.welcomeTitleForAuthenticatedUser).to eq("Welcome to VA FitHeart")
    end
  end

  context "#AC CREHAB-643|CREHAB-677:  Screen reader alt text info for Minutes Graph
           #AC CREHAB-645|CREHAB-677:  Screen reader alt text info for Steps Graph
           #AC CREHAB-646|CREHAB-677:  Screen reader alt text info for Miles Graph" do
    it ". Minutes, Steps, Miles Graphs - No goal has been set" do
     #landing screen
      @landing.graphSection_element.when_visible

      Watir::Wait.until {@landing.minutesAltText_element.text == "You have not set a minutes goal. Select the Weekly Activity Goals button to set a minutes goal."}

      expect(@landing.minutesAltText_element.text).to eq("You have not set a minutes goal. Select the Weekly Activity Goals button to set a minutes goal.")
      expect(@landing.stepsAltText_element.text).to eq("You have not set a steps goal. Select the Weekly Activity Goals button to set a steps goal.")
      expect(@landing.milesAltText_element.text).to eq("You have not set a miles goal. Select the Weekly Activity Goals button to set a miles goal.")

      #Fitness activity list  screen
      @leftNav.leftMenu_element.when_visible
      @leftNav.leftMenu

      @leftNav.fitness_activity_element.when_visible
      @leftNav.fitness_activity

      @fitness_listview.primaryHeader_element.when_visible
      expect(@fitness_listview.primaryHeader_element.text).to eq("Fitness Activity")

      expect(@landing.minutesAltText_element.text).to eq("You have not set a minutes goal. Select the Weekly Activity Goals button to set a minutes goal.")
      expect(@landing.stepsAltText_element.text).to eq("You have not set a steps goal. Select the Weekly Activity Goals button to set a steps goal.")
      expect(@landing.milesAltText_element.text).to eq("You have not set a miles goal. Select the Weekly Activity Goals button to set a miles goal.")

    end

    it ". Minutes, Steps, Miles Graph - a goal has been set" do

      @leftNav.leftMenu_element.when_visible
      @leftNav.leftMenu

      @leftNav.home_element.when_visible
      @leftNav.home

      @landing.appTitle_element.when_visible

      expect(@landing.appTitle).to eq(app_title)
      @landing.welcomeTitleForAuthenticatedUser_element.when_visible
      expect(@landing.welcomeTitleForAuthenticatedUser).to eq("Welcome to VA FitHeart")

      resetCollection("fitnessgoals", "healthfactordb", "weekly_goals")
      #resetCollection("fitness", "healthfactordb", "fitness_activity")
      restoreCollection("fitness", "healthfactordb",  "spec/snapshots/fitness/fitness.bson")

      #update goals db
      startDate = Time.now - 3.days
      endDate = Time.now + 3.days
      updateDocument("fitnessgoals", "healthfactordb", "54b53a6203644ae0cd231bc5", "startDate", startDate)
      updateDocument("fitnessgoals", "healthfactordb", "54b53a6203644ae0cd231bc5", "endDate", endDate)

      #update fitness activity db

      threeDaysAgo = Time.now - 3.days
      twoDaysAgo = Time.now - 2.days
      updateDocument("fitness", "healthfactordb", "54b5a1d003644ae0cd231bf3", "activityDate", threeDaysAgo)
      updateDocument("fitness", "healthfactordb", "54b5a1d003644ae0cd231bf3", "createdDate", threeDaysAgo)
      updateDocument("fitness", "healthfactordb", "54b5a1ed03644ae0cd231bf4", "activityDate", twoDaysAgo)
      updateDocument("fitness", "healthfactordb", "54b5a1ed03644ae0cd231bf4", "createdDate", twoDaysAgo)

      @site.refreshBrowser

      #landing screen

      @landing.welcomeTitleForAuthenticatedUser_element.when_visible
      expect(@landing.welcomeTitleForAuthenticatedUser).to eq("Welcome to VA FitHeart")

      expect(@landing.minutesAltText_element.text).to eq("You have completed 164 minutes toward your goal of 234 minutes.")
      expect(@landing.stepsAltText_element.text).to eq("You have completed 8,013 steps toward your goal of 345 steps.")
      expect(@landing.milesAltText_element.text).to eq("You have completed 17.0 miles toward your goal of 456 miles.")

      #Fitness activity list  screen
      @leftNav.leftMenu_element.when_visible
      @leftNav.leftMenu

      @leftNav.fitness_activity_element.when_visible
      @leftNav.fitness_activity

      @fitness_listview.primaryHeader_element.when_visible
      expect(@fitness_listview.primaryHeader_element.text).to eq("Fitness Activity")

      expect(@landing.minutesAltText_element.text).to eq("You have completed 164 minutes toward your goal of 234 minutes.")
      expect(@landing.stepsAltText_element.text).to eq("You have completed 8,013 steps toward your goal of 345 steps.")
      expect(@landing.milesAltText_element.text).to eq("You have completed 17.0 miles toward your goal of 456 miles.")

    end

  end

  context "#AC CREHAB-1182|TC CREHAB-1194: Phone View: VA FitHeart Fitness Activity Weekly Activity Progress" do
    it ". When a phone user opens the VA FitHeart Fitness Activity page, the three Weekly Activity Progress graphs are shown one at a time" do
      @site.resizeWindowToPhone
      expect(@carousel.carousel_element.visible?).to eq(true)
      expect(@carousel.leftArrow_element.visible?).to eq(true)
      expect(@carousel.rightArrow_element.visible?).to eq(true)
    end
  end

  context "#AC CREHAB-1183|TC CREHAB-1194: First Image: Minute graph" do
    it ". First Graph: Minutes graph - pagination dot, alt text: Image 1 of e" do
      expect(@carousel.getPaginationAltText()).to eq("Image 1 of 3")
      expect(@carousel.getActiveGraphDataLabel()).to include("164\nminutes")
      expect(@carousel.leftArrow_element.attribute("title")).to eq("Previous Image")
      expect(@carousel.rightArrow_element.attribute("title")).to eq("Next Image")
    end
  end

  context "#AC CREHAB-1184|TC CREHAB-1194: Second image: Steps Graph " do
    it ". Second Graph: Steps Graph - pagination alt text: Image 2 of 3" do
      @carousel.rightArrow
      Watir::Wait.until { @carousel.getActiveGraphDataLabel().include?("8,013\nsteps")}
      expect(@carousel.getPaginationAltText()).to eq("Image 2 of 3")
      expect(@carousel.getActiveGraphDataLabel()).to include("8,013\nsteps")
      expect(@carousel.leftArrow_element.attribute("title")).to eq("Previous Image")
      expect(@carousel.rightArrow_element.attribute("title")).to eq("Next Image")
    end
  end

  context "#AC CREAHB-1185|TC CREHAB-1194:Third image: Miles Graph" do
    it ". Third Graph: Miles Graph - pagination alt text: Image 3 of 3" do
      @carousel.rightArrow
      Watir::Wait.until { @carousel.getActiveGraphDataLabel().include?("17.0\nmiles")}
      expect(@carousel.getPaginationAltText()).to eq("Image 3 of 3")
      expect(@carousel.getActiveGraphDataLabel()).to include("17.0\nmiles")
      expect(@carousel.leftArrow_element.attribute("title")).to eq("Previous Image")
      expect(@carousel.rightArrow_element.attribute("title")).to eq("Next Image")
    end
  end

end
