require 'rspec'
require 'watir-webdriver'

require_relative '../../lib/browser_container/site'
require_relative '../../lib/module/DateUtility'
require_relative '../../lib/module/MongoUtility'
require_relative '../rspec_helper'


describe '[Story #CREHAB-264: Fitness Activity - Add, Fields validation]
          [Story #CREHAB-853: Add Duplicate Entries Error Message for Fitness Activity]
          [Story #CREHAB-702: Enhancing: Rating intensity modal updates]', :acceptance => true do
  include DateUtility
  include MongoUtility

  app_title = "VA FitHeart"

  before(:all) do
    removeCollection("fitness", "healthfactordb")
    removeCollection("fitnessgoals", "healthfactordb")
    removeCollection("notifications", "notificationsdb")

    @driver = Watir::Browser.new
    @site = Site.new(BASE_URL, @driver)

    @login = @site.loginPage
    @eula = @site.eula
    @roa = @site.roa
    @landing = @site.landing
    @leftNav = @site.leftMenu
    @modal = @site.modal
    @goal = @site.goal
    @error = @site.error
    @fitness_listview = @site.fitness_listview
    @fitness_activity = @site.fitness_activity

    @today = getDateNthDaysAgo(0, "%m/%d/%Y")
    @yesterday = getDateNthDaysFromNow(-1,"%m/%d/%Y")

    @EXPECTED_ROW_0_FORM = [\
                          "* required field",
                          "* Date:",
                          "MM/DD/YYYY",
                          "This calendar control is not optimized for screen readers. Please directly enter dates in the text box provided. Open Calendar Control.",
                          "Day selected is " + getDayofWeek(@today),
                          getDayofWeek(@today),
                          "* Time:",
                          "hh:mm AM/PM",
                          "This time selection control is not optimized for screen readers. Please directly enter the time in the text box provided. Open Time Picker",
                          "* Amount (one entry is required to save)",
                          "Duration (minutes):",
                          "Steps:",
                          "Distance (miles):",
                          "Activity:",
                          "Select","Walking", "Running", "Cycling", "Swimming", "Dancing", "Tennis", "Hiking", "Gym", "Housework", "Gardening", "Golf", "Yoga", "Rowing", "Stairs", "Sports", "General Calisthenics", "Other",
                          "Intensity:", "Select", "Very Easy", "Easy", "Somewhat Easy", "Somewhat Hard", "Hard", "Very Hard",
                          "Note:",
                          "250 character maximum",
                          "250 characters left",
                          "Notification:",
                          "Status: Off",
                          "",
                          "To edit the Fitness Activity notification, select the Notification button on the previous screen.",
                          "Save Cancel"

                           ]
  end

  after(:all) do
    @site.quitDriver
  end

  context "[Before] Log in and navigate to" do
    it ". Login and navigate to" do
      acceptEulaAndQuickLogin("zztest.patient01", "pass", "D123401")

      @landing.appTitle_element.when_visible

      expect(@landing.appTitle).to eq(app_title)
      @landing.welcomeTitleForAuthenticatedUser_element.when_visible
      expect(@landing.welcomeTitleForAuthenticatedUser).to eq("Welcome to VA FitHeart")

      @leftNav.leftMenu_element.when_visible
      @leftNav.leftMenu

      @leftNav.title_element.when_visible
      expect(@leftNav.title_element.text).to eq("App Options\nClose App Options Menu")
      @leftNav.fitness_activity_element.when_visible
      @leftNav.fitness_activity

      @fitness_listview.primaryHeader_element.when_visible
      expect(@fitness_listview.primaryHeader_element.text).to eq("Fitness Activity")
    end
  end

  context "AC #CREHAB-487,CREHAB-1202|TC #CREHAB-518,CREHAB-1221: Add a New Entry for Fitness Activity"  do
    it ". Add button (+ icon, alt text Add) is available in detail screen" do

      @fitness_listview.add_element.when_visible
      expect(@fitness_listview.add_element.attribute("title")).to eq("Add")
      @fitness_listview.add

      @fitness_activity.primaryHeader_element.when_visible
      expect(@fitness_activity.primaryHeader_element.text).to eq("Add Fitness Activity Entry")

    end

    it ". Unless otherwise indicated, all values default to null (or select for drop lists), no guidance text (light gray placeholder text) is shown, and all fields are optional. * = required is displayed at the top of the screen " do
      fitnessActivityForm = @fitness_activity.getScreenContent().split("\n")

      @EXPECTED_ROW_0_FORM.each_with_index do | expectedScreenRow, i |
        expect(fitnessActivityForm[i]).to eq(expectedScreenRow)
      end
    end

    it ". Date (required, calendar picker, MM/DD/YYYY, show guidance text, cannot be future date - constrain entry to prevent future date, default to today)" do
      expect(@fitness_activity.date_element.attribute("required")).to eq("true")
      expect(@fitness_activity.date_element.attribute("placeholder")).to eq("MM/DD/YYYY")
      expect(@fitness_activity.date).to eq(@today.to_s)
      tomorrow = getDateNthDaysFromNow(1, "%m/%d/%Y")
      @fitness_activity.date = tomorrow.to_s
      @fitness_activity.save_element.when_visible
      @fitness_activity.save

      sixYearsAgo = getDateNthYearsFromNow(-6, "%m/%d/%Y")
      errorMessages = ["Date must be between " + sixYearsAgo + " and " + @today.to_s + "."]
      verifyErrorMessage(errorMessages)

      @fitness_activity.date = ""
      @fitness_activity.save

      errorMessages = ["Date field is required."]
      verifyErrorMessage(errorMessages)

    end

    it ". Time (required, time picker, format: HH:MM AM/PM, default to current system time)" do
      expect(@fitness_activity.time_element.attribute("required")).to eq("true")
      expect(@fitness_activity.time_element.attribute("placeholder")).to eq("HH:MM AM/PM")
      expect(@fitness_activity.time).not_to eq("")

      @fitness_activity.time = ""
      @fitness_activity.save

      errorMessages = ["Time field is required."]
      verifyErrorMessage(errorMessages)

      @fitness_activity.date = @today
      @fitness_activity.time = getNthMinFromNow(5)
      @fitness_activity.save

      errorMessages = ["Time cannot be a future time."]
      verifyErrorMessage(errorMessages)

      @fitness_activity.cancel
      @fitness_activity.cancel_element.when_not_visible

      @fitness_listview.primaryHeader_element.when_visible
      expect(@fitness_listview.primaryHeader_element.text).to eq("Fitness Activity")

    end

    it ". Amount (one entry is required to save) (primary label, show as required (asterisk) include parenthetical comment next to the label in instructional text font)" do
      @fitness_listview.add_element.when_visible
      @fitness_listview.add

      @fitness_activity.primaryHeader_element.when_visible
      expect(@fitness_activity.primaryHeader_element.text).to eq("Add Fitness Activity Entry")

      @fitness_activity.duration = ""
      @fitness_activity.steps = ""
      @fitness_activity.distance = ""

      @fitness_activity.save

      errorMessages = ["At least one amount entry is required to save."]
      verifyErrorMessage(errorMessages)

      @fitness_activity.date = @yesterday.to_s
      @fitness_activity.time = "08:39 AM"
      @fitness_activity.duration = "20"
      @fitness_activity.steps = ""
      @fitness_activity.distance = ""

      @fitness_activity.save

      if (@modal.continue2_element.exists?)
        @modal.continue2
        @modal.continue2_element.when_not_visible
      end

      @fitness_listview.primaryHeader_element.when_visible
      expect(@fitness_listview.primaryHeader_element.text).to eq("Fitness Activity")

      @fitness_listview.tableList_elements[0].click

      @fitness_activity.duration = ""
      @fitness_activity.steps = ""
      @fitness_activity.distance = "5"

      @fitness_activity.save
      @fitness_activity.save_element.when_not_visible

      @fitness_listview.primaryHeader_element.when_visible
      expect(@fitness_listview.primaryHeader_element.text).to eq("Fitness Activity")

      @fitness_listview.tableList_elements[0].click

      @fitness_activity.duration = "1"
      @fitness_activity.steps = "20"
      @fitness_activity.distance = "5"

      @fitness_activity.save
      @fitness_activity.save_element.when_not_visible

      @fitness_listview.primaryHeader_element.when_visible
      expect(@fitness_listview.primaryHeader_element.text).to eq("Fitness Activity")

    end

    it ". Duration (minutes) (show as a sub-label, numeric data type, constrain entry to whole numbers between 0 and 10080, do not show guidance text" do
      @fitness_listview.add

      @fitness_activity.primaryHeader_element.when_visible
      expect(@fitness_activity.primaryHeader_element.text).to eq("Add Fitness Activity Entry")

      @fitness_activity.duration = "abcdefg"
      expect(@fitness_activity.duration).to eq("")
      @fitness_activity.duration = "-1"
      expect(@fitness_activity.duration).to eq("1")
      @fitness_activity.duration = "10089"
      @fitness_activity.save

      errorMessages = ["Duration (minutes) is outside the expected range. Please enter a value between 0 and 10080."]
      verifyErrorMessage(errorMessages)

      expect(@fitness_activity.duration_element.attribute("placeholder")).to eq("")
    end

    it ". Steps (numeric entry) (show as a sub-label, numeric data type, constrain entry to whole numbers between 0 and 50000, do not show guidance text" do
      @fitness_activity.steps = "ABCDEF"
      expect(@fitness_activity.steps).to eq("")
      @fitness_activity.steps = "-234"
      expect(@fitness_activity.steps).to eq("234")
      @fitness_activity.steps = "59999"
      @fitness_activity.save

      errorMessages = ["Steps is outside the expected range. Please enter a value between 0 and 50000."]
      verifyErrorMessage(errorMessages)

      expect(@fitness_activity.steps_element.attribute("placeholder")).to eq("")
    end

    it ". Distance (miles) (show as a sub-label, numeric data type, constrain entry to whole numbers between 0 and 50000, do not show guidance text " do
      @fitness_activity.distance = "ABCDEFG1234"
      expect(@fitness_activity.distance).to eq("1234")
      @fitness_activity.distance = "-1111"
      expect(@fitness_activity.distance).to eq("1111")
      @fitness_activity.distance = "58899"

      @fitness_activity.save

      errorMessages = ["Distance (miles) is outside the expected range. Please enter a value between 0.1 and 50000."]
      verifyErrorMessage(errorMessages)

      @fitness_activity.steps = "45023"
      @fitness_activity.duration = "600"
      @fitness_activity.distance = "334.21"
      @fitness_activity.date = getDateNthDaysFromNow(-15, "%m/%d/%Y")

      @fitness_activity.save

      if (@modal.continue2_element.exists?)
        @modal.continue2
        @modal.continue2_element.when_not_visible
      end

      @fitness_listview.primaryHeader_element.when_visible
      expect(@fitness_listview.primaryHeader_element.text).to eq("Fitness Activity")

      Watir::Wait.until {@fitness_listview.tableList_elements.length == 2}
      expect(@fitness_listview.tableList_elements.length).to eq(2)
      @fitness_listview.tableList_elements[0].click

      @fitness_activity.primaryHeader_element.when_visible
      expect(@fitness_activity.primaryHeader_element.text).to eq("Edit Fitness Activity Entry")

      expect(@fitness_activity.distance).to eq("5.0")

    end

    it ". Rule: At least one amount (duration, steps or miles) must be valued on save, otherwise display error message: At least one amount entry is required to save." do
      @fitness_activity.duration = ""
      @fitness_activity.steps = ""
      @fitness_activity.distance = ""

      @fitness_activity.save

      errorMessages = ["At least one amount entry is required to save."]
      verifyErrorMessage(errorMessages)

      @fitness_activity.duration = "90"
      @fitness_activity.save

      @fitness_activity.save_element.when_not_visible

      @fitness_listview.primaryHeader_element.when_visible
      expect(@fitness_listview.primaryHeader_element.text).to eq("Fitness Activity")

      @fitness_listview.add_element.when_visible
      @fitness_listview.add

      @fitness_activity.primaryHeader_element.when_visible
      expect(@fitness_activity.primaryHeader_element.text).to eq("Add Fitness Activity Entry")

      @fitness_activity.steps = "4000"

      @fitness_activity.save

      if (@modal.continue2_element.exists?)
        @modal.continue2
        @modal.continue2_element.when_not_visible
      end

      @fitness_listview.primaryHeader_element.when_visible
      expect(@fitness_listview.primaryHeader_element.text).to eq("Fitness Activity")

      @fitness_listview.add_element.when_visible
      @fitness_listview.add

      @fitness_activity.primaryHeader_element.when_visible
      expect(@fitness_activity.primaryHeader_element.text).to eq("Add Fitness Activity Entry")

      @fitness_activity.date = getDateNthDaysFromNow(-10, "%m/%d/%Y")
      @fitness_activity.distance = "5"

      @fitness_activity.save

      if (@modal.continue2_element.exists?)
        @modal.continue2
        @modal.continue2_element.when_not_visible
      end

      @fitness_listview.contentLoaded_element.when_present(timeout=15)

      @fitness_listview.primaryHeader_element.when_visible
      expect(@fitness_listview.primaryHeader_element.text).to eq("Fitness Activity")
    end

    it ". Activity (single select, drop down list, default to Select, which is equivalent to Null); " do
      @fitness_listview.add_element.when_visible
      @fitness_listview.add

      @fitness_activity.primaryHeader_element.when_visible
      expect(@fitness_activity.primaryHeader_element.text).to eq("Add Fitness Activity Entry")

      expect(@fitness_activity.activitySelect).to eq("Select")
     activitySelectArr = ["Select", "Walking", "Running", "Cycling", "Swimming", "Dancing", "Tennis", "Hiking", "Gym", "Housework", "Gardening", "Golf", "Yoga", "Rowing", "Stairs", "Sports", "General Calisthenics", "Other"]
     expect(@fitness_activity.activitySelect_options).to eq(activitySelectArr)

      for i in 0..(activitySelectArr.length - 1)
        @fitness_activity.activitySelect = activitySelectArr[i]
        expect(@fitness_activity.activitySelect).to eq(activitySelectArr[i])
      end

    end

    it ". Intensity (single select, dropdown, default to Select)" do
      expect(@fitness_activity.intensitySelect).to eq("Select")
      intensitySelectArr = ["Select", "Very Easy", "Easy", "Somewhat Easy", "Somewhat Hard", "Hard", "Very Hard"]
      expect(@fitness_activity.intensitySelect_options).to eq(intensitySelectArr)

      for i in 0..(intensitySelectArr.length - 1)
        @fitness_activity.intensitySelect = intensitySelectArr[i]
        expect(@fitness_activity.intensitySelect).to eq(intensitySelectArr[i])
      end
    end

    it ". Display information button, when clicked a modal is displayed, OK button" do
      @fitness_activity.intensityRating_element.when_visible
      @fitness_activity.intensityRating

      @modal.modalTitle_element.when_visible
      expect(@modal.modalTitle_element.text).to eq("Intensity Rating")

      expect(@modal.raitingIntensityList_elements[0].text).to eq("Very Easy\nAnything other than sleeping")
      expect(@modal.raitingIntensityList_elements[1].text).to eq("Easy\nBreathing not noticeable, easy to carry a full conversation")
      expect(@modal.raitingIntensityList_elements[2].text).to eq("Somewhat Easy\nBreathing noticeable, can hold a short conversation")
      expect(@modal.raitingIntensityList_elements[3].text).to eq("Somewhat Hard\nShort of breath (becoming uncomfortable), can speak a sentence")
      expect(@modal.raitingIntensityList_elements[4].text).to eq("Hard\nDifficult to maintain intensity, can speak a few words")
      expect(@modal.raitingIntensityList_elements[5].text).to eq("Very Hard\nAlmost impossible to maintain, unable to talk")

      @modal.ok2_element.when_visible
      expect(@modal.ok2_element.text).to eq("OK")
      @modal.ok2
      @modal.ok2_element.when_not_visible

    end

    it ". Note (alphanumeric, textbox, 250 characters, display character countdown)" do
      notes_250 = "AAA4567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890"
      notes_260 = notes_250 + "ABCDEFGHIJ"
      expect(@fitness_activity.characterCounter_element.text).to eq("250 characters left")

      @fitness_activity.notes  = notes_250
      expect(@fitness_activity.notes).to eq(notes_250)
      expect(@fitness_activity.characterCounter_element.text).to eq("0 characters left")
      @fitness_activity.notes = notes_260
      expect(@fitness_activity.notes).to eq(notes_250)
      expect(@fitness_activity.characterCounter_element.text).to eq("0 characters left")

      @fitness_activity.cancel
      @fitness_activity.cancel_element.when_not_visible
    end

    it ".  Add Fitness Activity" do
      count = @fitness_listview.tableList_elements.length

      date = getDateNthDaysFromNow(-2, "%m/%d/%Y")
      time =  "10:30 AM"
      duration = 60
      steps = 4000
      distance = 3.5
      activity = "Walking"
      intensity = "Hard"
      note = ""
      addFitnessActivity(date, time, duration, steps, distance, activity, intensity, note)

      @fitness_listview.tableList_elements[0].when_visible
      expect(@fitness_listview.tableList_elements.length).to eq(count + 1)

      count = @fitness_listview.tableList_elements.length

      sixDaysAgo = getDateNthDaysFromNow(-6, "%m/%d/%Y")
      date = sixDaysAgo.to_s
      time =  "11:30 AM"
      duration = 100
      steps = ""
      distance = ""
      activity = "Running"
      intensity = "Very Hard"
      note = "Ran almost every day"
      addFitnessActivity(date, time, duration, steps, distance, activity, intensity, note)

      @fitness_listview.tableList_elements[0].when_visible
      expect(@fitness_listview.tableList_elements.length).to eq(count + 1)

    end

  end

  context "#AC CREHAB-1073|TC CREHAB-1076: Display error message for duplicate entires" do
    it ". Duplicate entries validation" do
      @fitness_listview.add_element.when_visible
      @fitness_listview.add

      @fitness_activity.primaryHeader_element.when_visible
      expect(@fitness_activity.primaryHeader_element.text).to eq("Add Fitness Activity Entry")
      @fitness_activity.date = getDateNthDaysFromNow(-6, "%m/%d/%Y")
      @fitness_activity.time = "11:30 AM"
      @fitness_activity.duration = 100
      @fitness_activity.steps = ""
      @fitness_activity.distance = ""
      @fitness_activity.activitySelect = "Running"
      @fitness_activity.intensitySelect = "Very Hard"
      @fitness_activity.notes = ""

      @fitness_activity.save

      errorMessages = ["An entry has already been saved with the same date and time. Please change your date and/or time and save again."]
      verifyErrorMessage(errorMessages)

    end
  end

  def addFitnessActivity(date, time, duration, steps, distance, activity, intensity, notes)
    @fitness_listview.add

    @fitness_activity.primaryHeader_element.when_visible
    expect(@fitness_activity.primaryHeader_element.text).to eq("Add Fitness Activity Entry")
    @fitness_activity.date = date
    @fitness_activity.time = time
    @fitness_activity.duration = duration
    @fitness_activity.steps = steps
    @fitness_activity.distance = distance
    @fitness_activity.activitySelect = activity
    @fitness_activity.intensitySelect = intensity
    @fitness_activity.notes = notes

    @fitness_activity.save

    if (@modal.continue2_element.exists?)
      @modal.continue2
      @modal.continue2_element.when_not_visible
    end

    @fitness_listview.primaryHeader_element.when_visible
    expect(@fitness_listview.primaryHeader_element.text).to eq("Fitness Activity")

  end

end
