require 'rspec'
require 'watir-webdriver'

require_relative '../../lib/browser_container/site'
require_relative '../../lib/module/DateUtility'
require_relative '../../lib/module/MongoUtility'
require_relative '../rspec_helper'


describe '[Story #CREHAB-264: Fitness Activity - Edit/Delete]', :acceptance do
  include DateUtility
  include MongoUtility

  app_title = "VA FitHeart"

  before(:all) do
    removeCollection("fitness", "healthfactordb")
    removeCollection("fitnessgoals", "healthfactordb")

    @driver = Watir::Browser.new
    @site = Site.new(BASE_URL, @driver)

    @login = @site.loginPage
    @eula = @site.eula
    @roa = @site.roa
    @landing = @site.landing
    @leftNav = @site.leftMenu
    @modal = @site.modal
    @goal = @site.goal
    @error = @site.error
    @fitness_listview = @site.fitness_listview
    @fitness_activity = @site.fitness_activity
    @learn_more = @site.learnMore

    @today = getDateNthDaysAgo(0, "%m/%d/%Y")
    @yesterday = getDateNthDaysFromNow(-1,"%m/%d/%Y")
  end

  after(:all) do
    @site.quitDriver
  end

  context "[Before] Log in and navigate to" do
    it ". Login and navigate to" do
      acceptEulaAndQuickLogin("zztest.patient01", "pass", "D123401")

      @landing.appTitle_element.when_visible

      expect(@landing.appTitle).to eq(app_title)
      @landing.welcomeTitleForAuthenticatedUser_element.when_visible
      expect(@landing.welcomeTitleForAuthenticatedUser).to eq("Welcome to VA FitHeart")

      @leftNav.leftMenu_element.when_visible
      @leftNav.leftMenu

      @leftNav.title_element.when_visible
      expect(@leftNav.title_element.text).to eq("App Options\nClose App Options Menu")
      @leftNav.fitness_activity_element.when_visible
      @leftNav.fitness_activity

      @fitness_listview.primaryHeader_element.when_visible
      expect(@fitness_listview.primaryHeader_element.text).to eq("Fitness Activity")

    end
  end


  context "AC #CREHAB-488|TC #CREHAB-518: Edit an Entry for Fitness Activity"  do
    it "When the user selects a Fitness Activity entry from the table view, the system displays the Fitness Activity record in the Detail screen in Edit mode:" do

      count = @fitness_listview.tableList_elements.length

      date = @yesterday.to_s
      time =  "12:30 PM"
      duration = "90"
      steps = "5000"
      distance = "12345.1"
      activity = "Walking"
      intensity = "Easy"
      notes = "N/A"
      addFitnessActivity(date, time, duration, steps, distance, activity, intensity, notes)

      @fitness_listview.tableList_elements[0].when_visible
      expect(@fitness_listview.tableList_elements.length).to eq(count + 1)

      distance = "12345.1"
      expect(@fitness_listview.getTextForRowCol(1, 1)).to eq(date + " " + time)
      expect(@fitness_listview.getTextForRowCol(1, 2)).to eq(activity)
      expect(@fitness_listview.getTextForRowCol(1, 3)).to eq(duration)
      expect(@fitness_listview.getTextForRowCol(1, 4)).to eq(steps)
      expect(@fitness_listview.getTextForRowCol(1, 5)).to eq(distance)
      expect(@fitness_listview.getTextForRowCol(1, 6)).to eq(intensity)

      @fitness_listview.tableList_elements[0].click

      @fitness_activity.primaryHeader_element.when_visible
      expect(@fitness_activity.primaryHeader_element.text).to eq("Edit Fitness Activity Entry")

      date = @yesterday.to_s
      time =  "12:30 PM"
      duration = "90"
      steps = "5000"
      distance = "12345.1"
      activity = "Walking"
      intensity = "Easy"
      notes = "N/A"
      verifyFitnessActivity(date, time, duration, steps, distance, activity, intensity, notes)

    end

    it ".  All values previously saved are visible. If no value was previously saved for a field, the display defaults to null (or select for drop lists); no guidance text (light gray placeholder text) is shown unless and all fields are optional unless otherwise indicated." do

      date =  getDateNthDaysFromNow(-3, "%m/%d/%Y")
      time =  "04:30 PM"
      duration = ""
      steps = "3500"
      distance = ""
      activity = "Select"
      intensity = "Select"
      notes = ""

      @fitness_activity.date = date
      @fitness_activity.time = time
      @fitness_activity.duration = duration
      @fitness_activity.steps = steps
      @fitness_activity.distance = distance
      @fitness_activity.activitySelect = activity
      @fitness_activity.intensitySelect = intensity
      @fitness_activity.notes =  notes

      @fitness_activity.save
      @fitness_activity.save_element.when_not_visible

      @fitness_listview.primaryHeader_element.when_visible
      expect(@fitness_listview.primaryHeader_element.text).to eq("Fitness Activity")

      expect(@fitness_listview.tableList_elements.length).to eq(1)
      @fitness_listview.tableList_elements[0].click

      @fitness_activity.primaryHeader_element.when_visible
      expect(@fitness_activity.primaryHeader_element.text).to eq("Edit Fitness Activity Entry")

      verifyFitnessActivity(date, time, duration, steps, distance, activity, intensity, notes)
    end

    it ". Standard Edit functions are available (Save, Cancel, Delete)" do
      expect(@fitness_activity.save_element.exists?).to eq(true)
      expect(@fitness_activity.save_element.text).to eq("Save")
      expect(@fitness_activity.cancel_element.exists?).to eq(true)
      expect(@fitness_activity.cancel_element.text).to eq("Cancel")
      expect(@fitness_activity.delete_element.exists?).to eq(true)
      expect(@fitness_activity.delete_element.text).to eq("Delete")

      @fitness_activity.cancel
      expect(@fitness_listview.primaryHeader_element.text).to eq("Fitness Activity")
      @fitness_listview.tableList_elements[0].when_visible

    end

  end

  context "AC #CREHAB-489|TC #CREHAB-518:  Save, Cancel, Delete Functionality" do
    it ". Save - When a user selects Save the system saves the record and the user is returned to the table view, with the focus on the header. Updates made to the record are visible in the table, if included in that view." do
      date =  getDateNthDaysFromNow(-3, "%m/%d/%Y")
      time =  "04:30 PM"
      duration = ""
      steps = "3500"
      distance = ""
      activity = "Select"
      intensity = "Select"
      notes = ""

      #todo  :focus isn't working properly
      expect(@fitness_listview.getFocusedElement() == @fitness_listview.primaryHeader_element).to eq(true)

      expect(@fitness_listview.getTextForRowCol(1, 1)).to eq(date + " " + time)
      expect(@fitness_listview.getTextForRowCol(1, 2)).to eq("")
      expect(@fitness_listview.getTextForRowCol(1, 3)).to eq("")
      expect(@fitness_listview.getTextForRowCol(1, 4)).to eq(steps)
      expect(@fitness_listview.getTextForRowCol(1, 5)).to eq("")
      expect(@fitness_listview.getTextForRowCol(1, 6)).to eq("")
    end

    it ". Cancel - When a user selects Cancel, no updates are made to the record and the user is shown the table view with the focus placed on the header" do
      @fitness_listview.tableList_elements[0].click
      @fitness_activity.primaryHeader_element.when_visible
      expect(@fitness_activity.primaryHeader_element.text).to eq("Edit Fitness Activity Entry")

      @fitness_activity.steps = "3333"
      @fitness_activity.activitySelect = "Running"
      @fitness_activity.intensitySelect = "Very Hard"

      @fitness_activity.cancel
      @fitness_activity.cancel_element.when_not_visible

      expect(@fitness_listview.getTextForRowCol(1, 1)).to eq(getDateNthDaysFromNow(-3, "%m/%d/%Y").to_s + " 04:30 PM")
      expect(@fitness_listview.getTextForRowCol(1, 2)).to eq("")
      expect(@fitness_listview.getTextForRowCol(1, 3)).to eq("")
      expect(@fitness_listview.getTextForRowCol(1, 4)).to eq("3500")
      expect(@fitness_listview.getTextForRowCol(1, 5)).to eq("")
      expect(@fitness_listview.getTextForRowCol(1, 6)).to eq("")

      expect(@fitness_listview.tableList_elements.length).to eq(1)
    end

    it ". Delete - When a user selects Delete, the system prompts the user to confirm the Delete by displaying a modal:" do
      date = getDateNthDaysFromNow(-3, "%m/%d/%Y").to_s
      time =  "06:30 PM"
      duration = 30
      steps = 2000
      distance = 2
      activity = "Walking"
      intensity = "Easy"
      note = "Nothing to add"
      addFitnessActivity(date, time, duration, steps, distance, activity, intensity, note)

      date = getDateNthDaysFromNow(-5, "%m/%d/%Y").to_s
      time =  "06:30 PM"
      duration = 60
      steps = ""
      distance = ""
      activity = "Housework"
      intensity = "Somewhat Hard"
      note = "N/A"
      addFitnessActivity(date, time, duration, steps, distance, activity, intensity, note)

      date = getDateNthDaysFromNow(-2, "%m/%d/%Y").to_s
      time =  "05:30 PM"
      duration = "90"
      steps = ""
      distance = "3.5"
      activity = "Running"
      intensity = "Somewhat Hard"
      note = "N/A"
      addFitnessActivity(date, time, duration, steps, distance, activity, intensity, note)

      count = @fitness_listview.tableList_elements.length
      for i in 0..(count -1)
        @fitness_listview.tableList_elements[0].click

        @fitness_activity.primaryHeader_element.when_visible
        expect(@fitness_activity.primaryHeader_element.text).to eq("Edit Fitness Activity Entry")

        @fitness_activity.delete_element.when_visible
        @fitness_activity.delete

        @modal.modalTitle_element.when_visible
        expect(@modal.modalTitle_element.text).to eq("Confirmation")
        expect(@modal.section1_element.text).to eq("Are you sure you want to remove this entry? Select YES to remove the entry or NO to return to the entry screen.")
        @modal.no_element.when_visible
        expect(@modal.no_element.text).to eq("No")
        @modal.no
        @modal.no_element.when_not_visible

        @fitness_activity.delete_element.when_visible
        expect(@fitness_activity.delete_element.focused?).to eq(true)
        @fitness_activity.delete

        @modal.modalTitle_element.when_visible
        expect(@modal.modalTitle_element.text).to eq("Confirmation")
        expect(@modal.section1_element.text).to eq("Are you sure you want to remove this entry? Select YES to remove the entry or NO to return to the entry screen.")
        @modal.yes_element.when_visible
        expect(@modal.yes_element.text).to eq("Yes")
        @modal.yes
        @modal.yes_element.when_not_visible

        @fitness_listview.primaryHeader_element.when_visible
        expect(@fitness_listview.primaryHeader_element.text).to eq("Fitness Activity")
        Watir::Wait.until {@fitness_listview.tableList_elements.length ==  (count - (i + 1))}

      end

    end

  end

  context "AC #CREHAB-490|TC #CREHAB-518: Navigation for Fitness Activity" do
    it ". On all form factors when the user views the Add or Edit Detail screen (secondary detail page), a < is shown (alt text: Return to Previous Screen; when clicked the user is returned to the Detail Screen for the group where the filter and table list view is presented." do

      @fitness_listview.add_element.when_visible
      @fitness_listview.add

      date = getDateNthDaysFromNow(-5, "%m/%d/%Y").to_s
      time =  "06:30 PM"
      duration = 60
      steps = ""
      distance = ""
      activity = "Housework"
      intensity = "Somewhat Hard"
      note = "N/A"

      @fitness_activity.primaryHeader_element.when_visible
      expect(@fitness_activity.primaryHeader_element.text).to eq("Add Fitness Activity Entry")
      @fitness_activity.date = date
      @fitness_activity.time = time
      @fitness_activity.duration = duration
      @fitness_activity.steps = steps
      @fitness_activity.distance = distance
      @fitness_activity.activitySelect = activity
      @fitness_activity.intensitySelect = intensity
      @fitness_activity.notes = note

      expect(@fitness_activity.backButtonIcon_element.exists?).to eq(true)
      expect(@fitness_activity.backButton_element.attribute("title")).to eq("Return to Previous Screen")

    end

    it ". If a user selects the < symbol and there are unsaved changes on the screen, the system will prompt the user to save their changes via a modal:" do
      @fitness_activity.backButton

      @modal.modalTitle_element.when_visible
      expect(@modal.modalTitle_element.text).to eq("Confirmation")
      expect(@modal.section1_element.text).to eq("You have not saved your changes. If you wish to continue without saving select CONTINUE, otherwise select RETURN to return to the entry screen.")
      @modal.return_element.when_visible
      expect(@modal.return_element.text).to eq("Return")
      @modal.return
      @modal.return_element.when_not_visible
      expect(@fitness_activity.getFocusedElement == @fitness_activity.primaryHeader_element).to eq(true)

      @fitness_activity.backButton_element.when_visible
      @fitness_activity.backButton

      @modal.modalTitle_element.when_visible
      expect(@modal.modalTitle_element.text).to eq("Confirmation")
      expect(@modal.section1_element.text).to eq("You have not saved your changes. If you wish to continue without saving select CONTINUE, otherwise select RETURN to return to the entry screen.")
      @modal.continue_element.when_visible
      expect(@modal.continue_element.text).to eq("Continue")
      @modal.continue
      @modal.continue_element.when_not_visible

      @fitness_listview.primaryHeader_element.when_visible
      expect(@fitness_listview.primaryHeader_element.text).to eq("Fitness Activity")

      @fitness_listview.noResultsMessage_element.when_visible
      expect(@fitness_listview.noResultsMessage_element.text).to eq("No results were found with the current filters. Use the Add button above to create a new record.")

    end

    it ". If the user tries to Navigate away from a detail record where a field has been updated without Saving, the user is prompted to confirm that they want to leave the record without saving via a modal:" do
      date = getDateNthDaysFromNow(-3, "%m/%d/%Y").to_s
      time =  "04:30 PM"
      duration = "30"
      steps = "2000"
      distance = "2"
      activity = "Walking"
      intensity = "Easy"
      note = "Nothing to add"
      addFitnessActivity(date, time, duration, steps, distance, activity, intensity, note)

      @fitness_listview.tableList_elements[0].when_visible
      @fitness_listview.tableList_elements[0].click

      @fitness_activity.primaryHeader_element.when_visible
      expect(@fitness_activity.primaryHeader_element.text).to eq("Edit Fitness Activity Entry")

      @fitness_activity.duration = "60"
      @fitness_activity.distance = "3"

      @leftNav.leftMenu_element.when_visible
      @leftNav.leftMenu

      @leftNav.home_element.when_visible
      @leftNav.home

      @modal.modalTitle_element.when_visible
      expect(@modal.modalTitle_element.text).to eq("Confirmation")
      expect(@modal.section1_element.text).to eq("You have not saved your changes. If you wish to continue without saving select CONTINUE, otherwise select RETURN to return to the entry screen.")
      @modal.return_element.when_visible
      expect(@modal.return_element.text).to eq("Return")
      @modal.return
      @modal.return_element.when_not_visible
      expect(@fitness_activity.getFocusedElement() ==@fitness_activity.primaryHeader_element).to eq(true)

      @leftNav.leftMenu_element.when_visible
      @leftNav.leftMenu

      @leftNav.learnMore_element.when_visible
      @leftNav.learnMore

      @modal.modalTitle_element.when_visible
      expect(@modal.modalTitle_element.text).to eq("Confirmation")
      expect(@modal.section1_element.text).to eq("You have not saved your changes. If you wish to continue without saving select CONTINUE, otherwise select RETURN to return to the entry screen.")
      @modal.continue_element.when_visible
      expect(@modal.continue_element.text).to eq("Continue")
      @modal.continue
      @modal.continue_element.when_not_visible

      @learn_more.primaryHeader_element.when_visible
      expect(@learn_more.primaryHeader_element.text).to eq("Learn More")

      @leftNav.leftMenu_element.when_visible
      @leftNav.leftMenu

      @leftNav.fitness_activity_element.when_visible
      @leftNav.fitness_activity

      @fitness_listview.primaryHeader_element.when_visible
      expect(@fitness_listview.primaryHeader_element.text).to eq("Fitness Activity")

      @fitness_listview.tableList_elements[0].when_visible
      @fitness_listview.tableList_elements[0].click

      @fitness_activity.primaryHeader_element.when_visible
      expect(@fitness_activity.primaryHeader_element.text).to eq("Edit Fitness Activity Entry")

      distance = "2.0"
      verifyFitnessActivity(date, time, duration, steps, distance, activity, intensity, note)

    end
  end


  def addFitnessActivity(date, time, duration, steps, distance, activity, intensity, notes)
    count = @fitness_listview.tableList_elements.length

    @fitness_listview.add_element.when_visible
    @fitness_listview.add

    @fitness_activity.primaryHeader_element.when_visible
    expect(@fitness_activity.primaryHeader_element.text).to eq("Add Fitness Activity Entry")
    @fitness_activity.date = date
    @fitness_activity.time = time
    @fitness_activity.duration = duration
    @fitness_activity.steps = steps
    @fitness_activity.distance = distance
    @fitness_activity.activitySelect = activity
    @fitness_activity.intensitySelect = intensity
    @fitness_activity.notes = notes

    @fitness_activity.save

    if (@modal.continue2_element.exists?)
      @modal.continue2
      @modal.continue2_element.when_not_visible
    end

    @fitness_listview.contentLoaded_element.when_present(timeout=10)

    Watir::Wait.until { @fitness_listview.tableList_elements.length == count + 1 }

    @fitness_listview.primaryHeader_element.when_visible
    expect(@fitness_listview.primaryHeader_element.text).to eq("Fitness Activity")

  end

  def verifyFitnessActivity(date, time, duration, steps, distance, activity, intensity, notes)
    expect(@fitness_activity.date).to eq(date)
    expect(@fitness_activity.time).to eq(time)
    expect(@fitness_activity.duration).to eq(duration)
    expect(@fitness_activity.steps).to eq(steps)
    expect(@fitness_activity.distance).to eq(distance)
    expect(@fitness_activity.activitySelect).to eq(activity)
    expect(@fitness_activity.intensitySelect).to eq(intensity)
    expect(@fitness_activity.notes).to eq(notes)
  end

end
