#!/bin/env ruby
# encoding: utf-8
require 'rspec'
require 'watir-webdriver'

require_relative '../../lib/browser_container/site'
require_relative '../../lib/module/DateUtility'
require_relative '../../lib/module/MongoUtility'
require_relative '../rspec_helper'


describe '[Story #CREHAB-263: Fitness Activity - Filter and List View]
          [Story #CREHAB-517: Fitness Activity - List/Filter Screen - Graph]
          [Story #CREHAB-514: Phone Use: Health Filter and List View Screens - Options Control]', :acceptance => true do
  include DateUtility
  include MongoUtility

  app_title = "VA FitHeart"

  before(:all) do
    resetCollection("fitnessgoals", "healthfactordb", "weekly_goals")
    resetCollection("fitness", "healthfactordb", "fitness_activity_list")

    @driver = Watir::Browser.new
    @site = Site.new(BASE_URL, @driver)

    @login = @site.loginPage
    @eula = @site.eula
    @roa = @site.roa
    @landing = @site.landing
    @leftNav = @site.leftMenu
    @modal = @site.modal
    @goal = @site.goal
    @error = @site.error
    @fitness_listview = @site.fitness_listview
    @fitness_activity = @site.fitness_activity
    @notification_detail = @site.notification_detail
    @date_filter = @site.date_filter

    @today = getDateNthDaysAgo(0, "%m/%d/%Y")
    @threeMonthAgo = getDateNthMonthsFromNow(-3, "%m/%d/%Y")

  end

  after(:all) do
    @site.quitDriver
  end

  context "[Before] Log in and navigate to" do
    it ". Login and navigate to" do
      acceptEulaAndQuickLogin("zztest.patient01", "pass", "D123401")

      @landing.appTitle_element.when_visible

      expect(@landing.appTitle).to eq(app_title)
      @landing.welcomeTitleForAuthenticatedUser_element.when_visible
      expect(@landing.welcomeTitleForAuthenticatedUser).to eq("Welcome to VA FitHeart")
    end
  end

  context "AC #CREHAB-482|TC #CREHAB-511 : Sub Feature Selection and Default View for Fitness Activity"  do
    it ". 	When a user opens the Fitness Activity feature from the Left Panel menu, the system displays the Fitness Activity detail screen the Detail Column (column 3) (no column 2 content sub-feature selects exist for this feature):" do
      @leftNav.leftMenu_element.when_visible
      @leftNav.leftMenu

      @leftNav.title_element.when_visible
      expect(@leftNav.title_element.text).to eq("App Options\nClose App Options Menu")
      @leftNav.fitness_activity_element.when_visible
      @leftNav.fitness_activity

      @fitness_listview.primaryHeader_element.when_visible
      expect(@fitness_listview.primaryHeader_element.text).to eq("Fitness Activity")

      #Graph section included
      @landing.graphSection_element.when_visible
      expect(@landing.graphSection_element.exists?).to eq(true)

    end

    it ". Weekly Activity Goals section" do
        #No Goals
        removeCollection("fitnessgoals", "healthfactordb")
        @site.refreshBrowser

        @fitness_listview.primaryHeader_element.when_visible
        expect(@fitness_listview.primaryHeader_element.text).to eq("Fitness Activity")

        expect(@fitness_listview.goalTitle).to eq("Weekly Activity Goals")

        #Graph section included
        expect(@fitness_listview.graphSection_element.exists?).to eq(true)

        #modal error validation
        minutes = "500"
        steps = "50000"
        miles = "30"
        startDate = "01/0"

        @fitness_listview.weeklyActivityGoals_element.when_visible
        @fitness_listview.weeklyActivityGoals
        @goal.modalTitle_element.when_visible
        expect(@goal.modalTitle_element.text).to eq("Set Weekly Activity Goals")

        @goal.minutes = minutes
        @goal.steps = steps
        @goal.miles = miles
        @goal.startDate = startDate

        @goal.save

        startDate = @today.to_s

        errorMessages = ["Start Date must be formatted MM/DD/YYYY."]
        verifyErrorMessage(errorMessages)

        @goal.cancel
        @goal.cancel_element.when_not_visible

        #goal status validation
        addWeeklyGoal(minutes, steps, miles, startDate)

        @fitness_listview.primaryHeader_element.when_visible
        expect(@fitness_listview.primaryHeader_element.text).to eq("Fitness Activity")

        @sixDaysLater = getDateNthDaysFromNow(6, "%m/%d/%Y")

        expect(@fitness_listview.goalStatus).to eq("Status: " + @today.to_s + " - " + @sixDaysLater.to_s + " (7 days remaining)")

        resetCollection("fitnessgoals", "healthfactordb", "weekly_goals")

        startDate = Time.now - 3.days
        endDate = Time.now + 3.days

        updateDocument("fitnessgoals", "healthfactordb", "54b53a6203644ae0cd231bc5", "startDate", startDate)
        updateDocument("fitnessgoals", "healthfactordb", "54b53a6203644ae0cd231bc5", "endDate", endDate)

        @site.refreshBrowser

        threeDaysAgo = getDateNthDaysFromNow(-3,"%m/%d/%Y")
        threeDaysLater = getDateNthDaysFromNow(3,"%m/%d/%Y")

        Watir::Wait.until { @fitness_listview.goalStatus_element.text != "" and @fitness_listview.goalStatus_element.text != nil }
        expect(@fitness_listview.goalStatus).to eq("Status: " + threeDaysAgo.to_s + " - " + threeDaysLater.to_s + " (4 days remaining)")

    end

    it ". Filter defaults to an expanded view. Filter button alt text reads Collapse filter options when expanded." do
      @date_filter.filterSection_element.when_visible
      expect(@date_filter.isFilterExpanded).to eq(true)
      expect(@date_filter.filterSection_element.attribute("title")).to eq("Collapse filter options")

      @date_filter.filterSection
      expect(@date_filter.isFilterExpanded).to eq(false)
      expect(@date_filter.filterSection_element.attribute("title")).to eq("Expand filter options")

      @date_filter.filterSection
    end

  end

  context "#CREAHB-1186|TC CREHAB-1195: Phone View: Option Button" do
    it ".Option control for Phone user" do
      @site.resizeWindowToPhone
      @fitness_listview.options_element.when_visible
      expect(@fitness_listview.options_element.text).to eq("Options")
    end
  end

  context "#CREHAB-1187|TC CREHAB-1195: Options control Display for phone user > Health filter and list view" do
    it ". When a user selects the Options button, a list of available options is displayed" do
      @fitness_listview.options
      expect(@fitness_listview.graphOption_element.text).to eq("Graph")
      expect(@fitness_listview.notificationOption_element.text).to include("Notification")
    end

    it ". When a user selects an option in the Options list the appropriate screen is displayed." do
      @fitness_listview.weeklyActivityGoalsOption
      Watir::Wait.until {@goal.modalTitle_element.visible?}

      @goal.modalTitle_element.when_visible
      expect(@goal.modalTitle_element.text).to eq("Set Weekly Activity Goals")

      @goal.cancel_element.when_visible
      @goal.cancel

      @goal.cancel_element.when_not_visible

      @fitness_listview.options_element.when_visible
      @fitness_listview.options
      @fitness_listview.graphOption_element.when_visible
      @fitness_listview.graphOption

      Watir::Wait.until { @fitness_listview.tableReferenceMsg_element.text == "A text description of the information on this page is available in the table view." }
      expect(@fitness_listview.tableReferenceMsg_element.text).to eq("A text description of the information on this page is available in the table view.")

      @fitness_listview.options
      expect(@fitness_listview.tableOption_element.visible?).to eq(true)
      @fitness_listview.notificationOption_element.when_visible
      @fitness_listview.notificationOption

      @notification_detail.contentLoaded_element.when_present(timeout=10)
      expect(@notification_detail.primaryHeader_element.text).to eq("Fitness Activity Notification")
      @notification_detail.cancel

      @fitness_listview.contentLoaded_element.when_present(timeout=10)

      @fitness_listview.options_element.when_visible
      @fitness_listview.options

      expect(@fitness_listview.graphOption_element.text).to eq("Graph")
      expect(@fitness_listview.notificationOption_element.text).to include("Notification")
      expect(@fitness_listview.options_element.attribute("aria-label")).to eq("Collapse Options list button")

      @fitness_listview.options
      Watir::Wait.until {@fitness_listview.options_element.attribute("aria-label") == "Expand Options list button"}

      expect(@fitness_listview.options_element.attribute("aria-label")).to eq("Expand Options list button")

    end

    it ". Phone user: the Weekly Activity Goals button is minimized to the trophy icon and word Goals" do
      expect(@fitness_listview.trophyIcon_element.visible?).to eq(true)
      expect(@fitness_listview.weeklyActivityGoalsOption_element.text).to eq("Goals")

      @site.resizeWindowToDefault
    end

  end

  context "CREHAB-483|TC #CREHAB-511 : Filter Options for Fitness Activity" do
    it ". Date Range, Start Date, End Date" do
      #No goals entered
      removeCollection("fitnessgoals", "healthfactordb")
      @site.refreshBrowser

      Watir::Wait.until { @fitness_listview.goalTitle_element.exists?}

      expect(@date_filter.dateRangeLabel_element.text).to eq("Date Range:")

      expect(@date_filter.startDateLabel_element.text).to eq("* Start Date:\nMM/DD/YYYY")
      expect(@date_filter.startDate_element.attribute("required")).to eq("true")
      expect(@date_filter.startDate_element.attribute("placeholder")).to eq("MM/DD/YYYY")

      expect(@date_filter.startDate).to eq(@threeMonthAgo)
      expect(@date_filter.endDate).to eq(@today.to_s)

      # goals entered  (goal's start date is not in the future)
      resetCollection("fitnessgoals", "healthfactordb", "weekly_goals")
      startDate = Time.now - 2.days
      endDate = Time.now + 4.days

      updateDocument("fitnessgoals", "healthfactordb", "54b53a6203644ae0cd231bc5", "startDate", startDate)
      updateDocument("fitnessgoals", "healthfactordb", "54b53a6203644ae0cd231bc5", "endDate", endDate)

      @site.refreshBrowser

      @fitness_listview.primaryHeader_element.when_visible
      expect(@fitness_listview.primaryHeader_element.text).to eq("Fitness Activity")

      expect(@date_filter.startDate).to eq(@threeMonthAgo)
      #when end date is in the future, it should display today's date.
      expect(@date_filter.endDate).to eq(@today.to_s)

      #golas entered (goal's start date is in the future)
      resetCollection("fitnessgoals", "healthfactordb", "weekly_goals")
      startDate = Time.now + 2.days
      endDate = Time.now + 8.days

      updateDocument("fitnessgoals", "healthfactordb", "54b53a6203644ae0cd231bc5", "startDate", startDate)
      updateDocument("fitnessgoals", "healthfactordb", "54b53a6203644ae0cd231bc5", "endDate", endDate)

      @site.refreshBrowser
      @fitness_listview.primaryHeader_element.when_visible
      expect(@fitness_listview.primaryHeader_element.text).to eq("Fitness Activity")

      #when the goal's end date is in the future, it should display today's date.
      expect(@date_filter.startDate).to eq(@threeMonthAgo)
      expect(@date_filter.endDate).to eq(@today.to_s)

      #Start date must occur before End Date
      @date_filter.startDate = getDateNthDaysFromNow(-10, "%m/%d/%Y")
      @date_filter.endDate = getDateNthDaysFromNow(-15, "%m/%d/%Y")
      @date_filter.filter

      errorMessages = ["Start Date must occur before End Date."]
      verifyErrorMessage(errorMessages)

      # 6 year date range validation
      @date_filter.endDate = @today.to_s
      @date_filter.startDate = getDateNthDaysFromNow(-2205, "%m/%d/%Y")
      @date_filter.filter

      sixYearsAgo = getDateNthYearsAgo(6, "%m/%d/%Y")

      #puts  + "Start Date must be between " + sixYearsAgo.to_s + " and " + @today.to_s

      errorMessages = ["Start Date must be between " + sixYearsAgo.to_s + " and " + @today.to_s + "."]
      verifyErrorMessage(errorMessages)

      @date_filter.startDate = getDateNthDaysFromNow(-30,  "%m/%d/%Y")
      @date_filter.endDate = getDateNthDaysFromNow(1,  "%m/%d/%Y")
      @date_filter.filter
      errorMessages = ["End Date must be between " + sixYearsAgo.to_s + " and " + @today.to_s + "."]
      verifyErrorMessage(errorMessages)

      #Input behavior: Constrain fields to match format; prevent future year entry and invalid month or day entry
      @date_filter.startDate = "02/13/2016"
      expect(@date_filter.startDate).to eq("02/13/201")
      @date_filter.endDate = "02/20/2017"
      expect(@date_filter.endDate).to eq("02/20/201")

      @date_filter.filter

      errorMessages = ["Start Date must be formatted MM/DD/YYYY.", "End Date must be formatted MM/DD/YYYY."]
      verifyErrorMessage(errorMessages)

    end

  end


  context "#AC CREHAB-647|TC CREHAB-681: Llist and Fitler View & Graph/Table functionality " do
    it ".  When a user opens the summary view (list and filter) detail screen, a graph button is shown in the utility bar (right justified, shows graph icon and is labeled 'Graph', alt text matches label)" do
      expect(@fitness_listview.graph_element.visible?).to eq(true)
      expect(@fitness_listview.graph_element.text).to eq("Graph")
    end

    it ". When a user selects the graph icon, the table list is replaced by the graph view:" do
      @fitness_listview.graph

      Watir::Wait.until { @fitness_listview.tableReferenceMsg_element.text == "A text description of the information on this page is available in the table view." }
      expect(@fitness_listview.tableReferenceMsg_element.text).to eq("A text description of the information on this page is available in the table view.")
    end

    it ". The Graph button in the utility bar toggles to the 'Table' button view (displays table icon, is labeled 'Table', alt text matches label)." do
      expect(@fitness_listview.table_element.visible?).to eq(true)
      expect(@fitness_listview.table_element.text).to eq("Table")
      @fitness_listview.table

      Watir::Wait.until {@fitness_listview.tableList_elements.length > 0 }

      tableHeaderArr = ["Date Entered", "Activity", "Duration (min)", "Steps", "Distance (miles)", "Intensity"]
      expect(@fitness_listview.getTableHeaders()).to eq(tableHeaderArr)

    end
  end

  context "AC #CREHAB-484|TC #CREHAB-511: Filter Action and Results Display for Fitness Activity" do
    it ". When a user selects the filter button, the system updates the table view to show records that match the filter options entered." do
      @date_filter.startDate = "01/15/2015"
      @date_filter.endDate = "01/15/2015"

      @date_filter.filter_element.when_visible
      @date_filter.filter

      Watir::Wait.until {@fitness_listview.tableList_elements.length == 2}

      @fitness_listview.firstTableRow_element.when_visible

      expect(@fitness_listview.getTextForRowCol(1, 1)).to eq("01/15/2015 12:47 PM")
      expect(@fitness_listview.getTextForRowCol(1, 2)).to eq("Running")
      expect(@fitness_listview.getTextForRowCol(1, 3)).to eq("90")
      expect(@fitness_listview.getTextForRowCol(1, 4)).to eq("")
      expect(@fitness_listview.getTextForRowCol(1, 5)).to eq("5.0")
      expect(@fitness_listview.getTextForRowCol(1, 6)).to eq("Somewhat Hard")

      expect(@fitness_listview.getTextForRowCol(2, 1)).to eq("01/15/2015 01:28 AM")
      expect(@fitness_listview.getTextForRowCol(2, 2)).to eq("Walking")
      expect(@fitness_listview.getTextForRowCol(2, 3)).to eq("60")
      expect(@fitness_listview.getTextForRowCol(2, 4)).to eq("3")
      expect(@fitness_listview.getTextForRowCol(2, 5)).to eq("")
      expect(@fitness_listview.getTextForRowCol(2, 6)).to eq("Easy")

    end

    it ". Given that results are found, entries are displayed in a table format, sorted in reverse chronological order; null values are shown as empty table cells unless otherwise noted" do
      @date_filter.startDate = "12/01/2014"
      @date_filter.endDate = "01/15/2015"

      @date_filter.filter_element.when_visible
      @date_filter.filter

      Watir::Wait.until {@fitness_listview.tableList_elements.length == 10}
      expect(@fitness_listview.tableList_elements.length).to eq(10)

      tableHeaderArr = ["Date Entered", "Activity", "Duration (min)", "Steps", "Distance (miles)", "Intensity"]
      expect(@fitness_listview.getTableHeaders()).to eq(tableHeaderArr)

      expect(@fitness_listview.getTextForRowCol(1, 1)).to eq("01/15/2015 12:47 PM")
      expect(@fitness_listview.getTextForRowCol(1, 2)).to eq("Running")
      expect(@fitness_listview.getTextForRowCol(1, 3)).to eq("90")
      expect(@fitness_listview.getTextForRowCol(1, 4)).to eq("")
      expect(@fitness_listview.getTextForRowCol(1, 5)).to eq("5.0")
      expect(@fitness_listview.getTextForRowCol(1, 6)).to eq("Somewhat Hard")

      expect(@fitness_listview.getTextForRowCol(2, 1)).to eq("01/15/2015 01:28 AM")
      expect(@fitness_listview.getTextForRowCol(2, 2)).to eq("Walking")
      expect(@fitness_listview.getTextForRowCol(2, 3)).to eq("60")
      expect(@fitness_listview.getTextForRowCol(2, 4)).to eq("3")
      expect(@fitness_listview.getTextForRowCol(2, 5)).to eq("")
      expect(@fitness_listview.getTextForRowCol(2, 6)).to eq("Easy")

      expect(@fitness_listview.getTextForRowCol(3, 1)).to eq("01/08/2015 01:06 PM")
      expect(@fitness_listview.getTextForRowCol(3, 2)).to eq("Gardening")
      expect(@fitness_listview.getTextForRowCol(3, 3)).to eq("70")
      expect(@fitness_listview.getTextForRowCol(3, 4)).to eq("")
      expect(@fitness_listview.getTextForRowCol(3, 5)).to eq("")
      expect(@fitness_listview.getTextForRowCol(3, 6)).to eq("")

      expect(@fitness_listview.getTextForRowCol(4, 1)).to eq("01/08/2015 01:06 PM")
      expect(@fitness_listview.getTextForRowCol(4, 2)).to eq("Yoga")
      expect(@fitness_listview.getTextForRowCol(4, 3)).to eq("50")
      expect(@fitness_listview.getTextForRowCol(3, 4)).to eq("")
      expect(@fitness_listview.getTextForRowCol(3, 5)).to eq("")
      expect(@fitness_listview.getTextForRowCol(3, 6)).to eq("")

      expect(@fitness_listview.getTextForRowCol(9, 1)).to eq("12/15/2014 01:04 PM")
      expect(@fitness_listview.getTextForRowCol(9, 2)).to eq("Dancing")
      expect(@fitness_listview.getTextForRowCol(9, 3)).to eq("90")
      expect(@fitness_listview.getTextForRowCol(9, 4)).to eq("")
      expect(@fitness_listview.getTextForRowCol(9, 5)).to eq("")
      expect(@fitness_listview.getTextForRowCol(9, 6)).to eq("")

      expect(@fitness_listview.getTextForRowCol(10, 1)).to eq("12/15/2014 12:48 PM")
      expect(@fitness_listview.getTextForRowCol(10, 2)).to eq("Swimming")
      expect(@fitness_listview.getTextForRowCol(10, 3)).to eq("80")
      expect(@fitness_listview.getTextForRowCol(10, 4)).to eq("")
      expect(@fitness_listview.getTextForRowCol(10, 5)).to eq("")
      expect(@fitness_listview.getTextForRowCol(10, 6)).to eq("")

      @site.resizeWindowToPhone

      tableHeaderArr = ["Date Entered", "Activity"]
      expect(@fitness_listview.getTableHeaders()).to eq(tableHeaderArr)

      @site.resizeWindowToDefault

      @date_filter.startDate = "01/01/2014"
      @date_filter.endDate = "12/31/2014"

      @date_filter.filter_element.when_visible
      @date_filter.filter

      Watir::Wait.until {@fitness_listview.tableList_elements.length == 4}

      expect(@fitness_listview.getTextForRowCol(1, 1)).to eq("12/15/2014 01:04 PM")
      expect(@fitness_listview.getTextForRowCol(1, 2)).to eq("Dancing")
      expect(@fitness_listview.getTextForRowCol(1, 3)).to eq("90")
      expect(@fitness_listview.getTextForRowCol(1, 4)).to eq("")
      expect(@fitness_listview.getTextForRowCol(1, 5)).to eq("")
      expect(@fitness_listview.getTextForRowCol(1, 6)).to eq("")

      expect(@fitness_listview.getTextForRowCol(2, 1)).to eq("12/15/2014 12:48 PM")
      expect(@fitness_listview.getTextForRowCol(2, 2)).to eq("Swimming")
      expect(@fitness_listview.getTextForRowCol(2, 3)).to eq("80")
      expect(@fitness_listview.getTextForRowCol(2, 4)).to eq("")
      expect(@fitness_listview.getTextForRowCol(2, 5)).to eq("")
      expect(@fitness_listview.getTextForRowCol(2, 6)).to eq("")

      expect(@fitness_listview.getTextForRowCol(3, 1)).to eq("11/01/2014 01:01 PM")
      expect(@fitness_listview.getTextForRowCol(3, 2)).to eq("Walking")
      expect(@fitness_listview.getTextForRowCol(3, 3)).to eq("45")
      expect(@fitness_listview.getTextForRowCol(3, 4)).to eq("3000")
      expect(@fitness_listview.getTextForRowCol(3, 5)).to eq("")
      expect(@fitness_listview.getTextForRowCol(3, 6)).to eq("")

      expect(@fitness_listview.getTextForRowCol(4, 1)).to eq("08/01/2014 12:48 PM")
      expect(@fitness_listview.getTextForRowCol(4, 2)).to eq("Running")
      expect(@fitness_listview.getTextForRowCol(4, 3)).to eq("3")
      expect(@fitness_listview.getTextForRowCol(4, 4)).to eq("")
      expect(@fitness_listview.getTextForRowCol(4, 5)).to eq("")
      expect(@fitness_listview.getTextForRowCol(4, 6)).to eq("")

      #focus moved to first row
      expect(@fitness_listview.getFocusedElement == @fitness_listview.firstTableRow_element).to eq(true)

      @fitness_listview.tableList_elements[0].click

      @fitness_activity.primaryHeader_element.when_visible
      expect(@fitness_activity.primaryHeader_element.text).to eq("Edit Fitness Activity Entry")

      @fitness_activity.cancel

      @fitness_listview.primaryHeader_element.when_visible
      expect(@fitness_listview.primaryHeader_element.text).to eq("Fitness Activity")

      #filtered list should display
      Watir::Wait.until {@fitness_listview.tableList_elements.length == 4}
      expect(@fitness_listview.tableList_elements.length).to eq(4)

      expect(@date_filter.startDate).to eq("01/01/2014")
      expect(@date_filter.endDate).to eq("12/31/2014")

    end


    it ". After the filter button is selected, the focus moves to the returned results, first row, or the No results found message." do
      #focus moves to first row covered in above test scenario
      @date_filter.startDate = "01/15/2013"
      @date_filter.endDate = "01/20/2013"

      @date_filter.filter_element.when_visible
      @date_filter.filter

      Watir::Wait.until {@fitness_listview.noResultsMessage_element.exists? }
      expect(@fitness_listview.noResultsMessage_element.text).to eq("No results were found with the current filters. Use the Add button above to create a new record.")
      #focus moves to no results found message - manual testing
      #expect(@fitness_listview.noResultsMessage_element.focused?).to eq(true)

    end
  end


  def addWeeklyGoal(minutes, steps, miles, startDate)
    @fitness_listview.weeklyActivityGoals_element.when_visible
    @fitness_listview.weeklyActivityGoals

    @goal.modalTitle_element.when_visible
    expect(@goal.modalTitle_element.text).to eq("Set Weekly Activity Goals")

    @goal.minutes = minutes
    @goal.steps = steps
    @goal.miles = miles
    @goal.startDate = startDate

    @goal.save

    if (@modal.continue2_element.exists?)
      @modal.continue2
      @modal.continue2_element.when_not_visible
    end
  end

end
