require 'rspec'
require 'watir-webdriver'

require_relative '../../lib/browser_container/site'
require_relative '../../spec/rspec_helper'
require_relative '../../spec/shared_context'
require_relative 'health_helper'
require_relative '../../lib/module/DateUtility'
require_relative '../../lib/module/MongoUtility'

describe '[Story #CREHAB-410: Blood Pressure - Add]
          [Story #CREHAB-1097: Update text for BP & P]', :regression => true do
  include DateUtility
  include MongoUtility
  app_title = "VA FitHeart"

  before(:all) do
    removeAllDocuments()

    @driver = Watir::Browser.new
    @site = Site.new(BASE_URL, @driver)

    @title = "Blood Pressure & Pulse"

    @login = @site.loginPage
    @eula = @site.eula
    @roa = @site.roa
    @landing = @site.landing
    @leftNav = @site.leftMenu
    @modal = @site.modal
    @health = @site.health
    @detail = @site.health_detail
    @list_view = @site.list_view
    @error = @site.error
    @calendar = @site.calendar_picker

    @today = getDateNthDaysAgo(0, "%m/%d/%Y")

    @EXPECTED_ROW_0_FORM = [\
          "* required field",
          "* Date:",
          "MM/DD/YYYY",
          "This calendar control is not optimized for screen readers. Please directly enter dates in the text box provided. Open Calendar Control.",
          "Day selected is " +  getDayofWeek(@today),
          getDayofWeek(@today),
          "* Time:",
          "hh:mm AM/PM",
          "This time selection control is not optimized for screen readers. Please directly enter the time in the text box provided. Open Time Picker",
          "* Results  (provide both blood pressure values or a pulse value to save)",
          "Systolic (mmHg):",
          "Between 80 and 200",
          "Diastolic (mmHg):",
          "Between 50 and 150",
          "Pulse (bpm):",
          "Between 40 and 250",
          "Position:",
          "Select",
          "Sitting",
          "Standing",
          "Lying",
          "Note:",
          "250 character maximum",
          "250 characters left",
          "Notification:",
          "Status: Off",
          "",
          "To edit the Blood Pressure & Pulse notification, select the Notification button on the previous screen.",
          "Save Cancel"]
  end

  after(:all) do
    @site.quitDriver
  end

  context "[Before] Log in and navigate to
            #AC CREHAB-1189|TC CREHAB-1191: Update and to & for blood pressure and pulse" do
    it ". Login and navigate to" do
      acceptEulaAndQuickLogin("zztest.patient01", "pass", "D123401")

      navigateToHealth(app_title)
      navigateTo(@title)
    end
  end

  context "#AC CREHAB-410|TC CREHAB-908: Add New Entry for FitHeart BP " do
    it_behaves_like "Add Button Validation"

    it_behaves_like  "Labels Validation"

    it_behaves_like "Date Validation"

    it_behaves_like "Time Validation"

    it_behaves_like "Note Validation"

  end

  context "#AC CREHAB-410|TC CREHAB-908: Add New Entry for FitHeart BP " do
    it ". Duplicate entries on the same date are not permitted." do
      time = getNthHoursFromNow(0)
      date = @detail.date

      @detail.systolic = "101"
      @detail.diastolic = "89"
      @detail.pulse = ""

      @detail.time = time
      @detail.save

      Watir::Wait.until {@list_view.secondaryHeader_element.text == "Health"}
      Watir::Wait.until {@health.bloodPressureAmount_element.exists?}

      Watir::Wait.until {@health.bloodPressureAmount_element.text.strip == "101/89 mmHg"}
      expect(@health.bloodPressureAmount_element.text.strip).to eq("101/89 mmHg")
      expect(@health.pulseAmount_element.exists?).to eq(false)

      expect(@health.bpDate_element.text).to eq("Today")

      @list_view.primaryHeader_element.when_visible
      Watir::Wait.until {@list_view.primaryHeader_element.text == @title}

      expect(@list_view.primaryHeader_element.text).to eq(@title)

      @list_view.tableList_elements[0].when_visible

      @list_view.addButton_element.when_visible
      @list_view.addButton

      Watir::Wait.until {@detail.primaryHeader_element.text == "Add " + @title + " Entry"}
      expect(@detail.primaryHeader_element.text).to eq("Add " + @title + " Entry")

      @detail.date_element.when_visible

      @detail.date = date
      @detail.time = time
      @detail.systolic = "102"
      @detail.diastolic = "95"
      @detail.pulse = "89"

      @detail.save
      errorMessages = ["An entry has already been saved with the same date and time. Please change your date and/or time and save again."]
      verifyErrorMessage(errorMessages)

      clickCancelAndFocusToHeader
    end

    it ".  Results (provide both blood pressure values or a pulse value to save) (primary label, required)" do
      @list_view.addButton_element.when_visible
      @list_view.addButton_element.click

      Watir::Wait.until {@detail.primaryHeader_element.text == "Add " + @title + " Entry"}
      expect(@detail.primaryHeader_element.text).to eq("Add " + @title + " Entry")

      @detail.date_element.when_visible

      @detail.save
      errorMessages = ["Provide either both blood pressure values or a pulse value to save."]
      verifyErrorMessage(errorMessages)

    end

    it ". Systolic (mmHg): Numeric, textbox" do
      expect(@detail.systolic_element.attribute("placeholder")).to eq("Between 80 and 200")
      @detail.systolic = 79
      @detail.save

      errorMessages = ["Systolic (mmHg) is outside the expected range. Please enter a value between 80 and 200."]
      verifyErrorMessage(errorMessages)

      @detail.systolic = 201
      @detail.save

      errorMessages = ["Systolic (mmHg) is outside the expected range. Please enter a value between 80 and 200."]
      verifyErrorMessage(errorMessages)
    end

    it ". Diastolic (mmHg): Numeric, textbox" do
      expect(@detail.diastolic_element.attribute("placeholder")).to eq("Between 50 and 150")
      @detail.diastolic = 49
      @detail.save

      errorMessages = ["Diastolic (mmHg) is outside the expected range. Please enter a value between 50 and 150."]
      verifyErrorMessage(errorMessages)

      @detail.diastolic = 151
      @detail.save

      errorMessages = ["Diastolic (mmHg) is outside the expected range. Please enter a value between 50 and 150."]
      verifyErrorMessage(errorMessages)
    end

    it ". The Systolic value must be greater than the Diastolic value on Save, otherwise display the error message: " do
      @detail.systolic = 79
      @detail.diastolic = 80
      @detail.save

      errorMessages = ["The Systolic (mmHg) value must be greater than the Diastolic (mmHg) value. Please check the values and re-enter."]
      verifyErrorMessage(errorMessages)
    end

    it ". If only a Diastolic value is entered or if only a Systolic value is entered on Save, display the error message:" do
      @detail.systolic  = 82
      @detail.diastolic = ""
      @detail.save
      errorMessages = ["Both Systolic and Diastolic values are required to save a blood pressure reading."]
      verifyErrorMessage(errorMessages)

      @detail.systolic = ""
      @detail.diastolic = 87
      @detail.save
      errorMessages = ["Both Systolic and Diastolic values are required to save a blood pressure reading."]
      verifyErrorMessage(errorMessages)

    end

    it ". Pulse (bpm): Numeric, textbox" do
      expect(@detail.pulse_element.attribute("placeholder")).to eq("Between 40 and 250")
      @detail.pulse = 39
      @detail.save
      errorMessages = ["Pulse (bpm) is outside the expected range. Please enter a value between 40 and 250."]
      verifyErrorMessage(errorMessages)

      @detail.pulse = 251
      @detail.save
      errorMessages = ["Pulse (bpm) is outside the expected range. Please enter a value between 40 and 250."]
      verifyErrorMessage(errorMessages)

    end

    it ". If neither of the BP readings are valued and the pulse reading is not value don save, display the error message:"  do
      @detail.systolic = ""
      @detail.diastolic = ""
      @detail.pulse = ""
      @detail.save
      errorMessages = ["Provide either both blood pressure values or a pulse value to save."]
      verifyErrorMessage(errorMessages)
    end

    it ". Position: (single select, drop down values: Select (default), Sitting, Standing, Lying, " do
      expect(@detail.positionSelect).to eq("Select")
      selectArr = ["Select", "Sitting", "Standing", "Lying"]
      expect(@detail.positionSelect_options).to eq(selectArr)

      for i in 0..(selectArr.length - 1)
        @detail.positionSelect = selectArr[i]
        expect(@detail.positionSelect).to eq(selectArr[i])
      end
    end

    it ". Standard Save and Cancel options are available." do
      expect(@detail.save_element.visible?).to eq(true)
      expect(@detail.cancel_element.visible?).to eq(true)
    end
  end
end
