#!/bin/env ruby
# encoding: utf-8
require 'rspec'
require 'watir-webdriver'

require_relative '../../lib/browser_container/site'
require_relative '../../spec/rspec_helper'
require_relative '../../lib/module/DateUtility'
require_relative '../../lib/module/MongoUtility'
require_relative '../health/health_helper'
require_relative '../shared_context'

describe '[Story #CREHAB-409: Filter and List View ]
          [Story #CREHAB-411: Blood Pressure & Pulse - Graph]
          [Story #CREAHB-514: Phone User : Options Control]
          [Story #CREHAB-1097: Updated and to & Blood Pressure and Pulse]', :regression => true do
  include DateUtility
  include MongoUtility
  app_title = "VA FitHeart"

  before(:all) do
    removeAllDocuments()

    @driver = Watir::Browser.new
    @site = Site.new(BASE_URL, @driver)

    @title = "Blood Pressure & Pulse"

    @login = @site.loginPage
    @eula = @site.eula
    @roa = @site.roa
    @landing = @site.landing
    @leftNav = @site.leftMenu
    @modal = @site.modal
    @health = @site.health
    @detail = @site.health_detail
    @list_view = @site.list_view
    @date_filter = @site.date_filter
    @error = @site.error
    @calendar = @site.calendar_picker
    @learn_more = @site.learnMore
    @notification_detail = @site.notification_detail

    @today = getDateNthDaysFromNow(0, "%m/%d/%Y")
    @tomorrow = getDateNthDaysFromNow(1, "%m/%d/%Y")
    @yesterday = getDateNthDaysFromNow(-1, "%m/%d/%Y")
    @sixYearsAgo = getDateNthYearsFromNow(-6, "%m/%d/%Y")
    @threeMonthAgo = getDateNthMonthsFromNow(-3, "%m/%d/%Y")
    @hundredDaysAgo = getDateNthDaysFromNow(-100, "%m/%d/%Y")
    @sixYearsAndOneDayAgo = getDateNthDaysFromGivenDate(@sixYearsAgo, -1)
  end

  after(:all) do
    @site.quitDriver
  end

  context "[Before] Log in and navigate to" do
    it ". Login and navigate to" do
      acceptEulaAndQuickLogin("zztest.patient01", "pass", "D123401")

      expect(@landing.appTitle).to eq(app_title)
      @landing.welcomeTitleForAuthenticatedUser_element.when_visible
      expect(@landing.welcomeTitleForAuthenticatedUser).to eq("Welcome to VA FitHeart")
    end
  end

  context "#AC CREHAB-860|TC CREHAB-904: Sub Feature Selection for Fitheart BP
          #AC CREHAB-1189|TC CREHAB-1191: Update text for BP & P" do
    it ". When a user opens the Health feature, the system displays Blood Pressure & Pulse in the Main screen (column 2):" do
      @leftNav.leftMenu_element.when_visible
      @leftNav.leftMenu

      @leftNav.health_element.when_visible
      @leftNav.health

      @health.primaryHeader_element.when_visible
      expect(@health.secondaryHeader_element.text).to eq("Health")
      expect(@health.primaryHeader_element.text).to eq("Health")

      expect(@health.leftSubMenulist_elements[0].text.strip).to eq(Health::BLOOD_PRESSURE_PULSE_LABEL)
      expect(@health.bloodPressureAndPurlseIcon_element.visible?).to eq(true)
      @health.bloodPressureAndPulse_element.when_visible
      @health.bloodPressureAndPulse_element.click

      Watir::Wait.until {@list_view.secondaryHeader_element.text == "Health"}
      Watir::Wait.until {@list_view.primaryHeader_element.text == @title}

      expect(@list_view.secondaryHeader_element.text).to eq("Health")
      expect(@list_view.primaryHeader_element.text).to eq(@title)
    end

    it ". The date of the most recent tracker entry is displayed below the Pulse label. If date = today display Today if date = Today-1 display Yesterday, otherwise display the actual date (MM/DD/YYYY).
          The Blood Pressure reading  <Blood Pressure value> mmHg
          The Pulse reading <Pulse value> bpm" do
      date = getDateNthDaysFromNow(-2, "%m/%d/%Y")
      systolic = ""
      diastolic = ""
      pulse = "90"
      position = "Sitting"
      note = "Testing"
      addBP(date, systolic, diastolic, pulse, position, note)
      expect(@health.bpDate_element.text).to eq(date)
      expect(@health.bloodPressureAmount_element.exists?).to eq(false)
      expect(@health.pulseAmount_element.text).to eq(pulse + " bpm")

      date = getDateNthDaysFromNow(-1, "%m/%d/%Y")
      systolic = "110"
      diastolic = "92"
      pulse = ""
      position = "Standing"
      note = "n/a"
      addBP(date, systolic, diastolic, pulse, position, note)
      expect(@health.bpDate_element.text).to eq("Yesterday")
      expect(@health.bloodPressureAmount_element.text.strip).to eq(systolic +  "/" + diastolic + " mmHg")
      expect(@health.pulseAmount_element.exists?).to eq(false)

      date = @today
      systolic = "124"
      diastolic = "91"
      pulse = "99"
      position = "Lying"
      note = "n/a"
      addBP(date, systolic, diastolic, pulse, position, note)
      expect(@health.bpDate_element.text).to eq("Today")
      expect(@health.bloodPressureAmount_element.text.strip).to eq(systolic +  "/" + diastolic + " mmHg")
      expect(@health.pulseAmount_element.text).to eq(pulse + " bpm")

    end
  end

  context "#AC CREHAB-868|TC CREHAB-909: BP Button & Features for Fitheart " do
    include_examples "Graph/Table Button Validation"
  end
  context "#AC CREHAB-861|TC CREHAB-904: Sub Feature Default View for FitHeart BP" do
    include_examples "Date Filter Validation"
  end

  context "#AC CREHAB-1186|TC CREHAB-1195: Phone View Option Button
           #AC CREAHB-1187|TC CREHAB-1195: Options control display for Phone user" do
    include_examples "Phone User: Options control"
  end

  context "#AC CREHAB-862|TC CREHAB-904: Filter Action and REsults Display for FitHeart BP" do
    it ". When a user selects the filter button, the system updates the table view to show records that match the filter options entered." do
      @date_filter.startDate = @sixYearsAgo
      @date_filter.endDate = @today
      @date_filter.filter
      Watir::Wait.until { @list_view.primaryHeader_element.text == @title }
      expect(@list_view.primaryHeader_element.text).to eq(@title)
      Watir::Wait.until {@list_view.tableList_elements.length == 3}
      expect(@list_view.tableList_elements.length).to eq(3)

      date = @threeMonthAgo
      systolic = "114"
      diastolic = "88"
      pulse = "100"
      position = "Select"
      note = "n/a"
      addBP(date, systolic, diastolic, pulse, position, note)

      date = getDateNthYearsFromNow(-1, "%m/%d/%Y")
      systolic = "128"
      diastolic = "78"
      pulse = "89"
      position = "Sitting"
      note = "n/a"
      addBP(date, systolic, diastolic, pulse, position, note)

      date = getDateNthYearsFromNow(-2, "%m/%d/%Y")
      systolic = "126"
      diastolic = "82"
      pulse = "101"
      position = "Lying"
      note = "n/a"
      addBP(date, systolic, diastolic, pulse, position, note)

      date = getDateNthYearsFromNow(-6, "%m/%d/%Y")
      systolic = "109"
      diastolic = "83"
      pulse = "101"
      position = "Lying"
      note = "n/a"
      addBP(date, systolic, diastolic, pulse, position, note)
    end

    it ". After the filter button is selected, the focus moves to the returned results, first row, or the No results found message." do
      verifyFocusMoveToFirstRow(4)
    end

    it ". Given that results are found, entries are displayed in a table format, columns are left aligned, sorted in reverse chronological order by date and time entered: null values are shown as empty table cells unless otherwise noted" do
      tableHeaderArr = ["Systolic (mmHg)", "Diastolic (mmHg)", "Pulse (bpm)", "Position", "Date Entered"]
      expect(@list_view.getTableHeaders()).to eq(tableHeaderArr)

      verifyDefaultTableList

      @date_filter.startDate = getDateNthYearsFromNow(-2, "%m/%d/%Y")
      @date_filter.endDate = @today
      @date_filter.filter
      Watir::Wait.until { @list_view.primaryHeader_element.text == @title }
      expect(@list_view.primaryHeader_element.text).to eq(@title)
      Watir::Wait.until {@list_view.tableList_elements.length == 6}
      expect(@list_view.tableList_elements.length).to eq(6)

      expect(@list_view.getFocusedElement() == @list_view.firstTableRow_element).to eq(true)

      expect(@list_view.getTextForRowCol(5, 1)).to eq("128")
      expect(@list_view.getTextForRowCol(5, 2)).to eq("78")
      expect(@list_view.getTextForRowCol(5, 3)).to eq("89")
      expect(@list_view.getTextForRowCol(5, 4)).to eq("Sitting")
      expect(@list_view.getTextForRowCol(5, 5).include?(getDateNthYearsFromNow(-1, "%m/%d/%Y"))).to eq(true)
      expect(isDateFormatValid?(@list_view.getTextForRowCol(5, 5), "%m/%d/%Y %I:%M %p"))

      expect(@list_view.getTextForRowCol(6, 1)).to eq("126")
      expect(@list_view.getTextForRowCol(6, 2)).to eq("82")
      expect(@list_view.getTextForRowCol(6, 3)).to eq("101")
      expect(@list_view.getTextForRowCol(6, 4)).to eq("Lying")
      expect(@list_view.getTextForRowCol(6, 5).include?(getDateNthYearsFromNow(-2, "%m/%d/%Y"))).to eq(true)
      expect(isDateFormatValid?(@list_view.getTextForRowCol(6, 5), "%m/%d/%Y %I:%M %p"))

      @date_filter.startDate = getDateNthYearsFromNow(-6, "%m/%d/%Y")
      @date_filter.endDate = @today
      @date_filter.filter
      Watir::Wait.until { @list_view.primaryHeader_element.text == @title }
      expect(@list_view.primaryHeader_element.text).to eq(@title)
      Watir::Wait.until {@list_view.tableList_elements.length == 7}
      expect(@list_view.tableList_elements.length).to eq(7)

      expect(@list_view.getFocusedElement() == @list_view.firstTableRow_element).to eq(true)

      expect(@list_view.getTextForRowCol(7, 1)).to eq("109")
      expect(@list_view.getTextForRowCol(7, 2)).to eq("83")
      expect(@list_view.getTextForRowCol(7, 3)).to eq("101")
      expect(@list_view.getTextForRowCol(7, 4)).to eq("Lying")
      expect(@list_view.getTextForRowCol(7, 5).include?(getDateNthYearsFromNow(-6, "%m/%d/%Y"))).to eq(true)
      expect(isDateFormatValid?(@list_view.getTextForRowCol(7, 5), "%m/%d/%Y %I:%M %p"))

      #hide tested column on phone view
      @site.resizeWindowToPhone
      tableHeaderArr = ["Systolic (mmHg)", "Diastolic (mmHg)", "Pulse (bpm)", "Date Entered"]
      expect(@list_view.getTableHeaders()).to eq(tableHeaderArr)
      @site.resizeWindowToDefault

    end

    it ". If no records are found when a filter is applied the message the following message is shown in the screen area in place of the table view." do
      verifyNoResultsMessage(getDateNthDaysFromNow(-3, "%m/%d/%Y"), getDateNthDaysFromNow(-3, "%m/%d/%Y"))
    end
  end

  context "#AC CREHAB-863|TC CREAHB-904: Navigation and Reset for FitHeart BP" do
    it ". Reset Filter Functionality "  do
      verifyReset(4)
      verifyDefaultTableList
    end

    it ". When a phone user views the Filter and Table List View Detail screen,  a < is shown (alt text: Return to Health options); when clicked the user is returned to the Main Screen for the feature where the group options for that feature are listed." do
      @site.resizeWindowToPhone
      expect(@list_view.backButton_element.visible?).to eq(true)
      expect(@list_view.backButton_element.attribute("Return to Health options"))
      @list_view.backButton_element.click

      @health.bloodPressureAndPulse_element.when_visible
      Watir::Wait.until { @list_view.secondaryHeader_element.text == "Health" }
      Watir::Wait.until { @health.bloodPressureAndPulse_element.text.strip == "Blood Pressure\nPulse\n  Today\n  124/91 mmHg\n  99 bpm" }

      expect(@health.bloodPressureAndPulse_element.text.strip).to eq("Blood Pressure\nPulse\n  Today\n  124/91 mmHg\n  99 bpm")
      @site.resizeWindowToDefault
    end

  end


  def verifyDefaultTableList
    expect(@list_view.getTextForRowCol(1, 1)).to eq("124")
    expect(@list_view.getTextForRowCol(1, 2)).to eq("91")
    expect(@list_view.getTextForRowCol(1, 3)).to eq("99")
    expect(@list_view.getTextForRowCol(1, 4)).to eq("Lying")
    expect(@list_view.getTextForRowCol(1, 5).include?(@today)).to eq(true)
    expect(isDateFormatValid?(@list_view.getTextForRowCol(1, 5), "%m/%d/%Y %I:%M %p"))

    expect(@list_view.getTextForRowCol(2, 1)).to eq("110")
    expect(@list_view.getTextForRowCol(2, 2)).to eq("92")
    expect(@list_view.getTextForRowCol(2, 3)).to eq("")
    expect(@list_view.getTextForRowCol(2, 4)).to eq("Standing")
    expect(@list_view.getTextForRowCol(2, 5).include?(getDateNthDaysFromNow(-1, "%m/%d/%Y"))).to eq(true)
    expect(isDateFormatValid?(@list_view.getTextForRowCol(2, 5), "%m/%d/%Y %I:%M %p"))

    expect(@list_view.getTextForRowCol(3, 1)).to eq("")
    expect(@list_view.getTextForRowCol(3, 2)).to eq("")
    expect(@list_view.getTextForRowCol(3, 3)).to eq("90")
    expect(@list_view.getTextForRowCol(3, 4)).to eq("Sitting")
    expect(@list_view.getTextForRowCol(3, 5).include?(getDateNthDaysFromNow(-2, "%m/%d/%Y"))).to eq(true)
    expect(isDateFormatValid?(@list_view.getTextForRowCol(4, 5), "%m/%d/%Y %I:%M %p"))

    expect(@list_view.getTextForRowCol(4, 1)).to eq("114")
    expect(@list_view.getTextForRowCol(4, 2)).to eq("88")
    expect(@list_view.getTextForRowCol(4, 3)).to eq("100")
    expect(@list_view.getTextForRowCol(4, 4)).to eq("")
    expect(@list_view.getTextForRowCol(4, 5).include?(@threeMonthAgo)).to eq(true)
    expect(isDateFormatValid?(@list_view.getTextForRowCol(4, 5), "%m/%d/%Y %I:%M %p"))
  end
end

