#!/bin/env ruby
# encoding: utf-8
require 'rspec'
require 'watir-webdriver'

require_relative '../../lib/browser_container/site'
require_relative '../../spec/rspec_helper'
require_relative '../../lib/module/DateUtility'
require_relative '../../lib/module/MongoUtility'
require_relative '../health/health_helper'
require_relative '../../spec/shared_context'

describe '[Story #CREHAB-381: Cholesterol - Filter and List view]
          [Story #CREHAB-379: Cholesterol Graph]
          [Story #CREAHB-514: Phone User : Options control]', :regression => true do
  include DateUtility
  include MongoUtility
  app_title = "VA FitHeart"

  before(:all) do
    removeAllDocuments()

    @driver = Watir::Browser.new
    @site = Site.new(BASE_URL, @driver)

    @title = "Cholesterol"

    @login = @site.loginPage
    @eula = @site.eula
    @roa = @site.roa
    @landing = @site.landing
    @leftNav = @site.leftMenu
    @modal = @site.modal
    @health = @site.health
    @detail = @site.health_detail
    @list_view = @site.list_view
    @learn_more = @site.learnMore
    @error = @site.error
    @calendar = @site.calendar_picker
    @date_filter = @site.date_filter
    @notification_detail = @site.notification_detail

    @today = getDateNthDaysFromNow(0, "%m/%d/%Y")
    @yesterday = getDateNthDaysFromNow(-1, "%m/%d/%Y")
    @tomorrow = getDateNthDaysFromNow(1, "%m/%d/%Y")
    @twoDaysAgo = getDateNthDaysFromNow(-2, "%m/%d/%Y")
    @threeMonthAgo = getDateNthMonthsFromNow(-3, "%m/%d/%Y")
    @sixYearsAgo = getDateNthYearsFromNow(-6, "%m/%d/%Y")
    @hundredDaysAgo = getDateNthDaysFromNow(-100, "%m/%d/%Y")
    @oneYearAgo = getDateNthYearsFromNow(-1, "%m/%d/%Y")
    @twoYearAgo = getDateNthYearsFromNow(-2, "%m/%d/%Y")
    @sixYearsAndOneDayAgo = getDateNthDaysFromGivenDate(@sixYearsAgo, -1)

  end

  after(:all) do
    @site.quitDriver
  end

  context "[Before] Log in and navigate to" do
    it ". Login and navigate to" do
      acceptEulaAndQuickLogin("zztest.patient01", "pass", "D123401")

      expect(@landing.appTitle).to eq(app_title)
      @landing.welcomeTitleForAuthenticatedUser_element.when_visible
      expect(@landing.welcomeTitleForAuthenticatedUser).to eq("Welcome to VA FitHeart")
    end
  end

  context "#AC CREHAB-870|TC CREHAB-912: Sub Feature Selection for FitHeart Cholesterol" do
    it ".When a user opens the Health feature, the system displays the Cholesterol in the Main screen (column 2):" do
      @leftNav.leftMenu_element.when_visible
      @leftNav.leftMenu

      @leftNav.health_element.when_visible
      @leftNav.health

      @health.primaryHeader_element.when_visible
      expect(@health.secondaryHeader_element.text).to eq("Health")
      expect(@health.primaryHeader_element.text).to eq("Health")

      @health.cholesterol_element.when_visible
      expect(@health.leftSubMenulist_elements[3].text.strip).to eq(@title)
      expect(@health.cholesterolIcon_element.visible?).to eq(true)

      navigateTo(@title)
    end

    it ". The date of the most recent tracker entry is displayed below the Cholesterol label. If date = today display Today if date = Today-1 display Yesterday" do
      date = @twoDaysAgo
      totalSerumCholesterol = "555"
      idLipo = ""
      hdLipo = ""
      tryiglycerides = ""
      fasting = "Select"
      note = ""
      addCholesterol(date, totalSerumCholesterol, idLipo, hdLipo, tryiglycerides, fasting, note)
      expect(@health.cholesterolAmount_element.text.strip).to eq("555 mg/dl")
      expect(@health.cholesterolDate_element.text.strip).to eq(date)

      date = @yesterday
      totalSerumCholesterol = ""
      idLipo = "255"
      hdLipo = "257"
      tryiglycerides = ""
      fasting = "Yes"
      note = ""
      addCholesterol(date, totalSerumCholesterol, idLipo, hdLipo, tryiglycerides, fasting, note)
      expect(@health.cholesterolAmount_element.exists?).to eq(false)
      expect(@health.cholesterolDate_element.exists?).to eq(false)

      date = @today
      totalSerumCholesterol = "355"
      idLipo = "155"
      hdLipo = "157"
      tryiglycerides = ""
      fasting = "No"
      note = ""
      addCholesterol(date, totalSerumCholesterol, idLipo, hdLipo, tryiglycerides, fasting, note)
      expect(@health.cholesterolAmount_element.text.strip).to eq("355 mg/dl")
      expect(@health.cholesterolDate_element.text.strip).to eq("Today")

    end

  end

  context "#AC CREHAB-1186|TC CREHAB-1195: Phone View Option Button
           #AC CREAHB-1187|TC CREHAB-1195: Options control display for Phone user" do
    include_examples "Phone User: Options control"
  end

  context "#AC CREAHB-871|TC CREHAB-912: Sub Feature Defatul View for FitHeart Cholesterol, Filter, Start Date, End Date " do
    include_examples "Date Filter Validation"

    #todo reset
  end

  context "#AC CREHAB-880|TC CREHAB-914: Graph Button & Features for Cholesterol " do
    include_examples "Graph/Table Button Validation"
  end

  context "#AC CREAHB-872|TC CREHAB-912: Filter Action and REsults Display for FitHeart Cholesterol " do
    it ". 	When a user selects the filter button, the system updates the table view to show records that match the filter options entered." do
      @date_filter.startDate = @sixYearsAgo
      @date_filter.endDate = @today
      @date_filter.filter
      Watir::Wait.until { @list_view.primaryHeader_element.text == @title }
      expect(@list_view.primaryHeader_element.text).to eq(@title)
      Watir::Wait.until {@list_view.tableList_elements.length == 3}
      expect(@list_view.tableList_elements.length).to eq(3)

      date = @threeMonthAgo
      totalSerumCholesterol = "312"
      idLipo = "244"
      hdLipo = "131"
      tryiglycerides = "123"
      fasting = "Yes"
      note = ""
      addCholesterol(date, totalSerumCholesterol, idLipo, hdLipo, tryiglycerides, fasting, note)

      date = @hundredDaysAgo
      totalSerumCholesterol = "293"
      idLipo = "63"
      hdLipo = "68"
      tryiglycerides = "25"
      fasting = "Select"
      note = ""
      addCholesterol(date, totalSerumCholesterol, idLipo, hdLipo, tryiglycerides, fasting, note)

      date = @oneYearAgo
      totalSerumCholesterol = "249"
      idLipo = "126"
      hdLipo = "214"
      tryiglycerides = "33"
      fasting = "Select"
      note = ""
      addCholesterol(date, totalSerumCholesterol, idLipo, hdLipo, tryiglycerides, fasting, note)

      date = @twoYearAgo
      totalSerumCholesterol = "244"
      idLipo = "33"
      hdLipo = ""
      tryiglycerides = "22"
      fasting = "Yes"
      note = ""
      addCholesterol(date, totalSerumCholesterol, idLipo, hdLipo, tryiglycerides, fasting, note)

      date = @sixYearsAgo
      totalSerumCholesterol = "478"
      idLipo = "43"
      hdLipo = "25"
      tryiglycerides = "267"
      fasting = "Yes"
      note = ""
      addCholesterol(date, totalSerumCholesterol, idLipo, hdLipo, tryiglycerides, fasting, note)

    end

    it ". After the filter button is selected, the focus moves to the returned results, first row, or the No results found message." do
      verifyFocusMoveToFirstRow(4)
    end

    it ". Given that results are found, entries are displayed in a table format, columns are left aligned, sorted in reverse chronological order by date and time entered; null values are shown as empty table cells unless otherwise noted" do

      tableHeaderArr = ["Date Entered", "Total (mg/dl)", "HDL (mg/dl)", "LDL (mg/dl)", "Triglycerides (mg/dl)", "Fasting?"]
      expect(@list_view.getTableHeaders()).to eq(tableHeaderArr)

      verifyDefaultTableList

      @date_filter.startDate = @twoYearAgo
      @date_filter.endDate = @today
      @date_filter.filter
      Watir::Wait.until { @list_view.primaryHeader_element.text == @title }
      expect(@list_view.primaryHeader_element.text).to eq(@title)
      Watir::Wait.until {@list_view.tableList_elements.length == 7}
      expect(@list_view.tableList_elements.length).to eq(7)

      expect(@list_view.getTextForRowCol(5, 1).include?(@hundredDaysAgo)).to eq(true)
      expect(isDateFormatValid?(@list_view.getTextForRowCol(1, 1), "%m/%d/%Y %I:%M %p"))
      expect(@list_view.getTextForRowCol(5, 2)).to eq("293")
      expect(@list_view.getTextForRowCol(5, 3)).to eq("68")
      expect(@list_view.getTextForRowCol(5, 4)).to eq("63")
      expect(@list_view.getTextForRowCol(5, 5)).to eq("25")
      expect(@list_view.getTextForRowCol(5, 6)).to eq("")

      expect(@list_view.getTextForRowCol(6, 1).include?(@oneYearAgo)).to eq(true)
      expect(isDateFormatValid?(@list_view.getTextForRowCol(1, 1), "%m/%d/%Y %I:%M %p"))
      expect(@list_view.getTextForRowCol(6, 2)).to eq("249")
      expect(@list_view.getTextForRowCol(6, 3)).to eq("214")
      expect(@list_view.getTextForRowCol(6, 4)).to eq("126")
      expect(@list_view.getTextForRowCol(6, 5)).to eq("33")
      expect(@list_view.getTextForRowCol(6, 6)).to eq("")

      expect(@list_view.getTextForRowCol(7, 1).include?(@twoYearAgo)).to eq(true)
      expect(isDateFormatValid?(@list_view.getTextForRowCol(1, 1), "%m/%d/%Y %I:%M %p"))
      expect(@list_view.getTextForRowCol(7, 2)).to eq("244")
      expect(@list_view.getTextForRowCol(7, 3)).to eq("")
      expect(@list_view.getTextForRowCol(7, 4)).to eq("33")
      expect(@list_view.getTextForRowCol(7, 5)).to eq("22")
      expect(@list_view.getTextForRowCol(7, 6)).to eq("Yes")

      @date_filter.startDate = @sixYearsAgo
      @date_filter.endDate = @today
      @date_filter.filter
      Watir::Wait.until { @list_view.primaryHeader_element.text == @title }
      expect(@list_view.primaryHeader_element.text).to eq(@title)
      Watir::Wait.until {@list_view.tableList_elements.length == 8}
      expect(@list_view.tableList_elements.length).to eq(8)

      expect(@list_view.getTextForRowCol(8, 1).include?(@sixYearsAgo)).to eq(true)
      expect(isDateFormatValid?(@list_view.getTextForRowCol(1, 1), "%m/%d/%Y %I:%M %p"))
      expect(@list_view.getTextForRowCol(8, 2)).to eq("478")
      expect(@list_view.getTextForRowCol(8, 3)).to eq("25")
      expect(@list_view.getTextForRowCol(8, 4)).to eq("43")
      expect(@list_view.getTextForRowCol(8, 5)).to eq("267")
      expect(@list_view.getTextForRowCol(8, 6)).to eq("Yes")

      @site.resizeWindowToPhone
      tableHeaderArr = ["Date Entered", "Total (mg/dl)"]
      expect(@list_view.getTableHeaders()).to eq(tableHeaderArr)
      @site.resizeWindowToDefault

    end

    it ". If no records are found when a filter is applied the message the following message is shown in the screen area in place of the table view." do
      verifyNoResultsMessage(getDateNthDaysFromNow(-3, "%m/%d/%Y"), getDateNthDaysFromNow(-3, "%m/%d/%Y"))
    end
  end

  context "#AC CREHAB-871|TC CREAHB-912: Filter Button, Reset Button" do
    it ". Filter Button (alt text = Filter)" do
      expect(@date_filter.filter_element.text).to eq("Filter")
    end

    it ". Reset button" do
      verifyReset(4)

      expect(@list_view.getFocusedElement() == @list_view.firstTableRow_element).to eq(true)
      expect(@list_view.getTextForRowCol(1, 1).include?(@today)).to eq(true)

      verifyDefaultTableList
    end
  end

  context "#AC CREHAB-873|TC CREHAB-912: Navigation for FitHeart Cholesterol (Fitler and List View) " do
    it_behaves_like "Back button validation for Phone view"
  end

  def verifyDefaultTableList
    expect(@list_view.getTextForRowCol(1, 1).include?(@today)).to eq(true)
    expect(isDateFormatValid?(@list_view.getTextForRowCol(1, 1), "%m/%d/%Y %I:%M %p"))
    expect(@list_view.getTextForRowCol(1, 2)).to eq("355")
    expect(@list_view.getTextForRowCol(1, 3)).to eq("157")
    expect(@list_view.getTextForRowCol(1, 4)).to eq("155")
    expect(@list_view.getTextForRowCol(1, 5)).to eq("")
    expect(@list_view.getTextForRowCol(1, 6)).to eq("No")

    expect(@list_view.getTextForRowCol(2, 1).include?(@yesterday)).to eq(true)
    expect(isDateFormatValid?(@list_view.getTextForRowCol(1, 1), "%m/%d/%Y %I:%M %p"))
    expect(@list_view.getTextForRowCol(2, 2)).to eq("")
    expect(@list_view.getTextForRowCol(2, 3)).to eq("257")
    expect(@list_view.getTextForRowCol(2, 4)).to eq("255")
    expect(@list_view.getTextForRowCol(2, 5)).to eq("")
    expect(@list_view.getTextForRowCol(2, 6)).to eq("Yes")

    expect(@list_view.getTextForRowCol(3, 1).include?(@twoDaysAgo)).to eq(true)
    expect(isDateFormatValid?(@list_view.getTextForRowCol(1, 1), "%m/%d/%Y %I:%M %p"))
    expect(@list_view.getTextForRowCol(3, 2)).to eq("555")
    expect(@list_view.getTextForRowCol(3, 3)).to eq("")
    expect(@list_view.getTextForRowCol(3, 4)).to eq("")
    expect(@list_view.getTextForRowCol(3, 5)).to eq("")
    expect(@list_view.getTextForRowCol(3, 6)).to eq("")

    expect(@list_view.getTextForRowCol(4, 1).include?(@threeMonthAgo)).to eq(true)
    expect(isDateFormatValid?(@list_view.getTextForRowCol(1, 1), "%m/%d/%Y %I:%M %p"))
    expect(@list_view.getTextForRowCol(4, 2)).to eq("312")
    expect(@list_view.getTextForRowCol(4, 3)).to eq("131")
    expect(@list_view.getTextForRowCol(4, 4)).to eq("244")
    expect(@list_view.getTextForRowCol(4, 5)).to eq("123")
    expect(@list_view.getTextForRowCol(4, 6)).to eq("Yes")
  end
end
