#!/bin/env ruby
# encoding: utf-8
require 'rspec'
require 'watir-webdriver'

require_relative '../../lib/browser_container/site'
require_relative '../../lib/module/MongoUtility'
require_relative '../../lib/module/DateUtility'
require_relative '../rspec_helper'
require_relative '../shared_context'
require_relative 'health_helper'


describe '[Story #CREHAB-378: Cardiac: Glucose - Add]
          [Story #CREHAB-853: Add Duplicate Entries error message for Glucose]', :regression => true do
  include MongoUtility
  include DateUtility

  app_title = "VA FitHeart"

  before(:all) do
    removeAllDocuments()

    @driver = Watir::Browser.new
    @site = Site.new(BASE_URL, @driver)
    @login = @site.loginPage
    @eula = @site.eula
    @roa = @site.roa
    @landing = @site.landing
    @health = @site.health
    @leftNav = @site.leftMenu
    @modal = @site.modal
    @detail = @site.health_detail
    @list_view = @site.list_view
    @calendar = @site.calendar_picker
    @error = @site.error

    @today = getDateNthDaysFromNow(0, "%m/%d/%Y")

    @title = "Glucose"

    @EXPECTED_ROW_0_FORM = [\
        "* required field",
        "* Date:", "MM/DD/YYYY",
        "This calendar control is not optimized for screen readers. Please directly enter dates in the text box provided. Open Calendar Control.",
        "Day selected is " + getDayofWeek(@today),
        getDayofWeek(@today),
        "* Time:",
        "hh:mm AM/PM",
        "This time selection control is not optimized for screen readers. Please directly enter the time in the text box provided. Open Time Picker",
        "* Glucose Level (mg/dl):",
        "Between 0 and 400",
        "Tested:",
        "Unknown",
        "Fasting (8 hours)",
        "After Meal",
        "Note:",
        "250 character maximum",
        "250 characters left",
        "Notification:",
        "Status: Off",
        "",
        "To edit the Glucose notification, select the Notification button on the previous screen.",
        "Save Cancel"
        ]

  end

  after(:all) do
    @site.quitDriver
  end

  context "[Before] Log in and navigate to" do
    it ". Login and navigate to" do
      acceptEulaAndQuickLogin("zztest.patient01", "pass", "D123401")

      navigateToHealth(app_title)
      navigateTo(@title)
    end
  end

  context "#AC CREHAB-669|TC CREHAB-686: Add New Entry for FitHeart Glucose " do
    it_behaves_like "Add Button Validation"

    it_behaves_like  "Labels Validation"

    it_behaves_like "Date Validation"

    it_behaves_like "Time Validation"

    it_behaves_like "Note Validation"

  end

  context "#AC CREHAB-669|TC CREHAB-686: Add New Entry for FitHeart Glucose
           #AC CREHAB-698|TC CREHAB-851:  Update Glucose Test Field" do
    it ". Glucose Level (mg/dl) (constrain format to whole number, show light grey guidance text: Between 0 and 400) " do
      expect(@detail.glucoseLevel_element.attribute("required")).to eq("true")
      expect(@detail.glucoseLevel_element.attribute("placeholder")).to eq("Between 0 and 400")

      @detail.save

      errorMessages = ["Glucose Level (mg/dl) field is required."]
      verifyErrorMessage(errorMessages)

      @detail.glucoseLevel = 423
      @detail.save
      errorMessages = ["Glucose Level (mg/dl) is outside the expected range. Please enter a value between 0 and 400."]
      verifyErrorMessage(errorMessages)

      @detail.glucoseLevel = "Abc421"
      expect(@detail.glucoseLevel).to eq("421")
      @detail.save
      errorMessages = ["Glucose Level (mg/dl) is outside the expected range. Please enter a value between 0 and 400."]
      verifyErrorMessage(errorMessages)
    end

    it ". Tested (single choice, select list, values: Unknown (default), Fasting (8 hours), After Meal) within the Add/Edit/Delete Screen.)" do
      #story crehab-697
      expect(@detail.testedSelect).to eq("Unknown")
      testedSelectArr = ["Unknown", "Fasting (8 hours)", "After Meal"]
      expect(@detail.testedSelect_options).to eq(testedSelectArr)

      for i in 0..(testedSelectArr.length - 1)
        @detail.testedSelect = testedSelectArr[i]
        expect(@detail.testedSelect).to eq(testedSelectArr[i])
      end

    end

    it ". Standard Save and Cancel options are available." do
     expect(@detail.save_element.visible?).to eq(true)
     expect(@detail.cancel_element.visible?).to eq(true)
     @detail.cancel

     @list_view.primaryHeader_element.when_visible
     expect(@list_view.primaryHeader_element.text).to eq("Glucose")

    end

  end

  context "#AC CREHAB-1073|TC CREHAB-1076: Display error message for duplicate entires" do
    it ". Duplicate entries validation" do
      @list_view.addButton_element.when_visible
      count = @list_view.tableList_elements.length

      @list_view.addButton

      Watir::Wait.until {@detail.primaryHeader_element.text == "Add Glucose Entry"}
      expect(@detail.primaryHeader_element.text).to eq("Add Glucose Entry")

      Watir::Wait.until {@detail.date_element.visible? == true}
      @detail.date_element.when_visible

      now = getNthMinFromNow(-1)
      @time = now
      @detail.time = @time
      @detail.glucoseLevel = "258"
      @detail.testedSelect = "Unknown"

      @detail.save

      Watir::Wait.until {@list_view.tableList_elements.length == count + 1}
      Watir::Wait.until {@list_view.primaryHeader_element.text == "Glucose"}
      expect(@list_view.primaryHeader_element.text).to eq("Glucose")

      @list_view.addButton_element.when_visible
      @list_view.addButton

      Watir::Wait.until {@detail.primaryHeader_element.text == "Add Glucose Entry"}
      expect(@detail.primaryHeader_element.text).to eq("Add Glucose Entry")

      Watir::Wait.until {@detail.date_element.visible? == true}
      @detail.date_element.when_visible

      @time = now
      @detail.time = @time
      @detail.glucoseLevel = "333"
      @detail.testedSelect = "Unknown"

      @detail.save

      errorMessages = ["An entry has already been saved with the same date and time. Please change your date and/or time and save again."]
      verifyErrorMessage(errorMessages)

    end
  end

end
