#!/bin/env ruby
# encoding: utf-8

require 'rspec'
require 'watir-webdriver'

require_relative '../../lib/browser_container/site'
require_relative '../../lib/module/MongoUtility'
require_relative '../../lib/module/DateUtility'
require_relative '../rspec_helper'
require_relative '../health/health_helper'
require_relative 'health_helper'
require_relative '../shared_context'


describe '[Story #CREHAB-378: Cardiac: Glucose - Edit/Delete]', :regression => true do
  include MongoUtility
  include DateUtility

  app_title = "VA FitHeart"

  before(:all) do
    removeAllDocuments()

    @driver = Watir::Browser.new
    @site = Site.new(BASE_URL, @driver)
    @login = @site.loginPage
    @eula = @site.eula
    @roa = @site.roa
    @landing = @site.landing
    @health = @site.health
    @leftNav = @site.leftMenu
    @modal = @site.modal
    @detail = @site.health_detail
    @list_view = @site.list_view
    @calendar = @site.calendar_picker
    @error = @site.error
    @learn_more = @site.learnMore

    @today = getDateNthDaysFromNow(0, "%m/%d/%Y")
    @title = "Glucose"
  end

  after(:all) do
    @site.quitDriver
  end

  context "[Before] Log in and navigate to" do
    it ". Login and navigate to" do
      acceptEulaAndQuickLogin("zztest.patient01", "pass", "D123401")
      navigateToHealth(app_title)
      navigateTo(@title)
    end
  end

  context "#AC CREHAB-670|TC CREHAB-686: Edit an Entry for FitHeart Glucose " do
   it ". When the user selects a Glucose entry from the table view, the system displays the Glucose record in the Detail screen in Edit mode:" do
      @list_view.addButton_element.when_visible
      @list_view.addButton

      Watir::Wait.until {@detail.primaryHeader_element.text == "Add " + @title + " Entry"}
      expect(@detail.primaryHeader_element.text).to eq("Add " + @title + " Entry")

      Watir::Wait.until {@detail.date_element.visible? == true}
      @detail.date_element.when_visible

      @detail.date = @today
      @detail.glucoseLevel = "257"
      @detail.testedSelect = "Unknown"
      @detail.note = ""

      @detail.save

      Watir::Wait.until {@list_view.secondaryHeader_element.text == "Health"}
      Watir::Wait.until {@health.glucoseAmount_element.exists?}
      Watir::Wait.until {@health.glucoseAmount_element.text == "257 mg/dl"}
      expect(@health.glucoseAmount_element.text).to eq("257 mg/dl")

      @list_view.primaryHeader_element.when_visible
      Watir::Wait.until {@list_view.primaryHeader_element.text == @title}

      expect(@list_view.primaryHeader_element.text).to eq(@title)
      expect(@list_view.getFocusedElement() == @list_view.primaryHeader_element).to eq(true)

      @list_view.tableList_elements[0].when_visible
      expect(@list_view.getTextForRowCol(1, 1).include?(@today)).to eq(true)
      expect(isDateFormatValid?(@list_view.getTextForRowCol(1, 1), "%m/%d/%Y %I:%M %p"))
      expect(@list_view.getTextForRowCol(1, 2)).to eq("257")
      expect(@list_view.getTextForRowCol(1, 3)).to eq("Unknown")

      date = @today
      glucoseLevel = "257"
      tested = "Unknown"
      note = ""
      verifyGlucose(date, glucoseLevel, tested, note)

      @list_view.tableList_elements[0].when_visible
      expect(@list_view.getTextForRowCol(1, 1).include?(@today)).to eq(true)
      expect(isDateFormatValid?(@list_view.getTextForRowCol(1, 1), "%m/%d/%Y %I:%M %p"))
      expect(@list_view.getTextForRowCol(1, 2)).to eq("257")
      expect(@list_view.getTextForRowCol(1, 3)).to eq("Unknown")
   end

    it ". All values previously saved are visible. " do
      date = getDateNthDaysFromNow(-2, "%m/%d/%Y")
      glucoseLevel = "123"
      tested = "After Meal"
      note = ""
      addGlucose(date, glucoseLevel, tested, note)
      verifyGlucose(date, glucoseLevel, tested, note)

      date = getDateNthDaysFromNow(-4, "%m/%d/%Y")
      glucoseLevel = "234"
      note = "N/A"
      addGlucose(date, glucoseLevel, tested, note)
      verifyGlucose(date, glucoseLevel, tested, note)

    end

  end

  context "#AC CREHAB-672|TC CREHAB-686: Navigation for FitHeart Glucose (Add/Edit/Delete)" do
    it ". On all form factors when the user views the Add or Edit Detail screen (secondary detail page), a < is shown (alt text: Return to Previous Screen; when clicked the user is returned to the Detail Screen for the group where the filter and table list view is presented." do
      @list_view.addButton_element.when_visible
      @list_view.addButton_element.click

      Watir::Wait.until { @detail.primaryHeader_element.text == "Add " + @title + " Entry" }
      expect(@detail.primaryHeader_element.text).to eq("Add " + @title + " Entry")

      @detail.date_element.when_visible
      @detail.date = @today
      @detail.glucoseLevel = "332"
      @detail.testedSelect = "Unknown"
      @detail.note = ""

      expect(@detail.backButtonIcon_element.exists?).to eq(true)
      expect(@detail.backButton_element.attribute("title")).to eq("Return to Previous Screen")

    end

    it ". If a user selects the < symbol and there are unsaved changes on the screen, the system will prompt the user to save their changes via a modal:" do
      verifyBackButtonConfirmationPrompt()
    end

    it ". If the user tries to Navigate away from a detail record where a field has been updated without Saving, the user is prompted to confirm that they want to leave the record without saving via a modal:" do
      Watir::Wait.until {@list_view.tableList_elements[0].visible? }
      @list_view.tableList_elements[0].when_visible
      @list_view.tableList_elements[0].click

      Watir::Wait.until { @detail.primaryHeader_element.text == "Edit " + @title + " Entry" }
      expect(@detail.primaryHeader_element.text).to eq("Edit " + @title + " Entry")

      @detail.date_element.when_visible
      Watir::Wait.until {  @detail.date == @today.to_s }

      date = @detail.date
      glucoseLevel = @detail.glucoseLevel
      tested = @detail.testedSelect
      note = @detail.note

      @detail.date = getDateNthDaysFromNow(-2, "%m/%d/%Y")
      @detail.glucoseLevel = "199"
      @detail.testedSelect = "After Meal"
      @detail.note = "N/A"

      navigateAwayAndVerifyConfirmationPrompt
      navigateTo(@title)

      verifyGlucose(date, glucoseLevel, tested, note)
    end

  end

  context "#AC CREHAB-671|TC CREHAB-686: Save, Cancel for FitHeart Glucose" do
    it ". Cancel - When a user selects Cancel, no updates are made to the record and the user is shown the table view with the focus placed on the header" do
      Watir::Wait.until { @list_view.primaryHeader_element.text == @title}
      expect(@list_view.primaryHeader_element.text).to eq(@title)
      expect(@list_view.getFocusedElement == @list_view.primaryHeader_element).to eq(true)

    end

    it ". Save - When a user selects Save the system saves the record and the user is returned to the table view, with the focus on the header. Updates made to the record are visible in the table, if included in that view." do
      #this is covered in Glucose_ADD_CREHAB-378.rb
    end

  end

  context "#AC CREHAB-671|TC CREHAB-686: Delete for FitHeart Glucose " do
    it_behaves_like "Delete Validation"
  end

  context "#AC CREHAB-671|TC CREHAB-686: Soft Delete  for FitHeart Glucose " do
    it ". All deletes are soft deletes, meaning they are removed from the user’s screen but remain in the database for audit purposes." do
      expect(verifyDocumentsDeleted("chemlabs", "medicalhistorydb", "EDIPI", "D123401", "deletedFlag")).to eq(true)
    end
  end


end
