#!/bin/env ruby
# encoding: utf-8
require 'rspec'
require 'watir-webdriver'

require_relative '../../lib/browser_container/site'
require_relative '../../spec/rspec_helper'
require_relative '../../lib/module/DateUtility'
require_relative '../../lib/module/MongoUtility'
require_relative '../health/health_helper'
require_relative '../shared_context'

describe '[Story #CREHAB-377: Glucose - Filter and Table List View]
          [Story #CREHAB-697: Update Glucose Test Field]
          [Story #CREHAB-695: Update Health features to use latest endpoint]
          [Story #CREHAB-721:Update Health Navigation (Glucose & glucose) to include latest values]
          [Story #CREAHB-379: Glucose - Graph]
          [Story #CREAHB-514: Phone User : Options control]', :regression => true do
  include DateUtility
  include MongoUtility
  app_title = "VA FitHeart"

  before(:all) do
    removeAllDocuments()

    @driver = Watir::Browser.new
    @site = Site.new(BASE_URL, @driver)
    @login = @site.loginPage
    @eula = @site.eula
    @roa = @site.roa
    @landing = @site.landing
    @health = @site.health
    @leftNav = @site.leftMenu
    @modal = @site.modal
    @detail = @site.health_detail
    @list_view = @site.list_view
    @calendar = @site.calendar_picker
    @error = @site.error
    @date_filter = @site.date_filter
    @notification_detail = @site.notification_detail

    @title = "Glucose"
    @today = getDateNthDaysFromNow(0, "%m/%d/%Y")
    @yesterday = getDateNthDaysFromNow(-1, "%m/%d/%Y")
    @tomorrow = getDateNthDaysFromNow(1, "%m/%d/%Y")
    @twoDaysAgo = getDateNthDaysFromNow(-2, "%m/%d/%Y")
    @threeMonthAgo = getDateNthMonthsFromNow(-3, "%m/%d/%Y")
    @sixYearsAgo = getDateNthYearsFromNow(-6, "%m/%d/%Y")
    @hundredDaysAgo = getDateNthDaysFromNow(-100, "%m/%d/%Y")
    @oneYearAgo = getDateNthYearsFromNow(-1, "%m/%d/%Y")
    @twoYearAgo = getDateNthYearsFromNow(-2, "%m/%d/%Y")
    @sixYearsAndOneDayAgo = getDateNthDaysFromGivenDate(@sixYearsAgo, -1)
  end

  after(:all) do
    @site.quitDriver
  end

  context "[Before] Log in and navigate to" do
    it ". Login and navigate to" do
      acceptEulaAndQuickLogin("zztest.patient01", "pass", "D123401")

      expect(@landing.appTitle).to eq(app_title)
      @landing.welcomeTitleForAuthenticatedUser_element.when_visible
      expect(@landing.welcomeTitleForAuthenticatedUser).to eq("Welcome to VA FitHeart")

      @leftNav.leftMenu_element.when_visible
      @leftNav.leftMenu
    end
  end

  context "#AC CREHAB-664|TC CREJAB-685: Sub Feature Selection for FitHeart Glucose" do
    it ". When a user opens the Health feature, the system displays the Glucose tracker in the Main screen (column 2):" do
      @leftNav.health_element.when_visible
      @leftNav.health

      @health.primaryHeader_element.when_visible
      expect(@health.primaryHeader_element.text).to eq("Health")
      expect(@health.secondaryHeader_element.text).to eq("Health")

      navigateTo(@title)
    end

    it ". List Location: 3rd on the list, below Glucose " do
      #this scenario is covered in Health_Feature_options_CREHAB-376.rb
    end

    it ". Hand icon displayed on left side of button (icon included in wiki:" do
      expect(@health.handIconForGlucose_element.visible?).to eq(true)
    end
  end

  context "#AC CREHAB-855|TC CREHAB-906: Glucose Navigation for FitHeart Update
           #AC CREHAB-854|TC CREHAb-907: Use latest endpoint to retrieve values" do
    it ". The date of the most recent tracker entry is displayed below the Glucose label. If date = 'today' display 'Today' if date = Today-1 display 'Yesterday', otherwise display the actual date (MM/DD/YYYY).  The most recent value is displayed as center aligned with other values in the far right column (format: as <glucose level value> mg/dl)" do
      twoDaysAgo = getDateNthDaysFromNow(-2, "%m/%d/%Y")
      glucoseLevel = "123"
      tested = "After Meal"
      note = ""
      addGlucose(twoDaysAgo, glucoseLevel, tested, note)
      Watir::Wait.until { @list_view.primaryHeader_element.text == "Glucose" }
      expect(@list_view.getFocusedElement() == @list_view.primaryHeader_element).to eq(true)
      expect(@health.glucoseDate_element.text).to eq(twoDaysAgo)
      expect(@health.glucoseAmount_element.text).to eq("123 mg/dl")

      yesterday = getDateNthDaysFromNow(-1, "%m/%d/%Y")
      glucoseLevel = "221"
      tested = "Fasting (8 hours)"
      note = ""
      addGlucose(yesterday, glucoseLevel, tested, note)
      Watir::Wait.until { @list_view.primaryHeader_element.text == "Glucose" }
      expect(@list_view.getFocusedElement == @list_view.primaryHeader_element).to eq(true)
      expect(@health.glucoseDate_element.text).to eq("Yesterday")
      expect(@health.glucoseAmount_element.text).to eq("221 mg/dl")

      today = getDateNthDaysFromNow(0, "%m/%d/%Y")
      glucoseLevel = "226"
      tested = "Fasting (8 hours)"
      note = ""
      addGlucose(today, glucoseLevel, tested, note)
      Watir::Wait.until { @list_view.primaryHeader_element.text == "Glucose" }
      expect(@list_view.getFocusedElement() == @list_view.primaryHeader_element).to eq(true)
      expect(@health.glucoseDate_element.text).to eq("Today")
      expect(@health.glucoseAmount_element.text).to eq("226 mg/dl")
    end

  end

  context "#AC CREHAB-698|TC CREHAB-851: Modify Test Condition Values " do
    it ". On the Filter/List view screen: Update the order of the 'Tested' field drop down to Select, Unknown, Fasting (8 hours), After Meal" do
      testedSelectArr = ["Select", "Unknown", "Fasting (8 hours)", "After Meal"]
      expect(@list_view.testedSelect_options).to eq(testedSelectArr)
    end
  end

  context "#AC CREHAB-1186|TC CREHAB-1195: Phone View Option Button
           #AC CREAHB-1187|TC CREHAB-1195: Options control display for Phone user" do
    include_examples "Phone User: Options control"
  end

  context "#AC CREHAB-665|TC CREHAB-685: Sub Feature Default View for FitHeart Glucose  " do
    include_examples "Date Filter Validation"
  end

  context "#AC CREHAB-666|TC CREHAB-685: Filter Action and Results Display for Glucose" do
    it ". When a user selects the filter button, the system updates the table view to show records that match the filter options entered.
        . Given that results are found, entries are displayed in a table format, sorted in reverse chronological order by date and time entered; null values are shown as empty table cells unless otherwise noted" do

      removeCollection("chemlabs", "medicalhistorydb")
      @site.refreshBrowser
      Watir::Wait.until { @list_view.primaryHeader_element.text == "Glucose" }
      expect(@list_view.primaryHeader_element.text).to eq("Glucose")

      @date_filter.startDate = @sixYearsAgo
      @date_filter.endDate = @today
      @date_filter.filter

      Watir::Wait.until { @list_view.primaryHeader_element.text == "Glucose" }
      expect(@list_view.primaryHeader_element.text).to eq("Glucose")

      today = getDateNthDaysFromNow(0, "%m/%d/%Y")
      glucoseLevel = "123"
      tested = "After Meal"
      note = ""
      addGlucose(today, glucoseLevel, tested, note)

      threeDaysAgo = getDateNthDaysFromNow(-3, "%m/%d/%Y")
      glucoseLevel = "342"
      tested = "Fasting (8 hours)"
      note = ""
      addGlucose(threeDaysAgo, glucoseLevel, tested, note)

      oneMonthAgo = getDateNthMonthsFromNow(-1, "%m/%d/%Y")
      glucoseLevel = "332"
      tested = "After Meal"
      note = ""
      addGlucose(oneMonthAgo, glucoseLevel, tested, note)

      fourtyFiveDaysAgo =  getDateNthDaysFromNow(-45, "%m/%d/%Y")
      glucoseLevel = "298"
      tested = "Fasting (8 hours)"
      note = ""
      addGlucose(fourtyFiveDaysAgo, glucoseLevel, tested, note)

      twoMonthAgo = getDateNthMonthsFromNow(-2, "%m/%d/%Y")
      glucoseLevel = "211"
      tested = "Unknown"
      note = ""
      addGlucose(twoMonthAgo, glucoseLevel, tested, note)

      oneYearAgo = getDateNthYearsFromNow(-1, "%m/%d/%Y")
      glucoseLevel = "287"
      tested = "Unknown"
      note = ""
      addGlucose(oneYearAgo, glucoseLevel, tested, note)

      twoYearsAgo = getDateNthYearsFromNow(-2, "%m/%d/%Y")
      glucoseLevel = "265"
      tested = "Fasting (8 hours)"
      note = ""
      addGlucose(twoYearsAgo, glucoseLevel, tested, note)

      glucoseLevel = "121"
      tested = "After Meal"
      note = ""
      addGlucose(@sixYearsAgo, glucoseLevel, tested, note)

      @date_filter.reset
      #reset to 3 months range
      Watir::Wait.until { @list_view.primaryHeader_element.text == "Glucose" }
      expect(@list_view.primaryHeader_element.text).to eq("Glucose")
      Watir::Wait.until {@list_view.tableList_elements.length == 5}
      expect(@list_view.tableList_elements.length).to eq(5)

      tableHeaderArr = ["Date Entered", "Glucose Level (mg/dl)", "Tested"]
      expect(@list_view.getTableHeaders()).to eq(tableHeaderArr)

      verifyDefaultTableList

      @date_filter.startDate = twoYearsAgo
      @date_filter.endDate = twoMonthAgo
      @date_filter.filter
      Watir::Wait.until { @list_view.primaryHeader_element.text == "Glucose" }
      expect(@list_view.primaryHeader_element.text).to eq("Glucose")
      Watir::Wait.until {@list_view.tableList_elements.length == 3}
      expect(@list_view.tableList_elements.length).to eq(3)

      tableHeaderArr = ["Date Entered", "Glucose Level (mg/dl)", "Tested"]
      expect(@list_view.getTableHeaders()).to eq(tableHeaderArr)
      expect(@list_view.getTextForRowCol(1, 1).include?(twoMonthAgo)).to eq(true)
      expect(isDateFormatValid?(@list_view.getTextForRowCol(1, 1), "%m/%d/%Y %I:%M %p"))
      expect(@list_view.getTextForRowCol(1, 2)).to eq("211")
      expect(@list_view.getTextForRowCol(1, 3)).to eq("Unknown")

      expect(@list_view.getTextForRowCol(2, 1).include?(oneYearAgo)).to eq(true)
      expect(isDateFormatValid?(@list_view.getTextForRowCol(2, 1), "%m/%d/%Y %I:%M %p"))
      expect(@list_view.getTextForRowCol(2, 2)).to eq("287")
      expect(@list_view.getTextForRowCol(2, 3)).to eq("Unknown")

      expect(@list_view.getTextForRowCol(3, 1).include?(twoYearsAgo)).to eq(true)
      expect(isDateFormatValid?(@list_view.getTextForRowCol(3, 1), "%m/%d/%Y %I:%M %p"))
      expect(@list_view.getTextForRowCol(3, 2)).to eq("265")
      expect(@list_view.getTextForRowCol(3, 3)).to eq("Fasting (8 hours)")

      @date_filter.startDate = @sixYearsAgo
      @date_filter.endDate = today
      @date_filter.filter
      Watir::Wait.until { @list_view.primaryHeader_element.text == "Glucose" }
      expect(@list_view.primaryHeader_element.text).to eq("Glucose")

      Watir::Wait.until {@list_view.tableList_elements.length == 8}
      expect(@list_view.tableList_elements.length).to eq(8)

      tableHeaderArr = ["Date Entered", "Glucose Level (mg/dl)", "Tested"]
      expect(@list_view.getTableHeaders()).to eq(tableHeaderArr)
      expect(@list_view.getTextForRowCol(6, 1).include?(oneYearAgo)).to eq(true)
      expect(isDateFormatValid?(@list_view.getTextForRowCol(2, 1), "%m/%d/%Y %I:%M %p"))
      expect(@list_view.getTextForRowCol(6, 2)).to eq("287")
      expect(@list_view.getTextForRowCol(6, 3)).to eq("Unknown")

      expect(@list_view.getTextForRowCol(7, 1).include?(twoYearsAgo)).to eq(true)
      expect(isDateFormatValid?(@list_view.getTextForRowCol(3, 1), "%m/%d/%Y %I:%M %p"))
      expect(@list_view.getTextForRowCol(7, 2)).to eq("265")
      expect(@list_view.getTextForRowCol(7, 3)).to eq("Fasting (8 hours)")

      expect(@list_view.getTextForRowCol(8, 1).include?(@sixYearsAgo)).to eq(true)
      expect(isDateFormatValid?(@list_view.getTextForRowCol(3, 1), "%m/%d/%Y %I:%M %p"))
      expect(@list_view.getTextForRowCol(8, 2)).to eq("121")
      expect(@list_view.getTextForRowCol(8, 3)).to eq("After Meal")

    end

    it ". 2.2.4. Test Condition (single choice, select list, values: Select (default), Fasting (8 hours), After Meal, Unknown)" do
      @list_view.testedSelect = "Fasting (8 hours)"
      @date_filter.filter
      Watir::Wait.until { @list_view.primaryHeader_element.text == "Glucose" }
      expect(@list_view.primaryHeader_element.text).to eq("Glucose")

      Watir::Wait.until {@list_view.tableList_elements.length == 3}
      expect(@list_view.tableList_elements.length).to eq(3)
      expect(@list_view.getTextForRowCol(1, 3)).to eq("Fasting (8 hours)")
      expect(@list_view.getTextForRowCol(2, 3)).to eq("Fasting (8 hours)")
      expect(@list_view.getTextForRowCol(3, 3)).to eq("Fasting (8 hours)")

      @list_view.testedSelect = "After Meal"
      @date_filter.filter
      Watir::Wait.until { @list_view.primaryHeader_element.text == "Glucose" }
      expect(@list_view.primaryHeader_element.text).to eq("Glucose")

      Watir::Wait.until {@list_view.getTextForRowCol(1, 3) == "After Meal"}
      expect(@list_view.tableList_elements.length).to eq(3)
      expect(@list_view.getTextForRowCol(1, 3)).to eq("After Meal")
      expect(@list_view.getTextForRowCol(2, 3)).to eq("After Meal")
      expect(@list_view.getTextForRowCol(3, 3)).to eq("After Meal")

      @list_view.testedSelect = "Unknown"
      @date_filter.filter
      Watir::Wait.until { @list_view.primaryHeader_element.text == "Glucose" }
      expect(@list_view.primaryHeader_element.text).to eq("Glucose")

      Watir::Wait.until {@list_view.tableList_elements.length == 2}
      expect(@list_view.tableList_elements.length).to eq(2)
      expect(@list_view.getTextForRowCol(1, 3)).to eq("Unknown")
      expect(@list_view.getTextForRowCol(2, 3)).to eq("Unknown")

      #hide tested column on phone view
      @site.resizeWindowToPhone
      tableHeaderArr = ["Date Entered", "Glucose Level (mg/dl)"]
      expect(@list_view.getTableHeaders()).to eq(tableHeaderArr)

      @site.resizeWindowToDefault
    end

    it ". After the filter button is selected, the focus moves to the returned results, first row, or the No results found message." do
      verifyFocusMoveToFirstRow(1)
      verifyNoResultsMessage(getDateNthDaysFromNow(-3, "%m/%d/%Y"), getDateNthDaysFromNow(-3, "%m/%d/%Y"))
    end
  end

  context "#AC CREHAB-668|TC CREHAB-685: Reset Filter Functionality for FitHeart Glucose(Filter and List View)" do
    it ".  When a user views a filter control, a Reset button is available located to the right of the Filter Button. (Alt text Reset)" do
      expect(@date_filter.reset_element.visible?).to eq(true)
      expect(@date_filter.reset_element.text).to eq("Reset")
    end

    it ". When the user selects the reset button, all filter field defaults are restored and the filter action is applied so that the table list view is updated per the default filter settings." do
      @list_view.testedSelect = "After Meal"
      verifyReset(5)
      verifyDefaultTableList
    end
  end

  context "#AC CREHAB-858|TC CREHAB-903: Glucose Button & Features for Fitheart " do
    include_examples "Graph/Table Button Validation"
  end

  context "#AC CREHAB-667|TC CREHAB-685: Navigation for FitHeart Glucose (Filter nand List View)" do
    include_examples "Back button validation for Phone view"
  end

  def verifyDefaultTableList
    expect(@list_view.getTextForRowCol(1, 1).include?(@today)).to eq(true)
    expect(isDateFormatValid?(@list_view.getTextForRowCol(1, 1), "%m/%d/%Y %I:%M %p"))
    expect(@list_view.getTextForRowCol(1, 2)).to eq("123")
    expect(@list_view.getTextForRowCol(1, 3)).to eq("After Meal")

    expect(@list_view.getTextForRowCol(2, 1).include?(getDateNthDaysFromNow(-3, "%m/%d/%Y"))).to eq(true)
    expect(isDateFormatValid?(@list_view.getTextForRowCol(2, 1), "%m/%d/%Y %I:%M %p"))
    expect(@list_view.getTextForRowCol(2, 2)).to eq("342")
    expect(@list_view.getTextForRowCol(2, 3)).to eq("Fasting (8 hours)")

    expect(@list_view.getTextForRowCol(3, 1).include?(getDateNthMonthsFromNow(-1, "%m/%d/%Y"))).to eq(true)
    expect(isDateFormatValid?(@list_view.getTextForRowCol(3, 1), "%m/%d/%Y %I:%M %p"))
    expect(@list_view.getTextForRowCol(3, 2)).to eq("332")
    expect(@list_view.getTextForRowCol(3, 3)).to eq("After Meal")

    expect(@list_view.getTextForRowCol(4, 1).include?(getDateNthDaysFromNow(-45, "%m/%d/%Y"))).to eq(true)
    expect(isDateFormatValid?(@list_view.getTextForRowCol(4, 1), "%m/%d/%Y %I:%M %p"))
    expect(@list_view.getTextForRowCol(4, 2)).to eq("298")
    expect(@list_view.getTextForRowCol(4, 3)).to eq("Fasting (8 hours)")

    expect(@list_view.getTextForRowCol(5, 1).include?(getDateNthMonthsFromNow(-2, "%m/%d/%Y"))).to eq(true)
    expect(isDateFormatValid?(@list_view.getTextForRowCol(5, 1), "%m/%d/%Y %I:%M %p"))
    expect(@list_view.getTextForRowCol(5, 2)).to eq("211")
    expect(@list_view.getTextForRowCol(5, 3)).to eq("Unknown")
  end


end


