require 'rspec'
require 'watir-webdriver'

require_relative '../../lib/browser_container/site'
require_relative '../../spec/rspec_helper'
require_relative '../../lib/module/DateUtility'
require_relative '../../lib/module/MongoUtility'
require_relative '../health/health_helper'

describe '[Story #CREHAB-720: Ensure Health Data is saved with Units]', :acceptance => true do
  include DateUtility
  include MongoUtility
  app_title = "VA FitHeart"

  before(:all) do
    @driver = Watir::Browser.new
    @site = Site.new(BASE_URL, @driver)

    @login = @site.loginPage
    @eula = @site.eula
    @roa = @site.roa
    @landing = @site.landing
    @leftNav = @site.leftMenu
    @modal = @site.modal
    @health = @site.health
    @detail = @site.health_detail
    @list_view = @site.list_view
    @select_mood = @site.select_mood

    @today = getDateNthDaysAgo(0, "%m/%d/%Y")

  end

  after(:all) do
    @site.quitDriver
  end

  context "[Before] Log in and navigate to" do
    it ". Login and navigate to" do
      acceptEulaAndQuickLogin("zztest.patient01", "pass", "D123401")
      navigateToHealth(app_title)
    end
  end

  context "Ensure that the units are saved along with the value" do
    it ". Weight" do
      removeCollection("vitalsigns", "medicalhistorydb")

      navigateTo("Weight")

      weight = "160.0"
      note = ""
      addWeight(@today, weight, note)

      values = retrieveThisFieldInDocument("vitalsigns", "medicalhistorydb", "EDIPI", "D123401", "valueQuantity")

      expect(values['value']).to eq("160.0")
      expect(values['units']).to eq("lbs")
    end

    it ". Blood Pressure & Pulse" do
      removeCollection("vitalsigns", "medicalhistorydb")
      removeCollection("vitalsignsbundles", "medicalhistorydb")
      navigateTo("Blood Pressure & Pulse")
      systolic = "120"
      diastolic = "70"
      pulse = "60"
      position = "Sitting"
      note = "Testing"
      addBP(@today, systolic, diastolic, pulse, position, note)

      quantities = retrieveThisFieldInDocumentWithMultiRowsAsArray("vitalsigns", "medicalhistorydb", "EDIPI", "D123401", "valueQuantity")

      expect(quantities).to include({"value"=>"120", "units"=>"mmHg"})
      expect(quantities).to include({"value"=>"70", "units"=>"mmHg"})
      expect(quantities).to include({"value"=>"60", "units"=>"bpm"})

    end

    it ". Glucose" do
      removeCollection("chemlabs", "medicalhistorydb")
      navigateTo("Glucose")

      glucoseLevel = "226"
      tested = "Fasting (8 hours)"
      note = ""
      addGlucose(@today, glucoseLevel, tested, note)

      values = retrieveThisFieldInDocument("chemlabs", "medicalhistorydb", "EDIPI", "D123401", "valueQuantity")

      expect(values['value']).to eq("226")
      expect(values['units']).to eq("mg/dl")
    end

    it ". Cholesterol" do
      removeCollection("chemlabs", "medicalhistorydb")
      removeCollection("chemlabsbundles", "medicalhistorydb")

      navigateTo("Cholesterol")

      totalSerumCholesterol = "250"
      ldLipo = "200"
      hdLipo = "70"
      tryiglycerides = "110"
      fasting = "Yes"
      note = "Testing Cholesterol"
      addCholesterol(@today, totalSerumCholesterol, ldLipo, hdLipo, tryiglycerides, fasting, note)

      quantities = retrieveThisFieldInDocumentWithMultiRowsAsArray("chemlabs", "medicalhistorydb", "EDIPI", "D123401", "valueQuantity")
      expect(quantities).to include({"value"=>"250", "units"=>"mg/dl"})
      expect(quantities).to include({"value"=>"200", "units"=>"mg/dl"})
      expect(quantities).to include({"value"=>"70", "units"=>"mg/dl"})
      expect(quantities).to include({"value"=>"110", "units"=>"mg/dl"})
    end

    it ". Mood" do
      removeCollection("mood", "medicalhistorydb")

      navigateTo("Mood")

      moodRateSlide = 3
      moveLeftOrRight = "right"
      smileyFaceIcon = HealthDetail::SMILEY_FACE_GOOD
      addMood(@today, moodRateSlide, moveLeftOrRight, smileyFaceIcon)

      quantities = retrieveThisFieldInDocument("mood", "medicalhistorydb", "EDIPI", "D123401", "valueQuantity")
      expect(quantities).to eq({"value"=>"8", "units"=>"NA"})
      expect(quantities['value']).to eq('8')
      expect(quantities['units']).to eq('NA')

    end
  end


end