#!/bin/env ruby
# encoding: utf-8
require 'rspec'
require 'watir-webdriver'

require_relative '../../lib/browser_container/site'
require_relative '../../spec/rspec_helper'
require_relative '../../lib/module/DateUtility'
require_relative '../../lib/module/MongoUtility'
require_relative '../health/health_helper'
require_relative '../shared_context'

describe '[Story #CREHAB-406: Mood - Edit/Delete]', :regression => true do
  include DateUtility
  include MongoUtility
  app_title = "VA FitHeart"

  before(:all) do
    removeAllDocuments()

    @driver = Watir::Browser.new
    @site = Site.new(BASE_URL, @driver)

    @login = @site.loginPage
    @eula = @site.eula
    @roa = @site.roa
    @landing = @site.landing
    @leftNav = @site.leftMenu
    @modal = @site.modal
    @health = @site.health
    @list_view = @site.list_view
    @detail = @site.health_detail
    @select_mood = @site.select_mood
    @error = @site.error
    @calendar = @site.calendar_picker
    @learn_more = @site.learnMore

    @title = "Mood"
    @today = getDateNthDaysAgo(0, "%m/%d/%Y")
  end

  after(:all) do
    @site.quitDriver
  end

  context "[Before] Log in and navigate to" do
    it ". Login and navigate to" do
      acceptEulaAndQuickLogin("zztest.patient01", "pass", "D123401")
      navigateToHealth(app_title)
      navigateTo(@title)
    end
  end

  context "#AC CREHAB-887|CREHAB-905: Edit Entry for FitHeart Mood" do
    it ". When the user selects a Mood entry from the table view, the system displays the record in the Detail screen in Edit mode:" do
      date = getDateNthDaysFromNow(-2, "%m/%d/%Y")
      moodRateSlide = 3
      moveLeftOrRight = "right"
      smileyFaceIcon = HealthDetail::SMILEY_FACE_GOOD
      addMood(date, moodRateSlide, moveLeftOrRight, smileyFaceIcon)
      verifyMood(date, moodRateSlide, moveLeftOrRight, smileyFaceIcon)


      date = getDateNthDaysFromNow(-4, "%m/%d/%Y")
      moodRateSlide = 1
      moveLeftOrRight = "left"
      smileyFaceIcon = HealthDetail::SMILEY_FACE_BAD
      addMood(date, moodRateSlide, moveLeftOrRight, smileyFaceIcon)
      verifyMood(date, moodRateSlide, moveLeftOrRight, smileyFaceIcon)

      date = getDateNthDaysFromNow(-5, "%m/%d/%Y")
      moodRateSlide = 4
      moveLeftOrRight = "left"
      smileyFaceIcon = HealthDetail::SMILEY_FACE_VERYBAD
      addMood(date, moodRateSlide, moveLeftOrRight, smileyFaceIcon)
      verifyMood(date, moodRateSlide, moveLeftOrRight, smileyFaceIcon)

      date = getDateNthDaysFromNow(-8, "%m/%d/%Y")
      moodRateSlide = 5
      moveLeftOrRight = "right"
      smileyFaceIcon = HealthDetail::SMILEY_FACE_VERYGOOD
      addMood(date, moodRateSlide, moveLeftOrRight, smileyFaceIcon)
      verifyMood(date, moodRateSlide, moveLeftOrRight, smileyFaceIcon)

    end

  end

  context "#AC CREHAB-889|TC CREHAB-905: Navigation for FitHeart Mood (Add/Edit/Delete)" do
    it ". On all form factors when the user views the Add or Edit Detail screen (secondary detail page), a < is shown (alt text: Return to Previous Screen; when clicked the user is returned to the Detail Screen for the group where the filter and table list view is presented." do
      @list_view.addButton_element.when_visible
      @list_view.addButton_element.click

      Watir::Wait.until { @detail.primaryHeader_element.text == "Add Mood Entry" }
      expect(@detail.primaryHeader_element.text).to eq("Add Mood Entry")

      @detail.date_element.when_visible
      @detail.date = @today
      @detail.note = "Test navigation for mood"

      expect(@detail.backButtonIcon_element.exists?).to eq(true)
      expect(@detail.backButton_element.attribute("title")).to eq("Return to Previous Screen")
    end

    it ". If a user selects the < symbol and there are unsaved changes on the screen, the system will prompt the user to save their changes via a modal:" do
      verifyBackButtonConfirmationPrompt()
    end

    it ". If the user tries to Navigate away from a detail record where a field has been updated without Saving, the user is prompted to confirm that they want to leave the record without saving via a modal:" do
      Watir::Wait.until {@list_view.tableList_elements[0].visible? }
      @list_view.tableList_elements[0].when_visible
      @list_view.tableList_elements[0].click

      Watir::Wait.until { @detail.primaryHeader_element.text == "Edit Mood Entry" }
      expect(@detail.primaryHeader_element.text).to eq("Edit Mood Entry")

      @detail.date_element.when_visible
      Watir::Wait.until { @detail.date == getDateNthDaysFromNow(-2, "%m/%d/%Y")}

      date = @detail.date
      moodRate = 8
      note = @detail.note

      @detail.date = getDateNthDaysFromNow(-2, "%m/%d/%Y")

      @detail.selectMood_element.when_visible
      @detail.selectMood

      Watir::Wait.until {@select_mood.modalTitle_element.text== "Select Mood"}
      expect(@select_mood.modalTitle_element.text).to eq("Select Mood")

      @select_mood.moveMoodRangeToLeft(2)
      @select_mood.ok3

      Watir::Wait.until {@detail.moodRate_element.text ==  "6 / 10"}
      expect(@detail.moodRate_element.text).to eq("6 / 10")

      @detail.note = "N/A"

      navigateAwayAndVerifyConfirmationPrompt()
      navigateTo(@title)

      @list_view.tableList_elements[0].when_visible
      @list_view.tableList_elements[0].click

      @detail.primaryHeader_element.when_visible

      Watir::Wait.until {@detail.primaryHeader_element.text == "Edit " + @title + " Entry"}
      expect(@detail.primaryHeader_element.text).to eq("Edit " + @title +" Entry")

      @detail.date_element.when_visible
      expect(@detail.date).to eq(date)
      expect(@detail.moodRate_element.text).to eq(moodRate.to_s + " / 10")
      expect(@detail.note).to eq(note)

    end

  end

  context "#AC CREHAB-888|TC CREHAB-905: Save, Cancel for FitHeart Mood" do
    it ". Cancel - When a user selects Cancel, no updates are made to the record and the user is shown the table view with the focus placed on the header" do
      clickCancelAndFocusToHeader
    end

    it ". Save - When a user selects Save the system saves the record and the user is returned to the table view, with the focus on the header. Updates made to the record are visible in the table, if included in that view." do
      #this is covered in #AC CREHAB-887 test script
    end
  end

  context "#AC CREHAB-888|TC CREHAB-905:  Delete for FitHeart Mood" do
    it_behaves_like "Delete Validation"
  end

  context "#AC CREHAB-888|TC CREHAB-905: Soft Delete for FitHeart Mood" do
    it ". All deletes are soft deletes, meaning they are removed from the user’s screen but remain in the database for audit purposes." do
      expect(verifyDocumentsDeleted("mood", "medicalhistorydb", "EDIPI", "D123401", "deletedFlag")).to eq(true)
    end

  end

end
