#!/bin/env ruby
# encoding: utf-8
require 'rspec'
require 'watir-webdriver'

require_relative '../../lib/browser_container/site'
require_relative '../../spec/rspec_helper'
require_relative '../../lib/module/DateUtility'
require_relative '../../lib/module/MongoUtility'
require_relative '../../spec/shared_context'
require_relative '../health/health_helper'


describe '[Story #CREHAB-403: Weight - Filter and Table List View]
          [Story #CREHAB-407: Weigth - Graph]
          [Story #CREHAB-721: Update Health Navigation (Glucose & Weight) to include latest values
          [Story #CREHAB-695: Update Health feature to use latest endpoint]
          [Story #CREHAB-514: Phone User : Options control]', :regression => true do
  include DateUtility
  include MongoUtility
  app_title = "VA FitHeart"

  before(:all) do
    removeAllDocuments()

    @driver = Watir::Browser.new
    @site = Site.new(BASE_URL, @driver)

    @login = @site.loginPage
    @eula = @site.eula
    @roa = @site.roa
    @landing = @site.landing
    @leftNav = @site.leftMenu
    @modal = @site.modal
    @health = @site.health
    @list_view = @site.list_view
    @detail = @site.health_detail
    @error = @site.error
    @calendar = @site.calendar_picker
    @date_filter = @site.date_filter
    @notification_detail = @site.notification_detail

    @today = getDateNthDaysFromNow(0, "%m/%d/%Y")
    @tomorrow = getDateNthDaysFromNow(1, "%m/%d/%Y")
    @threeMonthAgo = getDateNthMonthsFromNow(-3, "%m/%d/%Y")
    @sixYearsAgo = getDateNthYearsFromNow(-6, "%m/%d/%Y")
    @hundredDaysAgo = getDateNthDaysFromNow(-100, "%m/%d/%Y")
    @sixYearsAndOneDayAgo = getDateNthDaysFromGivenDate(@sixYearsAgo, -1)
  end

  after(:all) do
    @site.quitDriver
  end

  context "[Before] Log in and navigate to" do
    it ". Login and navigate to" do
      acceptEulaAndQuickLogin("zztest.patient01", "pass", "D123401")

      expect(@landing.appTitle).to eq(app_title)
      @landing.welcomeTitleForAuthenticatedUser_element.when_visible
      expect(@landing.welcomeTitleForAuthenticatedUser).to eq("Welcome to VA FitHeart")

      @leftNav.leftMenu_element.when_visible
      @leftNav.leftMenu
    end
  end

  context "#AC CREHAB-654|TC CREJAB-682: Sub Feature Selection for FitHeart Weight" do
    it ". When a user opens the Health feature, the system displays Weight in the Main screen (column 2):" do
      @leftNav.health_element.when_visible
      @leftNav.health

      @health.primaryHeader_element.when_visible
      expect(@health.primaryHeader_element.text).to eq("Health")
      expect(@health.secondaryHeader_element.text).to eq("Health")

      @health.weight_element.when_visible
      @health.weight_element.click

      Watir::Wait.until {@list_view.secondaryHeader_element.text == "Health"}
      Watir::Wait.until { @list_view.primaryHeader_element.text == "Weight" }

      expect(@list_view.primaryHeader_element.text).to eq("Weight")
      expect(@list_view.secondaryHeader_element.text).to eq("Health")
    end

    it ". List Location (if other sub features are present): 2nd item, below Blood Pressure & Pulse " do
      #this scenario is covered in Health_Feature_options_CREHAB-376.rb
    end

    it ". Apple icon is displayed as left-justified on the button" do
      expect(@health.appleIconForWeight_element.visible?).to eq(true)
    end

  end

  context "#AC CREHAB-855|TC CREHAB-906: Weight Navigation for FitHeart
           #AC CREHAB-854|TC CREHAB-907: Use latest endpoing to retrieve values" do
    it ". The date of the most recent tracker entry is displayed below the Weight label. If date = 'today' display 'Today' if date = Today-1 display 'Yesterday', otherwise display the actual date (MM/DD/YYYY).  Most recent weight entry displayed as center aligned" do
      twoDaysAgo = getDateNthDaysFromNow(-2, "%m/%d/%Y")
      weight = "150.8"
      note = ""
      addWeight(twoDaysAgo, weight, note)
      Watir::Wait.until { @list_view.primaryHeader_element.text == "Weight" }
      expect(@list_view.getFocusedElement == @list_view.primaryHeader_element).to eq(true)
      expect(@health.weightDate_element.text).to eq(twoDaysAgo)
      expect(@health.weightAmount_element.text).to eq("150.8 lbs")

      yesterday = getDateNthDaysFromNow(-1, "%m/%d/%Y")
      weight = "161.2"
      note = ""
      addWeight(yesterday, weight, note)
      Watir::Wait.until { @list_view.primaryHeader_element.text == "Weight" }
      expect(@list_view.getFocusedElement == @list_view.primaryHeader_element).to eq(true)
      expect(@health.weightDate_element.text).to eq("Yesterday")
      expect(@health.weightAmount_element.text).to eq("161.2 lbs")

      today = getDateNthDaysFromNow(0, "%m/%d/%Y")
      weight = "160.3"
      note = ""
      addWeight(today, weight, note)
      Watir::Wait.until { @list_view.primaryHeader_element.text == "Weight" }
      expect(@list_view.getFocusedElement == @list_view.primaryHeader_element).to eq(true)
      expect(@health.weightDate_element.text).to eq("Today")
      expect(@health.weightAmount_element.text).to eq("160.3 lbs")
    end

  end

  context "#AC CREHAB-1186|TC CREHAB-1195: Phone View Option Button
           #AC CREAHB-1187|TC CREHAB-1195: Options control display for Phone user" do
    include_examples "Phone User: Options control"
  end

  context "#AC CREHAB-655|TC CREHAB-682: Sub Feature Default View for FitHeart Weight  " do
    include_examples "Date Filter Validation"
  end

  context "#AC CREHAB-656|TC CREHAB-682: Filter Action and Results Display for Weight" do
    it ". When a user selects the filter button, the system updates the table view to show records that match the filter options entered.
        . Given that results are found, entries are displayed in a table format, sorted in reverse chronological order by date and time entered; null values are shown as empty table cells unless otherwise noted" do
      removeCollection("vitalsigns", "medicalhistorydb")
      @site.refreshBrowser
      Watir::Wait.until { @list_view.primaryHeader_element.text == "Weight" }
      expect(@list_view.primaryHeader_element.text).to eq("Weight")

      @date_filter.startDate = @sixYearsAgo
      @date_filter.endDate = @today
      @date_filter.filter

      Watir::Wait.until { @list_view.primaryHeader_element.text == "Weight" }
      expect(@list_view.primaryHeader_element.text).to eq("Weight")

      today = getDateNthDaysFromNow(0, "%m/%d/%Y")
      weight = "158.2"
      note = ""
      addWeight(today, weight, note)

      threeDaysAgo = getDateNthDaysFromNow(-3, "%m/%d/%Y")
      weight = "157"
      note = ""
      addWeight(threeDaysAgo, weight, note)

      oneMonthAgo = getDateNthMonthsFromNow(-1, "%m/%d/%Y")
      weight = "157.2"
      note = ""
      addWeight(oneMonthAgo, weight, note)

      fourtyFiveDaysAgo =  getDateNthDaysFromNow(-45, "%m/%d/%Y")
      weight = "159"
      note = ""
      addWeight(fourtyFiveDaysAgo, weight, note)

      twoMonthAgo = getDateNthMonthsFromNow(-2, "%m/%d/%Y")
      weight = "159.4"
      note = ""
      addWeight(twoMonthAgo, weight, note)

      oneYearAgo = getDateNthYearsFromNow(-1, "%m/%d/%Y")
      weight = "162"
      note = ""
      addWeight(oneYearAgo, weight, note)

      twoYearsAgo = getDateNthYearsFromNow(-2, "%m/%d/%Y")
      weight = "167"
      note = ""
      addWeight(twoYearsAgo, weight, note)

      weight = "142.2"
      note = ""
      addWeight(@sixYearsAgo, weight, note)

      @date_filter.reset
      #reset to 3 months range
      Watir::Wait.until { @list_view.primaryHeader_element.text == "Weight" }
      expect(@list_view.primaryHeader_element.text).to eq("Weight")
      Watir::Wait.until {@list_view.tableList_elements.length == 5}
      expect(@list_view.tableList_elements.length).to eq(5)

      tableHeaderArr = ["Date Entered", "Weight (lbs)"]
      expect(@list_view.getTableHeaders()).to eq(tableHeaderArr)
      verifyDefaultTableList

      @date_filter.startDate = twoYearsAgo
      @date_filter.endDate = twoMonthAgo
      @date_filter.filter
      Watir::Wait.until { @list_view.primaryHeader_element.text == "Weight" }
      expect(@list_view.primaryHeader_element.text).to eq("Weight")
      Watir::Wait.until {@list_view.tableList_elements.length == 3}
      expect(@list_view.tableList_elements.length).to eq(3)

      tableHeaderArr = ["Date Entered", "Weight (lbs)"]
      expect(@list_view.getTableHeaders()).to eq(tableHeaderArr)
      expect(@list_view.getTextForRowCol(1, 1).include?(twoMonthAgo)).to eq(true)
      expect(isDateFormatValid?(@list_view.getTextForRowCol(1, 1), "%m/%d/%Y %I:%M %p"))
      expect(@list_view.getTextForRowCol(1, 2)).to eq("159.4")

      expect(@list_view.getTextForRowCol(2, 1).include?(oneYearAgo)).to eq(true)
      expect(isDateFormatValid?(@list_view.getTextForRowCol(2, 1), "%m/%d/%Y %I:%M %p"))
      expect(@list_view.getTextForRowCol(2, 2)).to eq("162.0")

      expect(@list_view.getTextForRowCol(3, 1).include?(twoYearsAgo)).to eq(true)
      expect(isDateFormatValid?(@list_view.getTextForRowCol(3, 1), "%m/%d/%Y %I:%M %p"))
      expect(@list_view.getTextForRowCol(3, 2)).to eq("167.0")

      @date_filter.startDate = @sixYearsAgo
      @date_filter.endDate = today
      @date_filter.filter
      Watir::Wait.until { @list_view.primaryHeader_element.text == "Weight" }
      expect(@list_view.primaryHeader_element.text).to eq("Weight")

      Watir::Wait.until {@list_view.tableList_elements.length == 8}
      expect(@list_view.tableList_elements.length).to eq(8)

      tableHeaderArr = ["Date Entered", "Weight (lbs)"]
      expect(@list_view.getTableHeaders()).to eq(tableHeaderArr)

      expect(@list_view.getTextForRowCol(6, 1).include?(oneYearAgo)).to eq(true)
      expect(isDateFormatValid?(@list_view.getTextForRowCol(6, 1), "%m/%d/%Y %I:%M %p"))
      expect(@list_view.getTextForRowCol(6, 2)).to eq("162.0")

      expect(@list_view.getTextForRowCol(7, 1).include?(twoYearsAgo)).to eq(true)
      expect(isDateFormatValid?(@list_view.getTextForRowCol(7, 1), "%m/%d/%Y %I:%M %p"))
      expect(@list_view.getTextForRowCol(7, 2)).to eq("167.0")

      expect(@list_view.getTextForRowCol(8, 1).include?(@sixYearsAgo)).to eq(true)
      expect(isDateFormatValid?(@list_view.getTextForRowCol(8, 1), "%m/%d/%Y %I:%M %p"))
      expect(@list_view.getTextForRowCol(8, 2)).to eq("142.2")

    end

    it ". After the filter button is selected, the focus moves to the returned results, first row, or the No results found message." do
      expect(@list_view.getFocusedElement == @list_view.firstTableRow_element).to eq(true)
      verifyNoResultsMessage(getDateNthDaysFromNow(-4, "%m/%d/%Y"), getDateNthDaysFromNow(-4, "%m/%d/%Y"))
    end

    context "#AC CREHAB-659|TC CREHAB-682: Reset Filter Functionality for FitHeart Weight(Filter and List View)" do
      it ".  When a user views a filter control, a Reset button is available located to the right of the Filter Button. (Alt text Reset)" do
        expect(@date_filter.reset_element.visible?).to eq(true)
        expect(@date_filter.reset_element.text).to eq("Reset")
      end

      it ". When the user selects the reset button, all filter field defaults are restored and the filter action is applied so that the table list view is updated per the default filter settings." do
        verifyReset(5)
      end
    end

    context "#AC CREHAB-688|TC CREHAB-691: Graph Button & Features for Weight " do
      include_examples "Graph/Table Button Validation"
    end

    context "#AC CREHAB-657|TC CREHAB-682: Navigation for Weight" do
      include_examples "Back button validation for Phone view"
    end

  end

  def verifyDefaultTableList
    expect(@list_view.getTextForRowCol(1, 1).include?(@today)).to eq(true)
    expect(isDateFormatValid?(@list_view.getTextForRowCol(1, 1), "%m/%d/%Y %I:%M %p"))
    expect(@list_view.getTextForRowCol(1, 2)).to eq("158.2")

    expect(@list_view.getTextForRowCol(2, 1).include?(getDateNthDaysFromNow(-3, "%m/%d/%Y"))).to eq(true)
    expect(isDateFormatValid?(@list_view.getTextForRowCol(2, 1), "%m/%d/%Y %I:%M %p"))
    expect(@list_view.getTextForRowCol(2, 2)).to eq("157.0")

    expect(@list_view.getTextForRowCol(3, 1).include?(getDateNthMonthsFromNow(-1, "%m/%d/%Y"))).to eq(true)
    expect(isDateFormatValid?(@list_view.getTextForRowCol(3, 1), "%m/%d/%Y %I:%M %p"))
    expect(@list_view.getTextForRowCol(3, 2)).to eq("157.2")

    expect(@list_view.getTextForRowCol(4, 1).include?(getDateNthDaysFromNow(-45, "%m/%d/%Y"))).to eq(true)
    expect(isDateFormatValid?(@list_view.getTextForRowCol(4, 1), "%m/%d/%Y %I:%M %p"))
    expect(@list_view.getTextForRowCol(4, 2)).to eq("159.0")

    expect(@list_view.getTextForRowCol(5, 1).include?(getDateNthMonthsFromNow(-2, "%m/%d/%Y"))).to eq(true)
    expect(isDateFormatValid?(@list_view.getTextForRowCol(5, 1), "%m/%d/%Y %I:%M %p"))
    expect(@list_view.getTextForRowCol(5, 2)).to eq("159.4")
  end

end
