#!/bin/env ruby
# encoding: utf-8
require 'rspec'
require 'watir-webdriver'

require_relative '../../lib/browser_container/site'
require_relative '../../spec/rspec_helper'
require_relative '../../lib/module/DateUtility'
require_relative '../../lib/module/MongoUtility'
require_relative '../health/health_helper'
require_relative '../shared_context'

describe '[Story #CREHAB-404: Weight - Edit/Delete]', :regression => true do
  include DateUtility
  include MongoUtility
  app_title = "VA FitHeart"

  before(:all) do
    removeAllDocuments()

    @driver = Watir::Browser.new
    @site = Site.new(BASE_URL, @driver)

    @login = @site.loginPage
    @eula = @site.eula
    @roa = @site.roa
    @landing = @site.landing
    @learn_more  = @site.learnMore
    @leftNav = @site.leftMenu
    @modal = @site.modal
    @health = @site.health
    @list_view = @site.list_view
    @detail = @site.health_detail
    @error = @site.error
    @calendar = @site.calendar_picker

    @title = "Weight"
    @today = getDateNthDaysAgo(0, "%m/%d/%Y")
  end

  after(:all) do
    @site.quitDriver
  end

  context "[Before] Log in and navigate to" do
    it ". Login and navigate to" do
      acceptEulaAndQuickLogin("zztest.patient01", "pass", "D123401")
      navigateToHealth(app_title)
      navigateTo(@title)
    end
  end

  context "#AC CREHAB-661|CREHAB-684: Edit an Entry for Fitheart Weight" do
    it ". When the user selects a Weight entry from the table view, the system displays the Weight record in the Detail screen in Edit mode: (header: Edit Weight Entry)" do
      date = @today
      weight = "150.8"
      note = "Need to lose weight"
      addWeight(date, weight, note)

      @list_view.tableList_elements[0].when_visible
      @list_view.tableList_elements[0].click

      verifyWeight(date, note, weight)
      @detail.note = ""

      @detail.save

      @list_view.contentLoaded_element.when_present(timeout=15)

      Watir::Wait.until {@list_view.secondaryHeader_element.text == "Health"}
      Watir::Wait.until {@health.weightAmount_element.exists?}
      Watir::Wait.until {@health.weightAmount_element.text == "150.8 lbs"}
      expect(@health.weightAmount_element.text).to eq("150.8 lbs")

      Watir::Wait.until {@list_view.primaryHeader_element.when_visible}
      expect(@list_view.primaryHeader_element.text).to eq("Weight")

      Watir::Wait.until { @list_view.tableList_elements.length == 1 }

      @list_view.tableList_elements[0].when_visible
      @list_view.tableList_elements[0].click

      note = ""
      verifyWeight(date, note, weight)

    end

    it ". Standard Edit functions are available (Save, Cancel, Delete)" do
      expect(@detail.save_element.visible?).to eq(true)
      expect(@detail.delete_element.visible?).to eq(true)
      expect(@detail.cancel_element.visible?).to eq(true)

      clickCancelAndFocusToHeader
    end
  end

  context "#AC CREHAB-663|TC CREHAB-684: Navigation for FitHeart Weight (Add/Edit/Delete)" do
    it ". On all form factors when the user views the Add or Edit Detail screen (secondary detail page), a < is shown (alt text: Return to Previous Screen; when clicked the user is returned to the Detail Screen for the group where the filter and table list view is presented." do
      @list_view.addButton_element.when_visible
      @list_view.addButton_element.click

      Watir::Wait.until { @detail.primaryHeader_element.text == "Add Weight Entry" }
      expect(@detail.primaryHeader_element.text).to eq("Add Weight Entry")

      @detail.date_element.when_visible
      @detail.date = @today
      @detail.weight = "332"
      @detail.note = "N/A"

      expect(@detail.backButtonIcon_element.exists?).to eq(true)
      expect(@detail.backButton_element.attribute("title")).to eq("Return to Previous Screen")

    end

    it ". If a user selects the < symbol and there are unsaved changes on the screen, the system will prompt the user to save their changes via a modal:" do
      verifyBackButtonConfirmationPrompt
    end

    it ". If the user tries to Navigate away from a detail record where a field has been updated without Saving, the user is prompted to confirm that they want to leave the record without saving via a modal:" do
      Watir::Wait.until {@list_view.tableList_elements[0].visible? }
      @list_view.tableList_elements[0].when_visible
      @list_view.tableList_elements[0].click

      Watir::Wait.until { @detail.primaryHeader_element.text == "Edit Weight Entry" }
      expect(@detail.primaryHeader_element.text).to eq("Edit Weight Entry")

      @detail.date_element.when_visible
      Watir::Wait.until {  @detail.date == @today.to_s }

      date = @detail.date
      weight = @detail.weight
      note = @detail.note

      @detail.date = getDateNthDaysFromNow(-2, "%m/%d/%Y")
      @detail.weight = "156"
      @detail.note = "Cut down carb"

      navigateAwayAndVerifyConfirmationPrompt
      navigateTo(@title)

      @list_view.tableList_elements[0].when_visible
      @list_view.tableList_elements[0].click

      @detail.primaryHeader_element.when_visible
      expect(@detail.primaryHeader_element.text).to eq("Edit Weight Entry")

      verifyWeight(date, note, weight)

      clickCancelAndFocusToHeader
    end

  end

  context "#AC CREHAB-662|TC CREHAB-684: Save, Cancel for Weight " do
    it ".  Save - When a user selects Save the system saves the record and the user is returned to the table view, with the focus on the header. Updates made to the record are visible in the table, if included in that view." do
      date = getDateNthDaysFromNow(-1, "%m/%d/%Y")
      weight = "160.2"
      note = ""

      addWeight(date, weight, note)
      Watir::Wait.until { @list_view.primaryHeader_element.text == "Weight" }
      expect(@list_view.getFocusedElement == @list_view.primaryHeader_element).to eq(true)

    end

    it ". Cancel - When a user selects Cancel, no updates are made to the record and the user is shown the table view with the focus placed on the header" do
      @list_view.tableList_elements[1].when_visible
      @list_view.tableList_elements[1].click

      date = getDateNthDaysFromNow(-1, "%m/%d/%Y")
      weight = "160.2"
      note = ""

      verifyWeight(date, note, weight)

      @detail.weight = "180"
      @detail.note = "ABCDEFG"

      clickCancelAndFocusToHeader

      @list_view.tableList_elements[1].when_visible
      @list_view.tableList_elements[1].click

      date = getDateNthDaysFromNow(-1, "%m/%d/%Y")
      weight = "160.2"
      note = ""

      verifyWeight(date, note, weight)

      clickCancelAndFocusToHeader

      #stage data
      date = getDateNthDaysFromNow(-4, "%m/%d/%Y")
      weight = "160"
      note = ""
      addWeight(date, weight, note)

      date = getDateNthDaysFromNow(-6, "%m/%d/%Y")
      weight = "158"
      note = ""
      addWeight(date, weight, note)

      date = getDateNthDaysFromNow(-12, "%m/%d/%Y")
      weight = "159.5"
      note = ""
      addWeight(date, weight, note)
    end
  end

  context "#AC CREHAB-662|TC CREHAB-684: Delete for Weight " do
    it_behaves_like "Delete Validation"
  end

  context "#AC CREHAB-662|TC CREHAB-684: Soft Delete for Weight " do
    it ". All deletes are soft deletes, meaning they are removed from the user’s screen but remain in the database for audit purposes." do
      expect(verifyDocumentsDeleted("vitalsigns", "medicalhistorydb", "EDIPI", "D123401", "deletedFlag")).to eq(true)
    end
  end

end
