#!/bin/env ruby
# encoding: utf-8
require 'rspec'
require 'watir-webdriver'

require_relative '../../lib/browser_container/site'
require_relative '../rspec_helper'
require_relative '../../lib/module/MongoUtility'

describe '[Story #CREHAB-168: Learn More - Fitness Accordion]', :regression => true do
  include MongoUtility

  app_title = "VA FitHeart"
  safety_first = "Safety First"
  be_active = "Be Active"
  cardiac_rehab = "Cardiac Rehab"
  how_to = "How-To"
  intensity = "Intensity"
  examples = "Examples"
  benefits = "Benefits"
  counting_steps = "Counting Steps"
  amount = "Amount"
  links = "Links"

  before(:all) do
    @driver = Watir::Browser.new
    @site = Site.new(BASE_URL, @driver)

    @login = @site.loginPage
    @eula = @site.eula
    @roa = @site.roa
    @landing = @site.landing
    @leftNav = @site.leftMenu
    @learnMore = @site.learnMore
    @fitness = @site.fitness
    @modal = @site.modal

  end

  after(:all) do
    @site.quitDriver
  end

  context "[Before] Log in and navigate to" do
    it ". Login and navigate to" do
      acceptEulaAndQuickLogin("zztest.patient01", "pass", "D123401")

      expect(@landing.appTitle).to eq(app_title)
      @landing.welcomeTitleForAuthenticatedUser_element.when_visible
      expect(@landing.welcomeTitleForAuthenticatedUser).to eq("Welcome to VA FitHeart")
    end
  end

  context "#AC #CREHAB-342|TC CREHAB-371: Fitness Option from App Options" do
    it ". When a user selects the Fitness option from the App Options list, the user will see the following items in the detail panel/column 3:" do
      @leftNav.leftMenu_element.when_visible
      @leftNav.leftMenu

      @leftNav.title_element.when_visible
      expect(@leftNav.learnMore_element.visible?).to eq(true)
      expect(@leftNav.learnMore_element.text).to eq("Learn More")

      @leftNav.learnMore
      @learnMore.primaryHeader_element.when_visible
      expect(@learnMore.primaryHeader_element.text).to eq("Learn More")

      expect(@learnMore.fitness_element.text).to eq("Fitness")

      @learnMore.fitness
      Watir::Wait.until {@fitness.primaryHeader_element.text == "Fitness"}

    end

    it ". Fitness Learn More sections displayed in an accordion control" do
      expect(@fitness.displayedInAccordionControl?).to eq(true)
    end

    it ". < icon is displayed in phone form factors, alt text: Return to Learn More options" do
      @site.resizeWindowToPhone

      expect(@fitness.backButton_element.visible?).to eq(true)
      expect(@fitness.backButtonIcon_element.visible?).to eq(true)
      expect(@fitness.backButton_element.attribute("title")).to eq("Return to Learn More options")
      @fitness.backButton

      @learnMore.secondaryHeader_element.when_visible
      expect(@learnMore.secondaryHeader_element.text).to eq("Learn More")

      expect(@learnMore.fitness_element.text).to eq("Fitness")
      expect(@learnMore.health_element.text).to eq("Health")
      expect(@learnMore.nutrition_element.text).to eq("Nutrition")
      expect(@learnMore.mood_element.text).to eq("Mood")

      @site.resizeWindowToDefault
    end

  end

  context "#AC CREHAB-343|TC CREHAB-371: Accordion Display for Fitness Accordion" do
    it ". Accordion display" do
      @learnMore.fitness
      Watir::Wait.until {@fitness.primaryHeader_element.text == "Fitness"}

      expect(@fitness.getAccordionLabelByHeading(safety_first)).to eq("Safety First")
      expect(@fitness.getAccordionLabelByHeading(be_active)).to eq("Be Active")
      expect(@fitness.getAccordionLabelByHeading(cardiac_rehab)).to eq("Cardiac Rehab")
      expect(@fitness.getAccordionLabelByHeading(how_to)).to eq("How-To")
      expect(@fitness.getAccordionLabelByHeading(intensity)).to eq("Intensity")
      expect(@fitness.getAccordionLabelByHeading(examples)).to eq("Examples")
      expect(@fitness.getAccordionLabelByHeading(benefits)).to eq("Benefits")
      expect(@fitness.getAccordionLabelByHeading(counting_steps)).to eq("Counting Steps")
      expect(@fitness.getAccordionLabelByHeading(amount)).to eq("Amount")
      expect(@fitness.getAccordionLabelByHeading(links)).to eq("Links")

    end
  end

  context "#AC CREHAB-344|TC CREHAB-371: Accordion Behavior for Fitness Accordion" do
    it ". Accordion sections can be opened and closed" do

     accordionArr = [safety_first, be_active, cardiac_rehab, how_to, intensity, examples, benefits, links]

     accordionSize = accordionArr.length - 1

     #all accordions closed
     for i in 0..accordionSize
       expect(@fitness.isAccordionExpanded(accordionArr[i])).to eq(false)
       expect(@fitness.isRightArrowIconDisplayed(accordionArr[i])).to eq(true)
     end

     for j in 0..accordionSize
       expect(@fitness.getAccordionAltText(accordionArr[j])).to eq("Expand\n" + accordionArr[j])
       @fitness.clickAccordionByHeading(accordionArr[j])
       expect(@fitness.isAccordionExpanded(accordionArr[j])).to eq(true)
       expect(@fitness.getAccordionAltText(accordionArr[j])).to eq("Collapse\n" + accordionArr[j])
       expect(@fitness.isDownArrowIconDisplayed(accordionArr[j])).to eq(true)

       for k in 0..accordionSize
         if (k != j)
           expect(@fitness.isAccordionExpanded(accordionArr[k])).to eq(false)
           expect(@fitness.isRightArrowIconDisplayed(accordionArr[k])).to eq(true)
         end
       end

       @fitness.clickAccordionByHeading(accordionArr[j])
       expect(@fitness.isAccordionExpanded(accordionArr[j])).to eq(false)
       expect(@fitness.getAccordionAltText(accordionArr[j])).to eq("Expand\n" + accordionArr[j])
     end

    end
  end

  context "#AC CREHAB-345|TC CREHAB-371: Accordion section displays for Fitness Accordion" do
    it ". When a user expands the Safety First accordion" do
      @fitness.clickAccordionByHeading(safety_first)
      expect(@fitness.getAccordionTextByHeading(safety_first)).to eq("Exercise is safe and beneficial for almost everyone. If you have questions about whether it is safe for you to do a specific type of exercise, you can ask your health provider.")
    end

    it ". When a user expands the Be Active Accordion" do
      @fitness.clickAccordionByHeading(be_active)
      expect(@fitness.getAccordionTextByHeading(be_active)).to eq("Ideas for physical activity:\nTake a walk in your neighborhood\nTake a walk at a local shopping center\nJoin a gym\nRide a bike\nUse an arm or foot peddler at home\nGo to a community pool\nDance\nMarch in place while watching TV")
    end

    it ". When a user expands the Cardiac Rehab accordion" do
      @fitness.clickAccordionByHeading(cardiac_rehab)
      expect(@fitness.getAccordionTextByHeading(cardiac_rehab)).to eq("Cardiac rehabilitation is a program for people with heart conditions. It focuses on fitness, nutrition, weight, blood pressure, cholesterol, stress, diabetes, and smoking. This mobile app can help with many of these!\nLearn more about cardiac rehab topics.")
      @fitness.cardiacRehabTopicLink_element.when_visible
      @fitness.cardiacRehabTopicLink

      @site.switchWindowToLast
      Watir::Wait.until {@site.getCurrentURL == Fitness::CARDIAC_REHAB_TOPICS_URL}
      expect(@site.getCurrentURL).to eq(Fitness::CARDIAC_REHAB_TOPICS_URL)
      @site.closeBrowser

      @site.switchWindowToFirst

    end

    it ". When user expands the How-To accordion," do
      @fitness.clickAccordionByHeading(how_to)
      expect(@fitness.getAccordionTextByHeading(how_to)).to eq("Here are some ways to help you achieve the recommended 150 minutes/week of physical activity\nAdd physical activity to your daily schedule\nGet family and friends to be active with you\nTake the stairs\nPark your car far away and walk\nUse a fitness video or video game (like WiiFit)\nEnroll in cardiac rehab\nJoin a gym")
    end

    it ". When a user expands the Intensity accordion," do
      @fitness.clickAccordionByHeading(intensity)
      expect(@fitness.getAccordionTextByHeading(intensity)).to eq("Rate the intensity of your activity!\nVery Easy\n(sedentary, like watching TV)\nEasy\n(breathing unnoticeable, can carry a conversation)\nSomewhat Easy\n(breathing noticeable, can hold a conversation)\nSomewhat Hard\n(labored breathing, can hold a short conversation)\nHard\n(can only speak a few words)\nVery Hard\n(can’t talk)\nExert yourself enough so that you can still talk (but not sing) while you are being active.")
    end

    it ". When a user expands the Examples accordion," do
      @fitness.clickAccordionByHeading(examples)
      expect(@fitness.getAccordionTextByHeading(examples)).to eq("At least moderate intensity activity is best for your heart.\nLight\nEasy Walk\nStanding\nSitting\nModerate\nBrisk walk\nLight cycling or swimming\nDancing\nGardening\nHousehold chores\nHard\nRunning\nFast cycling or swimming\nVigorous Sports")
    end

    it ". When a user expands the Benefits accordion," do
      @fitness.clickAccordionByHeading(benefits)
      expect(@fitness.getAccordionTextByHeading(benefits)).to eq("Being active can improve your health in a lot of ways:\nMaintain a healthy weight\nPrevent progression of heart disease\nImprove stamina\nImprove strength\nImprove mood\nIncrease energy\nHave fun!")
    end

    it ". When a user expands the Counting Steps accordion," do
      @fitness.clickAccordionByHeading(counting_steps)
      expect(@fitness.getAccordionTextByHeading(counting_steps)).to eq("Many people like using a pedometer to measure their physical activity. A pedometer counts the number of steps that you take in a day. In an active day, you should try to get to 10,000 steps!")
    end

    it ". When a user expands the Amount accordion" do
      @fitness.clickAccordionByHeading(amount)
      expect(@fitness.getAccordionTextByHeading(amount)).to eq("Exercise ≈ intensity x duration x frequency.\nBy increasing intensity, duration, or frequency, you can increase the amount of exercise.")
    end

    it ". When a user expands the Links accordion," do
      @fitness.clickAccordionByHeading(links)
      expect(@fitness.getAccordionTextByHeading(links)).to eq("American Heart Association (AHA)\nAmerican College of Cardiology (ACC)\nAmerican Association of Cardiovascular and Pulmonary Rehabilitation (AACVPR)\nPreventive Cardiovascular Nurses Association")

      @fitness.fitnessAHALink_element.when_visible
      @fitness.fitnessAHALink

      @site.switchWindowToLast
      Watir::Wait.until {@site.getCurrentURL == Fitness::AHA_URL}
      expect(@site.getCurrentURL).to eq(Fitness::AHA_URL)
      @site.closeBrowser

      @site.switchWindowToFirst

      @fitness.fitnessACCLink_element.when_visible
      @fitness.fitnessACCLink

      @site.switchWindowToLast
      Watir::Wait.until {@site.getCurrentURL == Fitness::ACC_URL}
      expect(@site.getCurrentURL).to eq(Fitness::ACC_URL)
      @site.closeBrowser

      @fitness.fitnessAACVPRLink_element.when_visible
      @fitness.fitnessAACVPRLink

      @site.switchWindowToLast
      Watir::Wait.until {@site.getCurrentURL == Fitness::AACVPR_URL}
      expect(@site.getCurrentURL).to eq(Fitness::AACVPR_URL)
      @site.closeBrowser

      @fitness.fitnessPCNALink_element.when_visible
      @fitness.fitnessPCNALink

      @site.switchWindowToLast
      Watir::Wait.until {@site.getCurrentURL == Fitness::PREVENTIVE_CARDIO_NURSE_ASSO_URL}
      expect(@site.getCurrentURL).to eq(Fitness::PREVENTIVE_CARDIO_NURSE_ASSO_URL)
      @site.closeBrowser

    end

  end

end
