#!/bin/env ruby
# encoding: utf-8
require 'rspec'
require 'watir-webdriver'

require_relative '../../lib/browser_container/site'
require_relative '../../spec/rspec_helper'
require_relative '../../spec/health/health_helper'
require_relative '../../lib/module/DateUtility'
require_relative '../../lib/module/MongoUtility'
require_relative '../../lib/module/DBUtility'
require_relative '../shared_context'
require_relative 'notification_helper'

describe '[Story #CREHAB-393 Notification Filter and List Views (all subfeatures)]
          [Story #CREHAB-394: View/Delete Notifications]', :regression => true do
  include DateUtility
  include MongoUtility
  include DBUtility

  app_title = "VA FitHeart"

  before(:all) do
    removeAllDocuments()
    resetQuartzNotificationDb()
    resetCollection("notifications", "notificationsdb", "notifications")
    updateNotificationsDate()

    @driver = Watir::Browser.new
    @site = Site.new(BASE_URL, @driver)

    @login = @site.loginPage
    @eula = @site.eula
    @roa = @site.roa
    @landing = @site.landing
    @leftNav = @site.leftMenu
    @rightNav = @site.rightMenu
    @modal = @site.modal
    @error = @site.error
    @calendar = @site.calendar_picker
    @date_filter = @site.date_filter
    @notification_detail = @site.notification_detail
    @notifications = @site.notifications
    @list_view = @site.notification_listview
    @notification_setting = @site.notification_setting
    @view_notification = @site.view_notification

    @today = getDateNthDaysFromNow(0, "%m/%d/%Y")
    @yesterday = getDateNthDaysFromNow(-1, "%m/%d/%Y")
  end

  after(:all) do
    @site.quitDriver
  end

  context "[Before] Log in and navigate to" do
    it ". Login and navigate" do
      acceptEulaAndQuickLogin("zztest.patient01", "pass", "D123401")
      expect(@landing.appTitle).to eq(app_title)
      @landing.welcomeTitleForAuthenticatedUser_element.when_visible
      expect(@landing.welcomeTitleForAuthenticatedUser).to eq("Welcome to VA FitHeart")
      navigateToNotifications()
    end
  end

  context "#AC CREHAB-1051|TC CREHAB-1075: Notifications left panel navigation group options" do
    it ".  When the user opens the Notifications feature in the left panel navigation, the system displays All Notifications, VA FitHeart App notifes, VA Messages" do
      expect(@notifications.allNotifications_element.text.strip).to eq("All Notifications")
      expect(@notifications.vaFitHeartAppNotices_element.text.strip).to eq("VA FitHeart App Notices")
      expect(@notifications.vaHealthMsgs_element.text.strip).to eq("VA Health Messages")

      expect(@notifications.secondaryOptions_elements[0].text.strip).to eq("All Notifications")
      expect(@notifications.secondaryOptions_elements[1].text.strip).to eq("VA FitHeart App Notices")
      expect(@notifications.secondaryOptions_elements[2].text.strip).to eq("VA Health Messages")
    end
  end

  subFeaturesParam = [
      { :ACNumber => 'CREHAB-1052', :featureName => 'All Notifications', :defaultSource => 'All Notifications', :sourceEnabled => true},
      { :ACNumber => 'CREHAB-1053', :featureName => 'VA FitHeart App Notices', :defaultSource => 'VA FitHeart App Notices', :sourceEnabled => false},
      { :ACNumber => 'CREHAB-1054', :featureName => 'VA Health Messages', :defaultSource => 'VA Health Messages', :sourceEnabled => false}
  ]

  subFeaturesParam.each do |feature|

    context "#AC " + feature[:ACNumber]+ "|TC CREHAB-1075: " + feature[:featureName] do
      it ". When the user opens the " + feature[:featureName] + " sub-feature, the filter and notification list is displayed in the Detail Screen (Header 'All Notifications')" do
        navigateToNotificationOption(feature[:featureName])
        expect(@notifications.primaryHeader_element.text).to eq(feature[:featureName])
      end

      it ". Filter defaults to an expanded view. Collapse Filter option and Expands filter option" do
        @date_filter.filterSection_element.when_visible
        expect(@date_filter.isFilterExpanded).to eq(true)
        expect(@date_filter.filterSection_element.attribute("title")).to eq("Collapse filter options")

        #Users can collapse the filter list by clicking on the Collapse filter icon. When collapsed, the filter icon alt text reads “Expand filter options.”
        @date_filter.filterSection
        expect(@date_filter.isFilterExpanded).to eq(false)
        expect(@date_filter.filterSection_element.attribute("title")).to eq("Expand filter options")

        @date_filter.filterSection
      end

      it ". Date Range - Start Date and End Date are required fields" do
        expect(@date_filter.requiredFieldLabel).to eq("* required field")
        expect(@date_filter.dateRangeLabel_element.text).to eq("Date Range:")

        expect(@date_filter.startDateLabel_element.text).to eq("* Start Date:\nMM/DD/YYYY")
        expect(@date_filter.startDate_element.attribute("required")).to eq("true")
        expect(@date_filter.startDate_element.attribute("placeholder")).to eq("MM/DD/YYYY")

        expect(@date_filter.endDateLabel_element.text).to eq("* End Date:\nMM/DD/YYYY")
        expect(@date_filter.endDate_element.attribute("required")).to eq("true")
        expect(@date_filter.endDate_element.attribute("placeholder")).to eq("MM/DD/YYYY")
      end

      it ". Date Range - future dates not allowed.  date selection range is constrained to the last year.  Default date range is the last 30 days" do
        thirtyDaysAgo = getDateNthDaysFromNow(-30, "%m/%d/%Y")
        tomorrow = getDateNthDaysFromNow(1, "%m/%d/%Y")
        oneYearAndtwoDaysAgo =  getDateNthDaysFromNow(-367, "%m/%d/%Y")

        expect(@date_filter.startDate).to eq(thirtyDaysAgo)
        expect(@date_filter.endDate).to eq(@today)

        @date_filter.startDate =  tomorrow
        @date_filter.endDate = tomorrow
        @date_filter.filter
        oneYearAgo = getDateNthYearsFromNow(-1, "%m/%d/%Y")

        errorMessages = [ "End Date must be between " + oneYearAgo + " and " + @today + ".",  "Start Date must be between " + oneYearAgo+ " and " + @today + "."]
        verifyErrorMessage(errorMessages)

        @date_filter.startDate = oneYearAndtwoDaysAgo
        @date_filter.endDate = @today
        @date_filter.filter

        errorMessages = ["Start Date must be between " + oneYearAgo+ " and " + @today + "."]
        verifyErrorMessage(errorMessages)

        @date_filter.startDate = tomorrow
        @date_filter.endDate = @today
        @date_filter.filter

        errorMessages = ["Start Date must occur before End Date."]
        verifyErrorMessage(errorMessages)

        @date_filter.startDatePicker
        expect(@date_filter.isStartDateCalendarDisplayed).to eq(true)
        @calendar.pick(oneYearAndtwoDaysAgo)
        expect(@calendar.isDayEnabled(getDay(oneYearAndtwoDaysAgo))).to eq(false)
        #close calendar
        @date_filter.endDate_element.click

        @date_filter.startDate = oneYearAgo

        @date_filter.endDatePicker
        expect(@date_filter.isEndDateCalendarDisplayed).to eq(true)

        @calendar.pick(tomorrow)
        expect(@calendar.isDayEnabled(getDay(tomorrow))).to eq(false)

        @date_filter.endDate = tomorrow

      end

      it ". Source: single select, select list, values: All Notifications, VA FitHeart App Notices, VA Health Messages; default to All Notifications)" do
        expect(@list_view.sourceLabel_element.text).to eq("Source:")
        expect(@list_view.sourceSelect).to eq(feature[:defaultSource])
        expect(@list_view.sourceSelect_options).to eq(["All Notifications", "VA FitHeart App Notices", "VA Health Messages"])
      end

      it ". Status: radio button selection: Unread, Read, All (default)" do
        expect(@list_view.statusLabel_element.text).to eq("Status:")
        expect(@list_view.status_selected?).to eq("All")
        expect(@list_view.status_values).to eq(["Unread", "Read", "All"])
      end

      it ". If no records are found when a filter is applied the message No results found. " do
        @date_filter.startDate = getDateNthDaysFromNow(-362, "%m/%d/%Y")
        @date_filter.endDate = getDateNthDaysFromNow(-362, "%m/%d/%Y")
        @date_filter.filter

        Watir::Wait.until {@list_view.noResultsMessage_element.exists? }
        expect(@list_view.noResultsMessage_element.text).to eq("No results found.")
        expect(@list_view.getFocusedElement == @list_view.noResultsMessage_element).to eq(true)
      end

      it ". Notifications are displayed in a table format with the following columns (Notifications are sorted in reverse chronological order by date/time received )" do
        @date_filter.startDate = getDateNthYearsFromNow(-1, "%m/%d/%Y")
        @date_filter.endDate = getDateNthDaysFromNow(0, "%m/%d/%Y")


        if(feature[:sourceEnabled] == true)
          @list_view.sourceSelect = "All Notifications"
          @date_filter.filter
          Watir::Wait.until {@list_view.tableList_elements.length == 6}

          tableHeaderArr = ["Status", "Notification", "Source", "Date"]
          expect(@list_view.getTableHeaders()).to eq(tableHeaderArr)

          expect(@list_view.isUnreadIconDisplayed(1)).to eq(true)
          expect(@list_view.tableList_elements[0].attribute("class")).to include("bold-text")
          expect(@list_view.getTextForRowCol(1, 2)).to eq("Update your Blood Pressure & Pulse information now.")
          expect(@list_view.getTextForRowCol(1, 3)).to eq("VA FitHeart - Health")
          expect(@list_view.getTextForRowCol(1, 4)).to include(@today)
          expect(@list_view.getTextForRowCol(1, 4)).to include("America/New_York")

          expect(@list_view.isUnreadIconDisplayed(2)).to eq(true)
          expect(@list_view.tableList_elements[1].attribute("class")).to include("bold-text")
          expect(@list_view.getTextForRowCol(2, 2)).to eq("Update your Glucose information now.")
          expect(@list_view.getTextForRowCol(2, 3)).to eq("VA-Health")
          expect(@list_view.getTextForRowCol(2, 4)).to include(getDateNthDaysFromNow(-10, "%m/%d/%Y"))
          expect(@list_view.getTextForRowCol(2, 4)).to include("America/New_York")

          expect(@list_view.isUnreadIconDisplayed(3)).to eq(true)
          expect(@list_view.tableList_elements[2].attribute("class")).to include("bold-text")
          expect(@list_view.getTextForRowCol(3, 2)).to eq("Update your Cholesterol information now.")
          expect(@list_view.getTextForRowCol(3, 3)).to eq("VA FitHeart - Health")
          expect(@list_view.getTextForRowCol(3, 4)).to include(getDateNthDaysFromNow(-78, "%m/%d/%Y"))
          expect(@list_view.getTextForRowCol(3, 4)).to include("America/New_York")

          expect(@list_view.isUnreadIconDisplayed(4)).to eq(true)
          expect(@list_view.tableList_elements[3].attribute("class")).to include("bold-text")
          expect(@list_view.getTextForRowCol(4, 2)).to eq("Update your Mood information now.")
          expect(@list_view.getTextForRowCol(4, 3)).to eq("VA-Health")
          expect(@list_view.getTextForRowCol(4, 4)).to include(getDateNthDaysFromNow(-100, "%m/%d/%Y"))
          expect(@list_view.getTextForRowCol(4, 4)).to include("America/New_York")

          expect(@list_view.isUnreadIconDisplayed(5)).to eq(true)
          expect(@list_view.tableList_elements[4].attribute("class")).to include("bold-text")
          expect(@list_view.getTextForRowCol(5, 2)).to eq("Update your Weight information now.")
          expect(@list_view.getTextForRowCol(5, 3)).to eq("VA FitHeart - Health")
          expect(@list_view.getTextForRowCol(5, 4)).to include(getDateNthDaysFromNow(-300, "%m/%d/%Y"))
          expect(@list_view.getTextForRowCol(5, 4)).to include("America/New_York")

          expect(@list_view.isUnreadIconDisplayed(6)).to eq(true)
          expect(@list_view.tableList_elements[5].attribute("class")).to include("bold-text")
          expect(@list_view.getTextForRowCol(6, 2)).to eq("Update your Cholesterol information now.")
          expect(@list_view.getTextForRowCol(6, 3)).to eq("VA FitHeart - Health")
          expect(@list_view.getTextForRowCol(6, 4)).to include(getDateNthDaysFromNow(-346, "%m/%d/%Y"))
          expect(@list_view.getTextForRowCol(6, 4)).to include("America/New_York")

        end

        if ( feature[:sourceEnabled] == true || feature[:featureName] == "VA FitHeart App Notices")
          if (feature[:sourceEnabled])
            @list_view.sourceSelect = "VA FitHeart App Notices"
          end
          @date_filter.filter

          Watir::Wait.until {@list_view.tableList_elements.length == 4}

          tableHeaderArr = ["Status", "Notification", "Source", "Date"]

          if (feature[:sourceEnabled] == false)
            tableHeaderArr = ["Status", "Notification", "Date"]
          end

          expect(@list_view.getTableHeaders()).to eq(tableHeaderArr)

          expect(@list_view.isUnreadIconDisplayed(1)).to eq(true)
          expect(@list_view.getTextForRowCol(1, 2)).to eq("Update your Blood Pressure & Pulse information now.")
          if (feature[:sourceEnabled])
            expect(@list_view.getTextForRowCol(1, 3)).to eq("VA FitHeart - Health")
            expect(@list_view.getTextForRowCol(1, 4)).to include(@today)
            expect(@list_view.getTextForRowCol(1, 4)).to include("America/New_York")
          else
            expect(@list_view.getTextForRowCol(1, 3)).to include(@today)
            expect(@list_view.getTextForRowCol(1, 3)).to include("America/New_York")
          end

          expect(@list_view.isUnreadIconDisplayed(2)).to eq(true)
          expect(@list_view.getTextForRowCol(2, 2)).to eq("Update your Cholesterol information now.")
          if (feature[:sourceEnabled])
            expect(@list_view.getTextForRowCol(2, 3)).to eq("VA FitHeart - Health")
            expect(@list_view.getTextForRowCol(2, 4)).to include(getDateNthDaysFromNow(-78, "%m/%d/%Y"))
            expect(@list_view.getTextForRowCol(2, 4)).to include("America/New_York")
          else
            expect(@list_view.getTextForRowCol(2, 3)).to include(getDateNthDaysFromNow(-78, "%m/%d/%Y"))
            expect(@list_view.getTextForRowCol(2, 3)).to include("America/New_York")
          end

          expect(@list_view.isUnreadIconDisplayed(3)).to eq(true)
          expect(@list_view.getTextForRowCol(3, 2)).to eq("Update your Weight information now.")
          if (feature[:sourceEnabled])
            expect(@list_view.getTextForRowCol(3, 3)).to eq("VA FitHeart - Health")
            expect(@list_view.getTextForRowCol(3, 4)).to include(getDateNthDaysFromNow(-300, "%m/%d/%Y"))
            expect(@list_view.getTextForRowCol(3, 4)).to include("America/New_York")
          else
            expect(@list_view.getTextForRowCol(3, 3)).to include(getDateNthDaysFromNow(-300, "%m/%d/%Y"))
            expect(@list_view.getTextForRowCol(3, 3)).to include("America/New_York")
          end

          expect(@list_view.isUnreadIconDisplayed(4)).to eq(true)
          expect(@list_view.getTextForRowCol(4, 2)).to eq("Update your Cholesterol information now.")
          if (feature[:sourceEnabled])
            expect(@list_view.getTextForRowCol(4, 3)).to eq("VA FitHeart - Health")
            expect(@list_view.getTextForRowCol(4, 4)).to include(getDateNthDaysFromNow(-346, "%m/%d/%Y"))
            expect(@list_view.getTextForRowCol(4, 4)).to include("America/New_York")
          else
            expect(@list_view.getTextForRowCol(4, 3)).to include(getDateNthDaysFromNow(-346, "%m/%d/%Y"))
            expect(@list_view.getTextForRowCol(4, 3)).to include("America/New_York")
          end
        end

        if ( feature[:sourceEnabled] == true || feature[:featureName] == "VA Health Messages")
          if (feature[:sourceEnabled])
            @list_view.sourceSelect = "VA Health Messages"
          end

          @date_filter.filter

          Watir::Wait.until {@list_view.tableList_elements.length == 2}

          tableHeaderArr = ["Status", "Notification", "Source", "Date"]
          if (feature[:sourceEnabled] == false)
            tableHeaderArr = ["Status", "Notification", "Date"]
          end

          expect(@list_view.getTableHeaders()).to eq(tableHeaderArr)

          if (feature[:sourceEnabled] == true)
            expect(@list_view.isUnreadIconDisplayed(1)).to eq(true)
          end
          expect(@list_view.getTextForRowCol(1, 2)).to eq("Update your Glucose information now.")

          if (feature[:sourceEnabled])
            expect(@list_view.getTextForRowCol(1, 3)).to eq("VA-Health")
            expect(@list_view.getTextForRowCol(1, 4)).to include(getDateNthDaysFromNow(-10, "%m/%d/%Y"))
            expect(@list_view.getTextForRowCol(1, 4)).to include("America/New_York")
          else
            expect(@list_view.getTextForRowCol(1, 3)).to include(getDateNthDaysFromNow(-10, "%m/%d/%Y"))
            expect(@list_view.getTextForRowCol(1, 3)).to include("America/New_York")
          end

          expect(@list_view.isUnreadIconDisplayed(2)).to eq(true)
          expect(@list_view.getTextForRowCol(2, 2)).to eq("Update your Mood information now.")
          if (feature[:sourceEnabled])
            expect(@list_view.getTextForRowCol(2, 3)).to eq("VA-Health")
            expect(@list_view.getTextForRowCol(2, 4)).to include(getDateNthDaysFromNow(-100, "%m/%d/%Y"))
            expect(@list_view.getTextForRowCol(2, 4)).to include("America/New_York")
          else
            expect(@list_view.getTextForRowCol(2, 3)).to include(getDateNthDaysFromNow(-100, "%m/%d/%Y"))
            expect(@list_view.getTextForRowCol(2, 3)).to include("America/New_York")
          end

        end

      end

      it ".  Once a notification is read, display the notification in regular font text and change the indicator to the open circle icon." do
        @list_view.tableList_elements[0].click
        @view_notification.primaryHeader_element.when_visible
        expect(@view_notification.primaryHeader_element.text).to eq("View Notification")
        @view_notification.cancel_element.when_visible
        @view_notification.cancel

        Watir::Wait.until {@list_view.primaryHeader_element.text == feature[:featureName]}

        expect(@list_view.tableList_elements[0].attribute("class")).not_to include("bold-text")
        expect(@list_view.isUnreadIconDisplayed(1)).to eq(false)

        expect(@list_view.tableList_elements[1].attribute("class")).to include("bold-text")
        expect(@list_view.isUnreadIconDisplayed(2)).to eq(true)

        @list_view.select_status("Unread")
        @date_filter.filter

        @list_view.contentLoaded_element.when_present(timeout=15)

        count = @list_view.tableList_elements.length
        for i in 1..count
          expect(@list_view.tableList_elements[i - 1].attribute("class")).to include("bold-text")
          expect(@list_view.isUnreadIconDisplayed(i)).to eq(true)
        end

        @list_view.select_status("Read")
        @date_filter.filter
        @list_view.contentLoaded_element.when_present(timeout=15)

        count = @list_view.tableList_elements.length
        for j in 1..count
          expect(@list_view.tableList_elements[j - 1].attribute("class")).not_to include("bold-text")
          #expect(@list_view.isUnreadIconDisplayed(i)).to eq(false)
        end

      end

      it ". Reset" do
        @date_filter.reset_element
        @date_filter.reset
        @list_view.contentLoaded_element.when_present(timeout=15)

        expect(@date_filter.startDate).to eq(getDateNthDaysFromNow(-30, "%m/%d/%Y"))
        expect(@date_filter.endDate).to eq(@today)
        expect(@list_view.status_selected?).to eq("All")

      end

      if (feature[:featureName] == "VA Health Messages")
        it ". Opt out of VA Health Messages " do
          #opt out
          updateNotificationPreference("test@test.com", "No", "(-05:00) America/New_York (Eastern)")

          Watir::Wait.until {@list_view.noResultsMessage_element.exists? }
          expect(@list_view.noResultsMessage_element.text).to eq("You are not currently receiving VA Health Messages. Access your Notification settings using the icon shown in the header of the Notification options menu.\nYou are not currently receiving VA Health Messages. Access your Notification settings using the gear icon shown in the header of the Notification options menu.")
        end

        it ".Navigation" do
          @site.resizeWindowToPhone
          expect(@list_view.backButton_element.visible?).to eq(true)
          expect(@list_view.backButton_element.attribute("Return to Notifications options"))
          @list_view.backButton_element.click
          Watir::Wait.until { @notifications.allNotifications_element.visible?}
          expect(@notifications.allNotifications_element.visible?).to eq(true)
          @site.resizeWindowToDefault

        end
      end

    end

  end #end of subFeatureParam

  context "#AC CREHAB-1055|TC CREHAB-1081: Viewing a Notification Record for Fitheart" do
    it " 	Header: View Notification, source, date and time, timezone, message" do
      updateNotificationPreference("test@test.com", "Yes", "(-05:00) America/New_York (Eastern)")
      navigateToNotificationOption("All Notifications")

      @date_filter.startDate = getDateNthYearsFromNow(-1, "%m/%d/%Y")
      @date_filter.endDate = @today
      @date_filter.filter

      Watir::Wait.until {@list_view.tableList_elements.length == 6}

      @list_view.tableList_elements[4].when_visible

      expect(@list_view.isUnreadIconDisplayed(5)).to eq(true)
      expect(@list_view.tableList_elements[4].attribute("class")).to include("bold-text")
      expect(@list_view.getTextForRowCol(5, 2)).to eq("Update your Weight information now.")
      expect(@list_view.getTextForRowCol(5, 3)).to eq("VA FitHeart - Health")
      expect(@list_view.getTextForRowCol(5, 4)).to include(getDateNthDaysFromNow(-300, "%m/%d/%Y"))
      expect(@list_view.getTextForRowCol(5, 4)).to include("America/New_York")

      @list_view.tableList_elements[4].click

      Watir::Wait.until { @view_notification.primaryHeader_element.text == "View Notification"}
      expect(@view_notification.primaryHeader_element.text).to eq("View Notification")

      @view_notification.source_element.when_visible

      expect(@view_notification.source_element.text).to eq("VA FitHeart - Health")
      expect(@view_notification.dateAndTime_element.text).to include(getDateNthDaysFromNow(-300, "%m/%d/%Y"))
      expect(@view_notification.timeZone_element.text).to eq("America/New_York")
      expect(@view_notification.message_element.text).to eq("Update your Weight information now.")

    end

  end

  context "#AC CREHAB-1056|TC CREHAB-1081: Read/Unread Status for FitHeart" do
    it ". Read/Unread Status for FitHeart" do
      @view_notification.cancel_element.when_visible
      @view_notification.cancel

      Watir::Wait.until {@list_view.primaryHeader_element.text == "All Notifications"}

      @list_view.tableList_elements[4].when_visible
      expect(@list_view.isUnreadIconDisplayed(5)).to eq(false)
      expect(@list_view.tableList_elements[4].attribute("class")).not_to include("bold-text")

    end
  end

  context "#AC CREHAB-1057|TC CREHAB-1081: Navigation for FitHeart" do
    it ". On all form factors the < icon is shown in the upper left corner of the detail screen, alt text 'Return to Previous Screen'; selecting the icon returns the user to the Notification Table/Filter screen." do
      @list_view.tableList_elements[3].click

      Watir::Wait.until { @view_notification.primaryHeader_element.text == "View Notification"}
      expect(@view_notification.primaryHeader_element.text).to eq("View Notification")

      @view_notification.timeZone_element.when_visible

      @site.resizeWindowToPhone
      expect(@view_notification.backButton_element.visible?).to eq(true)
      expect(@view_notification.backButton_element.attribute("Return to Previous Screen"))
      @view_notification.backButton_element.click

      @list_view.contentLoaded_element.when_present(timeout=10)
      Watir::Wait.until {@list_view.primaryHeader_element.text == "All Notifications"}

      @site.resizeWindowToDefault
    end

  end

  context "#AC CREHAB-1093|TC CREHAB-1081: Delete Notification " do
    it ". The user can invoke a delete function for an individual Notification; user is prompted to confirm the deletion per the standard My VA Health pattern." do
      count = @list_view.tableList_elements.length

      for i in 0..(count -1)
        @list_view.tableList_elements[0].click

        @view_notification.primaryHeader_element.when_visible
        expect( @view_notification.primaryHeader_element.text).to eq("View Notification")

        @view_notification.delete_element.when_visible
        @view_notification.delete

        @modal.modalTitle_element.when_visible
        expect(@modal.modalTitle_element.text).to eq("Confirmation")
        expect(@modal.section1_element.text).to eq("Are you sure you want to remove this entry? Select YES to remove the entry or NO to return to the entry screen.")
        @modal.no_element.when_visible
        expect(@modal.no_element.text).to eq("No")
        @modal.no
        @modal.no_element.when_not_visible

        @view_notification.delete_element.when_visible
        expect(@view_notification.delete_element.focused?).to eq(true)
        @view_notification.delete

        @modal.modalTitle_element.when_visible
        expect(@modal.modalTitle_element.text).to eq("Confirmation")
        expect(@modal.section1_element.text).to eq("Are you sure you want to remove this entry? Select YES to remove the entry or NO to return to the entry screen.")
        @modal.yes_element.when_visible
        expect(@modal.yes_element.text).to eq("Yes")
        @modal.yes
        @modal.yes_element.when_not_visible

        @list_view.primaryHeader_element.when_visible
        expect(@list_view.primaryHeader_element.text).to eq("All Notifications")
        Watir::Wait.until {@list_view.tableList_elements.length ==  (count - (i + 1))}
      end
    end

    it ".Soft Delete " do
      expect(getFieldByObjId("notifications", "notificationsdb", "54ed0bd00364cdb769b6f56b", "deletedFlag").to_s).to eq("true")
      expect(getFieldByObjId("notifications", "notificationsdb", "54ed0bd00364cdb769b6f56c", "deletedFlag").to_s).to eq("true")
      expect(getFieldByObjId("notifications", "notificationsdb", "54ed0bd00364cdb769b6f56d", "deletedFlag").to_s).to eq("true")
      expect(getFieldByObjId("notifications", "notificationsdb", "54ed0cfc0364cdb769b6f56f", "deletedFlag").to_s).to eq("true")
      expect(getFieldByObjId("notifications", "notificationsdb", "54ed0cfc0364cdb769b6f570", "deletedFlag").to_s).to eq("true")
     #not deleted
      expect(getFieldByObjId("notifications", "notificationsdb", "54ed0e280364cdb769b6f571", "deletedFlag").to_s).to eq("")
    end
  end


  def updateNotificationsDate()
    updateDocument("notifications", "notificationsdb", "54ed0bd00364cdb769b6f56b", "effectiveDisplayDate",  Time.now )
    updateDocument("notifications", "notificationsdb", "54ed0bd00364cdb769b6f56c", "effectiveDisplayDate",  Time.now - 10.days)
    updateDocument("notifications", "notificationsdb", "54ed0bd00364cdb769b6f56d", "effectiveDisplayDate",  Time.now - 78.days)
    updateDocument("notifications", "notificationsdb", "54ed0bd00364cdb769b6f56e", "effectiveDisplayDate",  Time.now - 100.days)
    updateDocument("notifications", "notificationsdb", "54ed0cfc0364cdb769b6f56f", "effectiveDisplayDate",  Time.now - 300.days)
    updateDocument("notifications", "notificationsdb", "54ed0cfc0364cdb769b6f570", "effectiveDisplayDate",  Time.now - 346.days)
    updateDocument("notifications", "notificationsdb", "54ed0e280364cdb769b6f571", "effectiveDisplayDate",  Time.now - 369.days)

    updateDocument("notifications", "notificationsdb", "54ed0bd00364cdb769b6f56b", "createDate",  Time.now)
    updateDocument("notifications", "notificationsdb", "54ed0bd00364cdb769b6f56c", "createDate",  Time.now - 10.days)
    updateDocument("notifications", "notificationsdb", "54ed0bd00364cdb769b6f56d", "createDate",  Time.now - 78.days)
    updateDocument("notifications", "notificationsdb", "54ed0bd00364cdb769b6f56e", "createDate",  Time.now - 100.days)
    updateDocument("notifications", "notificationsdb", "54ed0cfc0364cdb769b6f56f", "createDate",  Time.now - 300.days)
    updateDocument("notifications", "notificationsdb", "54ed0cfc0364cdb769b6f570", "createDate",  Time.now - 346.days)
    updateDocument("notifications", "notificationsdb", "54ed0e280364cdb769b6f571", "createDate",  Time.now - 369.days)

    updateDocument("notifications", "notificationsdb", "54ed0bd00364cdb769b6f56c", "type",  "VA-Health")
    updateDocument("notifications", "notificationsdb", "54ed0bd00364cdb769b6f56c", "source", "VA-Health")
    updateDocument("notifications", "notificationsdb", "54ed0bd00364cdb769b6f56e", "type",  "VA-Health")
    updateDocument("notifications", "notificationsdb", "54ed0bd00364cdb769b6f56e", "source",  "VA-Health")
  end

end