require 'rspec'
require 'watir-webdriver'

require_relative '../../lib/browser_container/site'
require_relative '../../spec/rspec_helper'
require_relative '../../spec/health/health_helper'
require_relative '../../lib/module/DateUtility'
require_relative '../../lib/module/MongoUtility'
require_relative '../shared_context'
require_relative 'notification_helper'

describe '[Story #CREHAB-392: Notification Settings - First Time User]
          [Story #CREHAB-395: Notification Settings]
          [Story #CREHAB-722: Notifications: Add error handling for preferred email address deletions]
          [Story #CREHAB-1213: Notification: Fitness Activity - Default to On]', :acceptance => true do
  include DateUtility
  include MongoUtility
  app_title = "VA FitHeart"

  before(:all) do
    removeAllDocuments()

    @driver = Watir::Browser.new
    @site = Site.new(BASE_URL, @driver)

    @login = @site.loginPage
    @eula = @site.eula
    @roa = @site.roa
    @landing = @site.landing
    @leftNav = @site.leftMenu
    @rightNav = @site.rightMenu
    @modal = @site.modal
    @error = @site.error
    @notification_detail = @site.notification_detail
    @notifications = @site.notifications
    @notification_setting = @site.notification_setting
    @user_guide = @site.userGuide
    @list_view = @site.fitness_listview

    @today = getDateNthDaysFromNow(0, "%m/%d/%Y")
    @yesterday = getDateNthDaysFromNow(-1, "%m/%d/%Y")

  end

  after(:all) do
    @site.quitDriver
  end

  context "[Before] Log in and navigate to" do
    it ". Login and navigate" do
      login_EndsOnNotificationPreference("zztest.patient01", "pass", "D123401")
      expect(@landing.appTitle).to eq(app_title)
      expect(@notification_setting.welcomeTitle_element.text).to eq("Welcome zztest patient01")
    end
  end

  context "#AC CREHAB-1048|TC CREHAB-1079: Notification Settings display: Homepage feature access " do
    it ". App options menu icon is not shown, User menu shows About, Help, Launchpad, Logout" do
      expect(@leftNav.leftMenu_element.visible?).to eq(false)

      @rightNav.rightMenu_element.when_visible
      @rightNav.rightMenu

      @rightNav.menuHeader_element.when_visible
      @rightNav.about_element.when_visible
      expect(@rightNav.about_element.text).to eq("About")

      expect(@rightNav.getNthUserMenu(1)).to eq("About")
      expect(@rightNav.getNthUserMenu(2)).to eq("Help")

      expect(@rightNav.getNthNavMenu(1)).to eq("Launchpad")
      expect(@rightNav.getNthNavMenu(2)).to eq("Logout")

      @rightNav.rightMenu
    end
  end

  context "#AC CREHAB-1046|TC CREHAB-1079: Notification display for first-time user" do
    it ". User without saved notification preference logs into app - welcome, instructional text, learn more link, email instructional text" do
      expect(@notification_setting.welcomeTitle_element.text).to eq("Welcome zztest patient01")

      expect(@notification_setting.instructionalText1_element.text.strip).to eq("Notifications include reminder notices you create in this app, as well as Health Messages that may be sent to you by the Veteran's Administration. While you may adjust your notification delivery preferences for individual reminders, you must elect to receive Health Messages from the Veteran's Administration.")
      expect(@notification_setting.instructionalText2_element.text.strip).to eq("As a first time user, please save your Notification Settings before using other app features. Note that notification settings apply to all VA health applications.")
      expect(@notification_setting.notificationTitle_element.text).to eq("Notification Settings")

      expect(@notification_setting.emailInstructionalTextForFirstTime_element.text.strip).to eq("Your preferred email for notifications applies to all VA Health apps. You may modify this email under the Settings feature shown under Notifications.")

      #default value
      expect(@notification_setting.timeZoneSelect).to eq("Select")
      expect(@notification_setting.receiveMsgOption_selected?).to eq("Yes")
      expect(@notification_setting.email).to eq("")

    end
  end

  context "#AC CREHAB-1046|TC CREHAB-1079: Notification display for first-time user" do
    it_behaves_like "Notification Fields Validation"
  end

  context "#AC CREHAB-1047|TC CREHAB-1047: Notification Settings for Save Function" do
    it ". Save Function" do
      @notification_setting.save2_element.when_visible
      expect(@notification_setting.save2_element.text).to eq("Save")
      @notification_setting.save2

      @landing.contentLoaded_element.when_present(timeout=15)

      #click No for Weekly Activity Goals if modal displays
      if (@modal.no_element.exists?)
        @modal.no
        @modal.no_element.when_not_visible
      end

    end

    it ". The user may access features located in the left panel navigation. The landing page - logged in user is displayed" do
      expect(@landing.appTitle).to eq(app_title)
      @landing.welcomeTitleForAuthenticatedUser_element.when_visible
      expect(@landing.welcomeTitleForAuthenticatedUser).to eq("Welcome to VA FitHeart")
      expect(@landing.footerLabel_element.text).to eq("Logged in as patient01, zztest")

      expect(@leftNav.leftMenu_element.visible?).to eq(true)
    end

  end

  context "#AC CREHAB-1225|TC CREHAB-1226: Default Fitness Activity Notification" do
    it ". When a first time user saves their initial notification preferences, the Fitness Activity Notification is set as follows: Status On, Start Date = today, Daily, 9:00 AM Delivery in App and email" do
      @leftNav.leftMenu_element.when_visible
      @leftNav.leftMenu

      @leftNav.fitness_activity_element.when_visible
      @leftNav.fitness_activity

      @list_view.contentLoaded_element.when_present(timeout=15)
      expect(@list_view.primaryHeader_element.text).to eq("Fitness Activity")
      Watir::Wait.until {@notification_detail.notificationButton_element.text == "Notification (On)"}
      expect(@notification_detail.notificationButton_element.text).to eq("Notification (On)")
      expect(@list_view.primaryHeader_element.text).to eq("Fitness Activity")

      @notification_detail.notificationButton
      @notification_detail.contentLoaded_element.when_present(timeout=10)
      Watir::Wait.until {@notification_detail.notificationStatusOnDetail_selected? == "On"}
      expect(@notification_detail.primaryHeader_element.text).to eq("Fitness Activity Notification")

      expect(@notification_detail.notificationStatusOnDetail_selected?).to eq("On")
      expect(@notification_detail.startDateOnDetail_element.attribute("disabled")).to eq("true")
      expect(@notification_detail.startDateOnDetail).to eq(@today)
      expect(@notification_detail.notifyMe).to eq("Daily")

      expect(@notification_detail.getNotificationTimeArray("Daily")).to eq(["09:00 AM"])
      expect(@notification_detail.isNotificationDeliveryChecked("In-App")).to eq(true)
      expect(@notification_detail.isNotificationDeliveryChecked("Email")).to eq(true)

      @notification_detail.cancel_element.when_visible
      @notification_detail.cancel

      @list_view.contentLoaded_element.when_present(timeout=15)

    end
  end

  context "#AC CREHAB-1049|TC CREHAB-1074: Notification Setting modal for FitHeart " do
    it ". When the user accesses the Settings feature (gear icon) from within the Notification feature, the system displays a modal settings screen:" do
      @leftNav.leftMenu_element.when_visible
      @leftNav.leftMenu

      @leftNav.notifications_element.when_visible
      @leftNav.notifications

      @notifications.contentLoaded_element.when_present(timeout=10)

      @notifications.primaryHeader_element.when_visible
      expect(@notifications.secondaryHeader_element.text).to eq("Notifications")
      expect(@notifications.primaryHeader_element.text).to eq("Notifications")

      @notifications.notificationSettingButton_element.when_visible
      @notifications.notificationSettingButton

      @notifications.contentLoaded_element.when_present(timeout=10)

      verifyNotificationFields("Notification Settings", "test@test.com", "No", "(-05:00) America/New_York (Eastern)")

    end

    it ". Instructional Text and Learn More link" do
      expect(@notification_setting.section1_element.text.strip).to eq("Manage your Notification preferences by adjusting the settings options below. Note that notification settings apply to all VA health applications.")
      expect(@notification_setting.emailInstructionalText_element.text.strip).to eq("Your Preferred email for notifications applies to all VA Health apps.")

    end
  end

  context "#AC CREHAB-1049|TC CREHAB-1074: Notification Setting modal for FitHeart " do
     include_examples "Notification Fields Validation"
  end

  context "#AC CREHAB-1050|TC CREHAB-1074: Notification Settings Save and Cancel Functionality" do
    it ". When the user selects Save, users are returned to notification gear button " do
      @notification_setting.email = "test2@test.com"
      @notification_setting.select_receiveMsgOption("Yes")
      @notification_setting.timeZoneSelect = "(-06:00) America/Chicago (Central)"

      @notification_setting.save2_element.when_visible
      @notification_setting.save2

      @notifications.contentLoaded_element.when_present(timeout=15)
      expect(@notifications.notificationSettingButton_element.focused?).to eq(true)

    end
    it ".  If the user selects Cancel, the last saved preference settings are retained and the user is returned to the Notification List." do
      @notifications.notificationSettingButton_element.when_visible
      @notifications.notificationSettingButton

      @notifications.contentLoaded_element.when_present(timeout=10)

      verifyNotificationFields("Notification Settings", "test2@test.com", "Yes", "(-06:00) America/Chicago (Central)")

      @notification_setting.email = "test3@test.com"
      @notification_setting.select_receiveMsgOption("No")
      @notification_setting.timeZoneSelect = "(-06:00) America/Chicago (Central)"

      @notification_setting.cancel2_element.when_visible
      @notification_setting.cancel2
      @notifications.contentLoaded_element.when_present(timeout=10)

      @notifications.notificationSettingButton_element.when_visible
      @notifications.notificationSettingButton

      @notifications.contentLoaded_element.when_present(timeout=10)

      verifyNotificationFields("Notification Settings", "test2@test.com", "Yes", "(-06:00) America/Chicago (Central)")
    end
  end

  context "#AC CREHAB-1068|TC CREHAB-1083: Warning Message for email deletions" do
    it ". When the user removes their email address and tries to save the change, the user is shown a warning message" do
      @notification_setting.email = ""

      @notification_setting.save2_element.when_visible
      @notification_setting.save2

      @modal.modalTitle_element.when_visible

      expect(@modal.modalTitle_element.focused?).to eq(true)
      expect(@modal.modalTitle_element.text).to eq("Warning")
      expect(@modal.section1_element.text).to eq("An email address is required to receive email notifications. If you do not provide an email address any future email notifications will be cancelled.\nIf you wish to continue saving, click SAVE, otherwise, select CANCEL to update your Notification Settings.")
    end

    it ". When the user clicks cancel, they return to the the settings modal with focus on header and email populated" do
      @modal.cancel_element.when_visible
      @modal.cancel
      Watir::Wait.until {@notification_setting.modalTitle_element.exists?}
      Watir::Wait.until {@notification_setting.modalTitle_element.text == "Notification Settings"}

      verifyNotificationFields("Notification Settings", "test2@test.com", "Yes", "(-06:00) America/Chicago (Central)")
      Watir::Wait.until {@notification_setting.modalTitle_element.focused?}
      expect(@notification_setting.modalTitle_element.focused?).to eq(true)
    end

    it ". When the user clicks save, both modals close with empty email saved" do
      @notification_setting.email = ""

      @notification_setting.save2_element.when_visible
      @notification_setting.save2

      @modal.modalTitle_element.when_visible
      expect(@modal.modalTitle_element.text).to eq("Warning")

      @modal.save_element.when_visible
      @modal.save

      @notifications.notificationSettingButton_element.when_visible
      Watir::Wait.until {@notifications.notificationSettingButton_element.focused?}
      expect(@notifications.notificationSettingButton_element.focused?).to eq(true)
      @notifications.notificationSettingButton

      verifyNotificationFields("Notification Settings", "", "Yes", "(-06:00) America/Chicago (Central)")
    end
  end

  def verifyNotificationFields(modalTitle, email, optIn, timeZone)
    @notification_setting.modalTitle_element.when_visible
    expect(@notification_setting.modalTitle_element.text).to eq(modalTitle)
    expect(@notification_setting.email).to eq(email)
    expect(@notification_setting.receiveMsgOption_selected?).to eq(optIn)
    expect(@notification_setting.timeZoneSelect).to eq(timeZone)
  end
end